/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.api;

/**
 * The Transaction interface allows operations to be performed against the transaction in the target Transaction object.
 * A Transaction object is associated to a target Session.
 */
public interface Transaction {

    /**
     * Start the transaction represented by this Transaction object.
     * 
     * @throws org.objectweb.jalisto.se.exception.TransactionException
     *          Thrown if the transaction is already active.
     */
    void begin();

    /**
     * Complete the transaction represented by this Transaction object.
     * 
     * @throws org.objectweb.jalisto.se.exception.TransactionException
     *          Thrown if the transaction is inactive.
     */
    void commit();

    /**
     * Rollback the transaction represented by this Transaction object.
     * 
     * @throws org.objectweb.jalisto.se.exception.TransactionException
     *          Thrown if the transaction is inactive.
     */
    void rollback();

    /**
     * Set the transaction represented by this Transaction object to optimistic mode.
     * 
     * @throws org.objectweb.jalisto.se.exception.TransactionException
     *          Thrown if the transaction is already active.
     */
    void setOptimistic();

    /**
     * Set the transaction represented by this Transaction object to pessimistic mode.
     * 
     * @throws org.objectweb.jalisto.se.exception.TransactionException
     *          Thrown if the transaction is already active.
     */
    void setPessimistic();

    /**
     * Tell if we are in an active transaction.
     * 
     * @return true if the transaction represented by this Transaction object is active; otherwise false.
     */
    boolean isActive();

    /**
     * Get the transaction's associated Jalisto ressource.
     * 
     * @return Session object associated with this Transaction.
     */
    Session getSession();
}
