/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl;

import java.io.Serializable;

/**
 * Identity class of all persistent instances in Jalisto.
 * It's a wrapper around a long value.
 */
public class LogicalOid implements Serializable, Comparable {
    public LogicalOid(long floidAsLong) {
        this.floidAsLong = floidAsLong;
    }

    public LogicalOid(String floidAsString) {
        int index = floidAsString.indexOf(".");
        int clid = Integer.parseInt(floidAsString.substring(0, index));
        long instanceNumber = Long.parseLong(floidAsString.substring(index + 1));
        this.floidAsLong = clid * classMulti + instanceNumber;
    }

    public LogicalOid getClone() {
        return new LogicalOid(floidAsLong);
    }

    public long getFloidAsLong() {
        return floidAsLong;
    }

    public String getFloidAsString() {
        return Long.toString(floidAsLong);
    }

    public short getClid() {
        return (short) (floidAsLong / classMulti);
    }

    public long getInstanceNumber() {
        return floidAsLong % classMulti;
    }

    public int compareTo(Object o) {
        LogicalOid candidate = (LogicalOid) o;

        if (this.floidAsLong < candidate.floidAsLong) {
            return -1;
        } else if (this.floidAsLong > candidate.floidAsLong) {
            return 1;
        } else {
            return 0;
        }
    }

    public boolean equals(Object obj) {
        try {
            return ((LogicalOid) obj).floidAsLong == this.floidAsLong;
        } catch (ClassCastException e) {
        }
        return false;
    }

    public int hashCode() {
        return (int) (floidAsLong ^ (floidAsLong >>> 32));
    }

    public String toString() {
        return getClid() + "." + getInstanceNumber();
    }

    private long floidAsLong;

    static final long serialVersionUID = -7589377069041161459L;
    public static final long classMulti = 1000000000000000L;
}
