/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl;

import org.objectweb.jalisto.se.exception.TransactionException;
import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.api.Transaction;

public class TransactionImpl implements Transaction {

    public TransactionImpl(Session session) {
        this.session = session;
    }

    public void begin() {
        checkIsNotActive();
        session.getInternalSession().begin();
        isActive = true;
    }

    public void commit() {
        checkIsActive();
        session.getInternalSession().commit();
        isActive = false;
    }

    public boolean isActive() {
        return isActive;
    }

    public Session getSession() {
        return session;
    }

    public void rollback() {
        checkIsActive();
        session.getInternalSession().rollback();
        isActive = false;
    }

    public void setOptimistic() {
        checkIsNotActive();
        session.getInternalSession().setOptimistic();
    }

    public void setPessimistic() {
        checkIsNotActive();
        session.getInternalSession().setPessimistic();
    }

    private void checkIsActive() {
        if (!isActive) {
            throw new TransactionException("Transaction is not active");
        }
    }

    private void checkIsNotActive() {
        if (isActive) {
            throw new TransactionException("Transaction is active");
        }
    }

    private Session session;
    protected boolean isActive = false;
}
