/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.cache;

import java.lang.ref.ReferenceQueue;
import java.lang.ref.WeakReference;

public class WeakCacheCell extends WeakReference {

    private WeakCacheCell(Object id, Object object, ReferenceQueue referenceQueue) {
        super(object, referenceQueue);
        this.id = id;
    }

    public void setId(Object id) {
        this.id = id;
    }

    public Object getId() {
        return id;
    }

    public void setPrecedingCell(WeakCacheCell cell) {
        precedingCell = cell;
    }

    public WeakCacheCell getPrecedingCell() {
        return precedingCell;
    }

    public void setNextCell(WeakCacheCell cell) {
        nextCell = cell;
    }

    public WeakCacheCell getNextCell() {
        return nextCell;
    }

    public void delink() {
        if (nextCell != null) {
            nextCell.setPrecedingCell(precedingCell);
        }
        if (precedingCell != null) {
            precedingCell.setNextCell(nextCell);
        }
        nextCell = null;
        precedingCell = null;
    }

    public int hashCode() {
        return id.hashCode();
    }

    public static WeakCacheCell newInstance(Object id, Object object, ReferenceQueue queue) {
        return new WeakCacheCell(id, object, queue);
    }

    public String toString() {
        return "WCC(" + String.valueOf(get()) + ")";
    }


    private WeakCacheCell precedingCell;
    private WeakCacheCell nextCell;
    private Object id;
}