/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.cache;

import org.objectweb.jalisto.se.api.cache.JalistoCache;
import org.objectweb.jalisto.se.impl.trace.Trace;

import java.io.PrintStream;
import java.lang.ref.ReferenceQueue;
import java.util.*;

public class WeakReferenceCache implements JalistoCache {

    public WeakReferenceCache() {
        table = new HashMap();
        queue = new ReferenceQueue();
    }

    public void init(int size, String name, double clearPourcent) {
        this.maxSize = size;
        this.name = name;
        this.cacheClearPourcent = clearPourcent;
    }

    public void setTrace(Trace trace) {
        this.trace = trace;
    }

    public boolean isEmpty() {
        return table.isEmpty();
    }

    public void clear() {
        trace.println(Trace.CACHE, "{0} : clear()", name);
        table.clear();
        firstCell = null;
        lastCell = null;
    }

    public boolean containsKey(Object key) {
        return table.containsKey(key);
    }

    public boolean containsValue(Object value) {
        throw new UnsupportedOperationException("not used in this implementation");
    }

    public Set entrySet() {
        throw new UnsupportedOperationException("not used in this implementation");
    }

    public Object get(Object key) {
        trace.println(Trace.CACHE, "{0} : get({1})", name, key);
        Object result = table.get(key);
        if (result != null) {
            return ((WeakCacheCell) result).get();
        }
        return null;
    }

    public Set keySet() {
        return table.keySet();
    }

    public Object put(Object key, Object value) {
        trace.println(Trace.CACHE, "{0} : put({1}, {2})", name, key, value);
        remove(key);
        if (table.size() >= maxSize) {
            makeSpace();
        }
        WeakCacheCell availableCacheCell = WeakCacheCell.newInstance(key, value, queue);
        if (lastCell != null) {
            lastCell.setNextCell(availableCacheCell);
            availableCacheCell.setPrecedingCell(lastCell);
            lastCell = availableCacheCell;
        } else {
            lastCell = availableCacheCell;
            firstCell = availableCacheCell;
        }
        table.put(key, availableCacheCell);
        return null;
    }

    public void putAll(Map map) {
        Iterator keys = map.keySet().iterator();
        while (keys.hasNext()) {
            Object key = keys.next();
            put(key, map.get(key));
        }
    }

    public Object remove(Object key) {
        trace.println(Trace.CACHE, "{0} : remove({1})", name, key);
        if (containsKey(key)) {
            synchronized (table) {
                WeakCacheCell cell = (WeakCacheCell) table.remove(key);
                if (cell.equals(firstCell)) {
                    firstCell = cell.getNextCell();
                }
                if (cell.equals(lastCell)) {
                    lastCell = cell.getPrecedingCell();
                }
                cell.delink();
                if (table.isEmpty()) {
                    lastCell = null;
                    firstCell = null;
                }
                return cell.get();
            }
        }
        return null;
    }

    public int size() {
        return table.size();
    }

    public synchronized void setMaxSize(int size) {
        maxSize = size;
        WeakCacheCell myFirst = firstCell;
        synchronized (table) {
            while (table.size() > maxSize) {
                WeakCacheCell mySecond = myFirst.getNextCell();
                table.remove(myFirst.getId());
                myFirst.delink();
                myFirst = mySecond;
            }
            if (table.isEmpty()) {
                lastCell = null;
                firstCell = null;
            }
            firstCell = myFirst;
        }
    }

    public int getMaxSize() {
        return maxSize;
    }

    public Collection values() {
        throw new UnsupportedOperationException("not used in this implementation");
    }


    private void makeSpace() {
        int nbrToRemove = (new Double(maxSize * cacheClearPourcent)).intValue() + 1;
        trace.println(Trace.CACHE, "{0} : makeSpace() : clear {1} objects", name, new Integer(nbrToRemove));


        int cleaned = cleanQueue();
        if (cleaned < nbrToRemove) {
            nbrToRemove = nbrToRemove - cleaned;
            WeakCacheCell myFirst = firstCell;
            synchronized (table) {
                for (int i = 0; i < nbrToRemove; i++) {
                    WeakCacheCell mySecond = myFirst.getNextCell();
                    if (mySecond == null) {
                        throw new NullPointerException();
                    }
                    table.remove(myFirst.getId());
                    myFirst.delink();
                    myFirst = mySecond;
                }
            }
            firstCell = myFirst;
        }
    }

    private int cleanQueue() {
        trace.println(Trace.CACHE, "{0} : cleanQueue()", name);
        WeakCacheCell ref = (WeakCacheCell) queue.poll();
        int initSize = table.size();
        if (initSize == 0) {
            return 0;
        }
        if (ref != null) {
            int i = 0;
            synchronized (table) {
                while (ref != null) {
                    table.remove(ref.getId());
                    if (firstCell == ref) {
                        firstCell = ref.getNextCell();
                    }
                    if (lastCell == ref) {
                        lastCell = ref.getPrecedingCell();
                    }
                    ref.delink();
                    i++;
                    ref = (WeakCacheCell) queue.poll();
                }
            }
            return i;
        }
        return 0;
    }

    public void print(PrintStream out) {
        out.println(table.toString());
    }

    public String toString() {
        return table.toString();
    }


    private WeakCacheCell firstCell;
    private WeakCacheCell lastCell;
    private HashMap table;
    private ReferenceQueue queue;
    private String name;
    private double cacheClearPourcent;
    private int maxSize;
    private Trace trace;
}
