/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.client;

import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.exception.SchemaException;
import org.objectweb.jalisto.se.api.*;
import org.objectweb.jalisto.se.api.remote.ClientCommunicationAgent;
import org.objectweb.jalisto.se.impl.meta.ClassDescriptionImpl;
import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.api.query.FieldDescription;
import org.objectweb.jalisto.se.api.internal.SessionInternal;
import org.objectweb.jalisto.se.api.internal.InternalMetaRepository;

import java.util.Collection;
import java.util.Map;

public class InternalMetaRepositoryClientImpl implements InternalMetaRepository {

    public InternalMetaRepositoryClientImpl(JalistoProperties properties, ClientCommunicationAgent connexion) {
        this.connexion = connexion;
    }

    public void init() {
        if (classMetas == null) {
            this.classMetas = connexion.getClassMetas();
        }
        if (classTable == null) {
            this.classTable = connexion.getClassTable();
        }
    }

    public void checkNoActive(String message) {
        throw new UnsupportedOperationException();
    }

    public void closeAllSessions() {
        throw new UnsupportedOperationException();
    }

    public void checkNoSessions(String message) {
        throw new UnsupportedOperationException();
    }

    public void defineClass(Session session, ClassDescription classMetaDescription) {
        String className = classMetaDescription.getClassName();
        Object clid = connexion.getClidFromClassName(className);
        classMetas.put(clid, classMetaDescription);
        classTable.put(className, clid);
    }

    public ClassDescription getMetaData(String className) {
        return (ClassDescription) classMetas.get(classTable.get(className));
    }

    public void removeClass(Session session, String className) {
        if (!classTable.containsKey(className)) {
            throw new SchemaException("Class " + className + " is not define in base");
        }
        Object clid = classTable.remove(className);
        classMetas.remove(clid);
    }

    public void addField(Session session, String className, FieldDescription fieldDescription) {
        throw new UnsupportedOperationException();
    }

    public String[] getFieldNames(String className) {
        return getMetaData(className).getFieldNames();
    }

    public void renameField(Session session, String className, String oldFieldName, String newFieldName) {
        throw new UnsupportedOperationException();
    }

    public void removeField(Session session, String className, String fieldName) {
        throw new UnsupportedOperationException();
    }

    public int getIndex(String className, String fieldName) {
        throw new UnsupportedOperationException();
    }

    public Collection getAllClassNames() {
        return classTable.keySet();
    }

    public void addSession(SessionInternal session) {
        throw new UnsupportedOperationException();
    }

    public void removeSession(SessionInternal session) {
        throw new UnsupportedOperationException();
    }

    public String getClassNameFromClid(Object clid) {
        ClassDescription meta = (ClassDescription) classMetas.get(clid);
        return meta.getClassName();
    }

    public Object getClidFromClassName(String fullClassName) {
        Object clid = classTable.get(fullClassName);
        if (clid == null) {
            throw new SchemaException("Class " + fullClassName + " is not define in base");
        }
        return clid;
    }

    public ClassDescription getPersistentClassMetaDescription(Object clid) {
        return (ClassDescriptionImpl) classMetas.get(clid);
    }


    private ClientCommunicationAgent connexion;
    private Map classMetas;         // clid -> classMeta
    private Map classTable;         // classname -> clid
}
