/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.client;

import org.objectweb.jalisto.se.api.JalistoProperties;
import org.objectweb.jalisto.se.api.internal.Constants;
import org.objectweb.jalisto.se.exception.JalistoException;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.URL;
import java.util.*;

public class JalistoPropertiesClientImpl implements JalistoProperties {

    private static final String NAME = "jalisto";
    private static final String SERVER_PROPERTIES_PATH = "jalisto.properties";
    private static final String CACHE_IMPLEMENTATION = "org.objectweb.jalisto.se.impl.cache.GenericCache";
    private static final String OBJECT_CACHE_SIZE = "500";
    private static final String CACHE_CLEAR_POURCENT = "20";
    private static final String ALLOWS_SPECIAL_FUNCTIONNALITIES = "no";
    public static final String COMMUNICATION_FACTORY_CLASS =
            "org.objectweb.jalisto.se.impl.remote.socket.CommunicationFactorySocketImpl";
    public static final String HOST = "localhost";
    public static final String PORT = "7777";
    private static final String QUERY_MANAGER_CLASS = Constants.QUERY_MANAGER;
    private static final String TRACE = "";


    public static final String SERVER_PROPERTIES_PATH_KEY = "serverPropertiesFile";
    public static final String HOST_KEY = "host";
    public static final String PORT_KEY = "port";


    private File parent;
    private Properties prop;
    private String propertiesPath;
    private JalistoProperties remoteProperties;


    static final long serialVersionUID = -7589477064771161459L;

    public JalistoPropertiesClientImpl(String databasePropertiesFilePath) {
        this.propertiesPath = databasePropertiesFilePath;

        try {
            prop = new Properties();
            prop.put(NAME_KEY, NAME);
            prop.put(SERVER_PROPERTIES_PATH_KEY, SERVER_PROPERTIES_PATH);
            prop.put(CACHE_IMPLEMENTATION_KEY, CACHE_IMPLEMENTATION);
            prop.put(OBJECT_CACHE_SIZE_KEY, OBJECT_CACHE_SIZE);
            prop.put(CACHE_CLEAR_POURCENT_KEY, CACHE_CLEAR_POURCENT);
            prop.put(ALLOWS_SPECIAL_FUNCTIONNALITIES_KEY, ALLOWS_SPECIAL_FUNCTIONNALITIES);
            prop.put(COMMUNICATION_FACTORY_CLASS_KEY, COMMUNICATION_FACTORY_CLASS);
            prop.put(HOST_KEY, HOST);
            prop.put(PORT_KEY, PORT);
            prop.put(QUERY_MANAGER_CLASS_KEY, QUERY_MANAGER_CLASS);
            prop.put(TRACE_KEY, TRACE);

            boolean isLoaded = false;
            if (!databasePropertiesFilePath.equalsIgnoreCase("")) {
                File propfile = new File(databasePropertiesFilePath);
                if (propfile.exists() && (!propfile.isDirectory())) {
                    parent = propfile.getAbsoluteFile().getParentFile();
                    FileInputStream fis = new FileInputStream(propfile);
                    prop.load(fis);
                    isLoaded = true;
                }
                if (!isLoaded) {
                    ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
                    InputStream testIS = classLoader.getResourceAsStream(databasePropertiesFilePath);
                    URL url = classLoader.getResource(databasePropertiesFilePath);
                    propfile = new File(url.getFile());
                    parent = propfile.getAbsoluteFile().getParentFile();
                    prop.load(testIS);
                    isLoaded = true;
                }
            } else {
                parent = new File(System.getProperty("user.dir"));
                System.out.println("Use default properties values with execution path : " + parent.getAbsolutePath());
                isLoaded = true;
            }

            if (!isLoaded) {
                parent = new File(System.getProperty("user.dir"));
                System.out.println("Exception during reading of Jalisto properties use only default values");
                System.out.println("Work in system user.dir directory : " + String.valueOf(parent));
            }

            truncateProperties();
        } catch (Exception e) {
            throw new JalistoException(
                    "Error during initialisation of JalistoProperties, using " + databasePropertiesFilePath, e);
        }
    }

    public void setRemoteProperties(JalistoProperties remoteProperties) {
        this.remoteProperties = remoteProperties;
    }

    public String getName() {
        return getProperty(NAME_KEY);
    }

    public String getServerPropertiesPath() {
        return getProperty(SERVER_PROPERTIES_PATH_KEY);
    }

    public int getObjectCacheSize() {
        return Integer.parseInt(getProperty(OBJECT_CACHE_SIZE_KEY));
    }

    public double getCacheClearPourcent() {
        return Double.parseDouble("0." + getProperty(CACHE_CLEAR_POURCENT_KEY));
    }

    public String getCommunicationFactoryClassName() {
        return getProperty(COMMUNICATION_FACTORY_CLASS_KEY);
    }

    public String getHost() {
        return getProperty(HOST_KEY);
    }

    public int getPort() {
        return Integer.parseInt(getProperty(PORT_KEY));
    }

    public Collection getKeys() {
        return prop.keySet();
    }

    public String getProperty(String key) {
        return prop.getProperty(key);
    }

    public void setProperty(String key, String property) {
        prop.setProperty(key, property);
    }

    public Collection getTraceModuleNames() {
        return getStrings(TRACE_KEY, ",");
    }

    public boolean isTraceEnable() {
        String modules = getProperty(TRACE_KEY);
        return ((modules != null) && (!modules.equals("")));
    }

    private Collection getStrings(Object key, String separator) {
        String value = prop.getProperty((String) key);
        List result = new ArrayList();
        if (value != null) {
            StringTokenizer tokenizer = new StringTokenizer(value, separator);
            while (tokenizer.hasMoreTokens()) {
                result.add(tokenizer.nextToken().trim());
            }
        }
        return result;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("JalistoClientProperties[");
        Iterator keys = prop.keySet().iterator();
        while (keys.hasNext()) {
            String key = (String) keys.next();
            sb.append(key).append("=").append(prop.getProperty(key));
            if (keys.hasNext()) {
                sb.append(",");
            }
        }
        sb.append("]");
        return sb.toString();
    }

    public JalistoProperties getClone() {
        return new JalistoPropertiesClientImpl(propertiesPath);
    }

    private void truncateProperties() {
        Iterator keys = prop.keySet().iterator();
        while (keys.hasNext()) {
            String key = (String) keys.next();
            String value = prop.getProperty(key);
            while (value.endsWith(" ") || value.endsWith("\t")) {
                value = value.substring(0, value.length() - 1);
            }
            prop.setProperty(key, value);
        }
    }

    public String getPropertiesPath() {
        return propertiesPath;
    }

    /**
     * **************************************   FROM INTERFACE   ******************************************
     */

    public void checkProperty(JalistoProperties oldProps, String property) {
        throw new UnsupportedOperationException();
    }

    public void compareProperties(JalistoProperties oldProps) {
        throw new UnsupportedOperationException();
    }

    public int getAdditionalSpace() {
        return remoteProperties.getAdditionalSpace();
    }

    public short getClassPageSize() {
        return remoteProperties.getClassPageSize();
    }

    public String getDbFileFullName() {
        return remoteProperties.getDbFileFullName();
    }

    public String getImplementation() {
        return remoteProperties.getImplementation();
    }

    public int getInitialSize() {
        return remoteProperties.getInitialSize();
    }

    public int getInstanceDbFileNumber() {
        return remoteProperties.getInstanceDbFileNumber();
    }

    public String getInstanceDbFilePathAt(int index) {
        return remoteProperties.getInstanceDbFilePathAt(index);
    }

    public short getInstancePageSize() {
        return remoteProperties.getInstancePageSize();
    }

    public int getKeyLength() {
        return remoteProperties.getKeyLength();
    }

    public String getLogFileName() {
        return remoteProperties.getLogFileName();
    }

    public int getMBeanJmxPort() {
        return remoteProperties.getMBeanJmxPort();
    }

    public String getOidDbFilePath() {
        return remoteProperties.getOidDbFilePath();
    }

    public int getOidPageSize() {
        return remoteProperties.getOidPageSize();
    }

    public int getOidTableSize() {
        return remoteProperties.getOidTableSize();
    }

    public int getPageCacheSize() {
        return remoteProperties.getPageCacheSize();
    }

    public String[] getPathes() {
        return remoteProperties.getPathes();
    }

    public String getSystemDbFilePath() {
        return remoteProperties.getSystemDbFilePath();
    }

    public short getSystemPageSize() {
        return remoteProperties.getSystemPageSize();
    }

    public boolean isCleanLog() {
        return remoteProperties.isCleanLog();
    }

    public boolean allowsSpecialFunctionnalities() {
        return getProperty(ALLOWS_SPECIAL_FUNCTIONNALITIES_KEY).equalsIgnoreCase("yes");
    }

    public boolean isKeepingInMemory() {
        return remoteProperties.isKeepingInMemory();
    }

    public boolean isMonoImplementation() {
        return remoteProperties.isMonoImplementation();
    }

    public boolean isMultiImplementation() {
        return remoteProperties.isMultiImplementation();
    }

    public boolean isReadOnlyImplementation() {
        return remoteProperties.isReadOnlyImplementation();
    }

    public String getInternalFactoryClass() {
        return remoteProperties.getInternalFactoryClass();
    }
}
