/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.client;

import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.api.internal.InternalMetaRepository;
import org.objectweb.jalisto.se.api.remote.ClientCommunicationAgent;
import org.objectweb.jalisto.se.exception.JalistoException;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

public class RemoteIdentityProvider {

    public RemoteIdentityProvider(ClientCommunicationAgent connexion, InternalMetaRepository repository) {
        this.connexion = connexion;
        this.repository = repository;
        this.numberOfOidToReserve = 1000;
        this.transactionnalFloids = new HashMap();
        int size = repository.getAllClassNames().size();
        currentFloids = new long[size];
        maxFloids = new long[size];
        for (int i = 0; i < size; i++) {
            currentFloids[i] = -1;
            maxFloids[i] = -1;
        }
    }

    public Object makeNewFileOid(String objectClassName) {
        short clid = ((Short) repository.getClidFromClassName(objectClassName)).shortValue();
        if ((currentFloids[clid] == -1) || (currentFloids[clid] >= maxFloids[clid])) {
            retrieveFloids(clid);
        }
        LogicalOid result = new LogicalOid(currentFloids[clid]++ + LogicalOid.classMulti * clid);
        transactionnalFloids.put(result, null);
        return result;
    }

    public void addClass(short clid) {
        if (clid == maxFloids.length) {
            long[] newCurrentFloids = new long[currentFloids.length + 1];
            long[] newMaxFloids = new long[maxFloids.length + 1];
            System.arraycopy(currentFloids, 0, newCurrentFloids, 0, currentFloids.length);
            System.arraycopy(maxFloids, 0, newMaxFloids, 0, maxFloids.length);
            currentFloids = newCurrentFloids;
            maxFloids = newMaxFloids;
            currentFloids[clid] = -1;
            maxFloids[clid] = -1;
        } else {
            throw new JalistoException();
        }
    }

    public void useFloid(Object floid) {
        transactionnalFloids.remove(floid);
    }

    public void addFloidForClid(ArrayList floids, short clid) {
        Iterator iterator = transactionnalFloids.keySet().iterator();
        while (iterator.hasNext()) {
            LogicalOid floid = (LogicalOid) iterator.next();
            if (floid.getClid() == clid) {
                floids.add(floid);
            }
        }
    }

    public void commit() {
        connexion.makeOids(new ArrayList(transactionnalFloids.keySet()));
        transactionnalFloids.clear();
    }

    public void rollback() {
        transactionnalFloids.clear();
    }

    private void retrieveFloids(short clid) {
        long floid = connexion.reserveFloids(clid, numberOfOidToReserve).longValue();
        currentFloids[clid] = floid;
        maxFloids[clid] = floid + numberOfOidToReserve;
    }

    private int numberOfOidToReserve;

    private long[] currentFloids;
    private long[] maxFloids;
    private HashMap transactionnalFloids;

    private InternalMetaRepository repository;
    private ClientCommunicationAgent connexion;
}
