/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.lock;

import org.objectweb.jalisto.se.exception.multi.LockException;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

public class DeadLockToken implements Serializable {

    public DeadLockToken(Object tokenId) {
        this.tokenId = tokenId;
        nexts = new ArrayList();
        tokens.put(tokenId, this);
    }

    public static DeadLockToken getToken(Object tokenId) {
        return (DeadLockToken) tokens.get(tokenId);
    }

    /**
     * ********** STATIC *****************************
     */
    public static Map getTokens() {
        return tokens;
    }

    public boolean isWaitingFor(DeadLockToken token) {
        return nexts.contains(token);
    }

    public boolean couldWakeUp() {
        return nexts.isEmpty();
    }

    public synchronized void noMoreWaitFor(DeadLockToken releasingSessionToken) {
        nexts.remove(releasingSessionToken);
    }

    public synchronized void timeOutWakeUp(DeadLockToken tokenWhoWaitForThisOne) {
        if (tokenWhoWaitForThisOne != null) {
            tokenWhoWaitForThisOne.noMoreWaitFor(this);
            for (int i = 0; i < nexts.size(); i++) {
                tokenWhoWaitForThisOne.waitFor((DeadLockToken) nexts.get(i));
            }
        }
        nexts.clear();
    }

    public String toString() {
        if (nexts.isEmpty()) {
            return "DLT " + tokenId + " waiting for nobody";
        }
        StringBuffer result = new StringBuffer();
        result.append("DLT ").append(tokenId).append(" waiting for {");
        for (int i = 0; i < nexts.size(); i++) {
            result.append(((DeadLockToken) nexts.get(i)).tokenId);
            if (i != (nexts.size() - 1)) {
                result.append(", ");
            }
        }
        result.append('}');
        return result.toString();
    }

    public synchronized void waitFor(DeadLockToken tokenToWaitFor) {
        tokenToWaitFor.detectDeadLock(this);
        nexts.add(tokenToWaitFor);
    }

    private void detectDeadLock(DeadLockToken candidate) {
        if (this.equals(candidate)) {
            throw new LockException("dead lock : candidate " + candidate + " with " + tokens.toString());
        }
        if (!nexts.isEmpty()) {
            for (int i = 0; i < nexts.size(); i++) {
                ((DeadLockToken) nexts.get(i)).detectDeadLock(candidate);
            }
        }
    }

    private static Map tokens = new HashMap();
    private ArrayList nexts;
    private Object tokenId;

    static final long serialVersionUID = -7589377087964761459L;
}
