/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.meta.type;

import org.objectweb.jalisto.se.api.MetaType;
import org.objectweb.jalisto.se.exception.SchemaException;

public class MetaTypeFactory {

    public static MetaType getBasicMetaType(Class javaClass) {
        String className = javaClass.getName();
        if (className.equals("double") || className.equals("java.lang.Double")) {
            return new DoubleType();
        } else if (className.equals("float") || className.equals("java.lang.Float")) {
            return new FloatType();
        } else if (className.equals("int") || className.equals("java.lang.Integer")) {
            return new IntegerType();
        } else if (className.equals("long") || className.equals("java.lang.Long")) {
            return new LongType();
        } else if (className.equals("short") || className.equals("java.lang.Short")) {
            return new ShortType();
        } else if (className.equals("boolean") || className.equals("java.lang.Boolean")) {
            return new BooleanType();
        } else if (className.equals("byte") || className.equals("java.lang.Byte")) {
            return new ByteType();
        } else if (className.equals("char") || className.equals("java.lang.Character")) {
            return new CharacterType();
        } else if (className.equals("java.lang.String")) {
            return new StringType();
        } else if (className.equals("java.math.BigDecimal")) {
            return new BigDecimalType();
        } else if (className.equals("java.math.BigInteger")) {
            return new BigIntegerType();
        } else if (className.equals("java.util.Date")) {
            return new DateType();
        }
        throw new SchemaException("unknown basic type : " + className);
    }

    public static MetaType getMetaType(String typeName) {
        if (typeName.equals("Double")) {
            return new DoubleType();
        } else if (typeName.equals("Float")) {
            return new FloatType();
        } else if (typeName.equals("Boolean")) {
            return new BooleanType();
        } else if (typeName.equals("Integer")) {
            return new IntegerType();
        } else if (typeName.equals("Link")) {
            return new LinkType();
        } else if (typeName.equals("Long")) {
            return new LongType();
        } else if (typeName.equals("Byte")) {
            return new ByteType();
        } else if (typeName.equals("Character")) {
            return new CharacterType();
        } else if (typeName.equals("Serialized")) {
            return new SerializedType();
        } else if (typeName.equals("Short")) {
            return new ShortType();
        } else if (typeName.equals("String")) {
            return new StringType();
        } else if (typeName.equals("BigInteger")) {
            return new BigIntegerType();
        } else if (typeName.equals("BigDecimal")) {
            return new BigDecimalType();
        } else if (typeName.equals("Date")) {
            return new DateType();
        } else if (typeName.equals("Array")) {
            return new ArrayType();
        } else if (typeName.equals("Collection")) {
            return new CollectionType();
        } else if (typeName.equals("Map")) {
            return new MapType();
        }
        throw new SchemaException("unknown type : " + typeName);
    }

    public static final MetaType ArrayType = new ArrayType();
    public static final MetaType BigDecimalType = new BigDecimalType();
    public static final MetaType BigIntegerType = new BigIntegerType();
    public static final MetaType BooleanType = new BooleanType();
    public static final MetaType ByteType = new ByteType();
    public static final MetaType CharacterType = new CharacterType();
    public static final MetaType CollectionType = new CollectionType();
    public static final MetaType DateType = new DateType();
    public static final MetaType DoubleType = new DoubleType();
    public static final MetaType FloatType = new FloatType();
    public static final MetaType IntegerType = new IntegerType();
    public static final MetaType LinkType = new LinkType();
    public static final MetaType LongType = new LongType();
    public static final MetaType MapType = new MapType();
    public static final MetaType SerializedType = new SerializedType();
    public static final MetaType ShortType = new ShortType();
    public static final MetaType StringType = new StringType();
}
