/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.multi;

import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.api.internal.OidTable;
import org.objectweb.jalisto.se.api.internal.InternalPhysicalFileAccess;
import org.objectweb.jalisto.se.api.JalistoProperties;
import org.objectweb.jalisto.se.exception.IdentityException;
import org.objectweb.jalisto.se.impl.ExtentImpl;
import org.objectweb.jalisto.se.impl.server.PhysicalOid;
import org.objectweb.jalisto.se.impl.server.oid.OidInfo;
import org.objectweb.jalisto.se.impl.server.oid.OidTableImpl;

import java.util.*;

public class OidTableMultiImpl extends OidTableImpl {

    private OidTableMultiImpl(InternalPhysicalFileAccess physicalAccess, JalistoProperties properties) {
        super(physicalAccess, properties);
        transactionnalOidsMap = new HashMap();
    }

    public synchronized void setUpdate(Object sessionId, LogicalOid floid, short value) {
        // info must be here because method is called just after modification during commit phase
        OidInfo info = (OidInfo) getTransactionnalOids(sessionId).get(floid);
        if (info == null) {
            info = (OidInfo) floidTable.get(floid);
            if (info == null) {
                if (loadOidPage(sessionId, floid)) {
                    info = (OidInfo) floidTable.get(floid);
                } else {
                    throw new IdentityException();
                }
            }
        }
        info.setUpdate(value);
    }

    public synchronized short getUpdate(Object sessionId, LogicalOid floid, boolean transactionnal) {
        OidInfo info = null;
        if (transactionnal) {
            info = (OidInfo) getTransactionnalOids(sessionId).get(floid);
        }
        if (info == null) {
            info = (OidInfo) floidTable.get(floid);
            if (info == null) {
                if (loadOidPage(sessionId, floid)) {
                    info = (OidInfo) floidTable.get(floid);
                } else {
                    return -1;
                }
            }
        }
        return info.getUpdate();
    }

    protected Map getTransactionnalOids(Object sessionId) {
        return (Map) transactionnalOidsMap.get(sessionId);
    }

    public void open(Object sessionId) {
        transactionnalOidsMap.put(sessionId, new TreeMap());
    }

    public void close(Object sessionId) {
        transactionnalOidsMap.remove(sessionId);
    }

    /**
     * *****************************  synchronized methods   ***********************************
     */

    public synchronized LogicalOid allocateNewFloid(short clid) {
        return super.allocateNewFloid(clid);
    }

    public synchronized Collection getFloidsFromClid(Object sessionId, Object clid, boolean fullyTransactionnal) {
        return super.getFloidsFromClid(sessionId, clid, fullyTransactionnal);
    }

    public synchronized Collection getFloidsFromClid(Object sessionId, Object clid, ExtentImpl extent, int number) {
        return super.getFloidsFromClid(sessionId, clid, extent, number);
    }

    public synchronized PhysicalOid getFpoid(Object sessionId, LogicalOid floid) {
        return super.getFpoid(sessionId, floid);
    }

    public synchronized PhysicalOid getDeletedFpoid(Object sessionId, LogicalOid floid) {
        return super.getDeletedFpoid(sessionId, floid);
    }

    public synchronized void removeFloid(Object sessionId, LogicalOid floid) {
        super.removeFloid(sessionId, floid);
    }

    public synchronized void updatePoid(Object sessionId, LogicalOid floid, PhysicalOid newFpoid) {
        super.updatePoid(sessionId, floid, newFpoid);
    }

    private Map transactionnalOidsMap;

    /**
     * *******************************    STATIC    *******************************************
     */

    public static OidTable getAnOidTable(InternalPhysicalFileAccess physicalAccess, JalistoProperties properties) {
        OidTableMultiImpl oidTable;
        try {
            oidTable = new OidTableMultiImpl(physicalAccess, properties);
            oidTable.readAtTable();
        } catch (Exception e) {
            oidTable = new OidTableMultiImpl(physicalAccess, properties);
        }
        oidTable.oidPageSize = properties.getOidPageSize();
        return oidTable;
    }

}
