/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.readonly;

import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.api.*;
import org.objectweb.jalisto.se.api.internal.*;
import org.objectweb.jalisto.se.api.internal.multi.LockManager;
import org.objectweb.jalisto.se.api.query.QueryManager;
import org.objectweb.jalisto.se.exception.IdentityException;
import org.objectweb.jalisto.se.exception.SchemaException;
import org.objectweb.jalisto.se.exception.TransactionException;
import org.objectweb.jalisto.se.impl.*;
import org.objectweb.jalisto.se.impl.InFileAddress;
import org.objectweb.jalisto.se.impl.server.PhysicalOid;
import org.objectweb.jalisto.se.impl.trace.Trace;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

public class SessionReadOnlyImpl implements SessionInternal {

    public SessionReadOnlyImpl(JalistoProperties properties) {
        InternalFactory internalFactory = JalistoFactory.getInternalFactory();
        trace = internalFactory.getTracer(properties);
        sessionId = internalFactory.getSessionId(properties, this);
        trace.println(Trace.DEBUG, "CREATE A READ ONLY SESSION, id = {0}", sessionId);
        trace.println(Trace.SESSION, "{0} : create new instance with properties : {1}", sessionId, properties);
        fileAccess = internalFactory.getLogicalAccess(properties);
        oidTable = internalFactory.getOidTable(properties);
        repository = internalFactory.getMetaRepository(properties);
        props = properties;
        transaction = createTransaction();
    }


    public void defineClass(ClassDescription classMetaDescription) {
        throw new UnsupportedOperationException();
    }

    public void removeClass(String fullClassName) {
        throw new UnsupportedOperationException();
    }

    public Collection getAllClassNames() {
        return repository.getAllClassNames();
    }

    public boolean isClassDefined(String fullQualifiedClassName) {
        try {
            repository.getClidFromClassName(fullQualifiedClassName);
            return true;
        } catch (SchemaException e) {
        }
        return false;
    }

    public String getClassNameFor(Object oid) {
        trace.println(Trace.SESSION, "{0} : getClassNameFor({1})", sessionId, oid);
        LogicalOid floid = getFloidFromOid(oid);
        return repository.getClassNameFromClid(new Short(floid.getClid()));
    }

    public Object createObject(Object[] objectToCreate, Class objectClass) {
        throw new UnsupportedOperationException();
    }

    public Object createObject(Object[] objectToCreate, String objectClassName) {
        throw new UnsupportedOperationException();
    }

    public Object[] readObjectByOid(Object oid) {
        return readObjectByOid(oid, true);
    }

    public Collection readObjectsByOids(Collection oids) {
        Iterator oidsIterator = oids.iterator();
        ArrayList result = new ArrayList();
        while(oidsIterator.hasNext()) {
            result.add(readObjectByOid(oidsIterator.next(), true));
        }
        return result;
    }

    public boolean contains(Object oid) {
        trace.println(Trace.SESSION, "{0} contains({1})", sessionId, oid);
        checkValidity("contains", true);
        return oidTable.containsFloid(sessionId, (LogicalOid) oid);
    }

    public Extent getExtent(Class theClass) {
        String fullClassName = theClass.getName();
        return getExtent(fullClassName);
    }

    public Collection getFloidsFromClid(Object sessionId, Object clid) {
        return oidTable.getFloidsFromClid(sessionId, clid, false);
    }

    public Long reserveFloids(short clid, int number) {
        throw new UnsupportedOperationException();
    }


    public void setOptimistic() {
        throw new UnsupportedOperationException();
    }

    public void setPessimistic() {
        throw new UnsupportedOperationException();
    }

    public void begin() {
        trace.println(Trace.SESSION, "{0} : begin()", sessionId);
        fileAccess.begin(sessionId);
    }

    public void commit() {
        trace.println(Trace.SESSION, "{0} : commit()", sessionId);
        fileAccess.commit(sessionId);
    }

    public void rollback() {
        trace.println(Trace.SESSION, "{0} : rollback()", sessionId);
        fileAccess.rollback(sessionId);
    }

    public void openSession() {
        repository.addSession(this);
        isOpen = true;
    }

    public void closeSession() {
        repository.removeSession(this);
        isOpen = false;
    }

    public boolean isOpen() {
        return isOpen;
    }

    public QueryManager getQueryManager() {
        if (queryManager == null) {
            try {
                String queryManagerClassName = props.getProperty(JalistoProperties.QUERY_MANAGER_CLASS_KEY);
                queryManager = (QueryManager) Class.forName(queryManagerClassName).newInstance();
                queryManager.init(this);
            } catch (InstantiationException e) {
                trace.println(Trace.TRACE_ON, "cannot instanciate query manager : {0}", e.getMessage());
            } catch (IllegalAccessException e) {
                trace.println(Trace.TRACE_ON, "cannot initialize query manager : {0}", e.getMessage());
            } catch (ClassNotFoundException e) {
                trace.println(Trace.TRACE_ON, "cannot  find query manager class : {0}", e.getMessage());
            }
        }
        return queryManager;
    }

    public Transaction currentTransaction() {
        return transaction;
    }

    public LogicalSystemPageAccess getFileAccess() {
        return fileAccess;
    }

    public MetaRepository getMetaRepository() {
        return repository;
    }

    public SessionInternal getInternalSession() {
        return this;
    }

    public JalistoProperties getProperties() {
        return props;
    }

    public OidTable getOidTable() {
        return oidTable;
    }

    public LockManager getLockManager() {
        throw new UnsupportedOperationException();
    }

    public boolean isNewBase() {
        return fileAccess.getPhysicalAccess().isNewBase();
    }

    public Object getSessionId() {
        return sessionId;
    }

    public void clearObjectCache() {
    }

    public Object makeNewFileOid(LogicalOid floid) {
        throw new UnsupportedOperationException();
    }

    public void reorganize() {
        throw new UnsupportedOperationException();
    }

    public String toString() {
        return "session " + sessionId;
    }

    public void checkValidity(String message, boolean mustBeActive) {
        if (mustBeActive) {
            if (!transaction.isActive()) {
                throw new TransactionException("Transaction must be active during " + message);
            }
        } else {
            if (transaction.isActive()) {
                throw new TransactionException("Transaction must not be active during " + message);
            }
        }
    }

    public boolean isRemoteSession() {
        return false;
    }

    /**
     * ******************************************************************************************
     */

    public Object[] readObjectByOid(Object oid, boolean withCache) {
        trace.println(Trace.SESSION, "{0} : readObjectByOid({1})", sessionId, oid);
        checkValidity("readObjectByOid", true);
        LogicalOid floid = getFloidFromOid(oid);
        PhysicalOid fpoid = oidTable.getFpoid(sessionId, floid);
        if (fpoid != null) {
            DataWrapper datas = fileAccess.readDatas(sessionId, fpoid);
            if (datas != null) {
                return datas.getDatas();
            }
        }
        throw new IdentityException("the given oid " + oid + " doesn't exist in base");
    }

    public Object[] refreshObjectByOid(Object oid) {
        return readObjectByOid(oid);
    }

    public Object createObject(Object oid, Object[] objectToCreate) {
        throw new UnsupportedOperationException();
    }

    public void deleteObjectByOid(Object oid) {
        throw new UnsupportedOperationException();
    }

    public Extent getExtent(String fullClassName) {
        trace.println(Trace.SESSION, "{0} : getExtent({1})", sessionId, fullClassName);
        checkValidity("getExtent", true);
        return new ExtentImpl(repository.getClidFromClassName(fullClassName), this);
    }

    public Object makeNewFileOid(Class objectClass) {
        throw new UnsupportedOperationException();
    }

    public Object makeNewFileOid(String objectClassName) {
        throw new UnsupportedOperationException();
    }

    public Object updateObjectByOid(Object oid, Object[] objectToUpdate) {
        throw new UnsupportedOperationException();
    }

    public void eraseStorage() {
        throw new UnsupportedOperationException();
    }

    /**
     * *************************************    PRIVATE    *************************************
     */

    private LogicalOid getFloidFromOid(Object oid) {
        //        trace.println(Trace.SESSION, "{0} : getFloidFromOid({1})", sessionId, oid);
        try {
            return (LogicalOid) oid;
        } catch (ClassCastException cce) {
            throw new IdentityException("oid is not a FileLoid but a " + oid.getClass().getName());
        }
    }

    protected Transaction createTransaction() {
        return new TransactionImpl(this);
    }

    private OidTable oidTable;
    private LogicalSystemPageAccess fileAccess;
    private InternalMetaRepository repository;
    private QueryManager queryManager;
    private Transaction transaction;
    private boolean isOpen = false;

    protected JalistoProperties props;
    private Object sessionId;

    protected Trace trace;

    public static final InFileAddress BSTATE_IFA = new InFileAddress("bState");
}
