/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.server.oid;

import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.api.JalistoProperties;
import org.objectweb.jalisto.se.api.cache.JalistoCache;
import org.objectweb.jalisto.se.api.internal.*;
import org.objectweb.jalisto.se.exception.IdentityException;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.impl.ExtentImpl;
import org.objectweb.jalisto.se.impl.InFileAddress;
import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.impl.server.IdentityProvider;
import org.objectweb.jalisto.se.impl.server.PhysicalOid;
import org.objectweb.jalisto.se.impl.server.page.OidPage;
import org.objectweb.jalisto.se.impl.trace.Trace;

import java.util.*;

public abstract class OidTableImpl implements OidTable, OidTableCallback {

    protected OidTableImpl(InternalPhysicalFileAccess physicalAccess, JalistoProperties properties) {
        trace = JalistoFactory.getInternalFactory().getTracer(properties);
        oidTableSize = properties.getOidTableSize();
        trace.println(Trace.OIDTABLE, "OidTableImpl : create new instance with size = {0}",
                       new Integer(oidTableSize));
        trace.println(Trace.DEBUG, "OidTableImpl : create new instance with size = {0}",
                       new Integer(oidTableSize));

        floidTable = JalistoFactory.getInternalFactory().getCache(properties, oidTableSize, "OidTableImpl");
        this.physicalAccess = physicalAccess;
        this.oidProvider = JalistoFactory.getInternalFactory().getIdentityProvider(properties);
        this.oidIfaBuffer = new InFileAddress("");
        this.oidIfaBuffer.setFileIndex(JalistoProperties.OID_INDEX);
    }


    public int getOidTableSize() {
        return oidTableSize;
    }

    public void setOidTableSize(int oidTableSize) {
        this.oidTableSize = oidTableSize;
        try {
            ((JalistoCache) floidTable).setMaxSize(oidTableSize);
        } catch (ClassCastException cce) {
        }
    }

    public int getTransactionalSize(Object sessionId) {
        return getTransactionnalOids(sessionId).size() + floidTable.size();
    }


    public void setUpdate(Object sessionId, LogicalOid floid, short value) {
        throw new UnsupportedOperationException("should not be used in this implementation");
    }

    public short getUpdate(Object sessionId, LogicalOid floid, boolean transactionnal) {
        throw new UnsupportedOperationException("should not be used in this implementation");
    }

    protected abstract Map getTransactionnalOids(Object sessionId);


    public LogicalOid allocateNewFloid(short clid) {
        return oidProvider.allocateNewFloid(clid);
    }

    public void insertFpoid(Object sessionId, LogicalOid floid, PhysicalOid fpoid) {
        trace.println(Trace.OIDTABLE, "OidTableImpl : insertFpoid({0}, {1})", floid, fpoid);
        OidInfo oidInfo = new OidInfo(floid, fpoid);
        oidInfo.setActionCaller(OidActionCaller.INSERT_CALLER);
        getTransactionnalOids(sessionId).put(floid, oidInfo);
    }

    public void updatePoid(Object sessionId, LogicalOid floid, PhysicalOid newFpoid) {
        trace.println(Trace.OIDTABLE, "OidTableImpl : updatePoid({0}, {1})", floid, newFpoid);

        OidInfo info = (OidInfo) getTransactionnalOids(sessionId).get(floid);
        if (info == null) {
            info = (OidInfo) floidTable.get(floid);
            if (info == null) {
                if (loadOidPage(sessionId, floid)) {
                    info = (OidInfo) floidTable.get(floid);
                } else {
                    throw new IdentityException("the given oid " + floid + " doesn't exist in this Jalisto datastore");
                }
            }
            info = info.getClone();
            getTransactionnalOids(sessionId).put(floid, info);
        } else {
            if (info.getActionCaller().isDelete()) {
                throw new IdentityException("the given oid " + floid + " has been deleted from this Jalisto datastore");
            }
        }
        info.setActionCaller(OidActionCaller.UPDATE_CALLER);
        info.setFpoid(newFpoid);
    }

    public void removeFloid(Object sessionId, LogicalOid floid) {
        trace.println(Trace.OIDTABLE, "OidTableImpl : removeFloid({0})", floid);
        OidInfo info = (OidInfo) getTransactionnalOids(sessionId).get(floid);
        if (info != null) {
            if (info.getActionCaller().isInsert()) {
                getTransactionnalOids(sessionId).remove(floid);
                return;
            }
        } else {
            info = (OidInfo) floidTable.get(floid);
            if (info == null) {
                if (loadOidPage(sessionId, floid)) {
                    info = (OidInfo) floidTable.get(floid);
                } else {
                    throw new IdentityException("the given oid " + floid + " doesn't exist in this Jalisto datastore");
                }
            }
            info = info.getClone();
            getTransactionnalOids(sessionId).put(floid, info);
        }
        info.setActionCaller(OidActionCaller.DELETE_CALLER);
    }

    public Collection getFloidsFromClid(Object sessionId, Object clid, boolean fullyTransactionnal) {
        trace.println(Trace.OIDTABLE, "OidTableImpl : getFloidsFromClid({0})", clid);
        Collection workingExtent = new ArrayList();
        Iterator oidIfaList = physicalAccess.getKeysStartingWith(IFA_PREFIXE + clid, true).iterator();
        while (oidIfaList.hasNext()) {
            InFileAddress ifa = new InFileAddress((String) oidIfaList.next());
            ifa.setFileIndex(JalistoProperties.OID_INDEX);
            OidPage oidPage = (OidPage) physicalAccess.readFileObjectAt(ifa);
            Object[] infos = oidPage.getAllDatas();
            for (short i = 0; i < infos.length; i++) {
                OidInfo info = (OidInfo) infos[i];
                if (info != null) {
                    if ((info.isCreated() || fullyTransactionnal) &&
                        (info.getFpoid().hasClidEqualsWith(clid))) {   
                        workingExtent.add(info.getFloid());
                    }
                }
            }
        }
        short clidShort = ((Short) clid).shortValue();
        Iterator infosTx = getTransactionnalOids(sessionId).values().iterator();
        while (infosTx.hasNext()) {
            OidInfo info = (OidInfo) infosTx.next();
            if (info.getFloid().getClid() == clidShort) {
                if (info.getActionCaller().isDelete()) {
                    workingExtent.remove(info.getFloid());
                } else if ((info.isCreated() || fullyTransactionnal) &&
                           (info.getActionCaller().isInsert())){
                    workingExtent.add(info.getFloid());
                }
            }
        }

        return workingExtent;
    }

    public Collection getFloidsFromClid(Object sessionId, Object clid, ExtentImpl extent, int number) {
        TreeSet newOids = new TreeSet();
        TreeSet removedOids = new TreeSet();
        short clidShort = ((Short) clid).shortValue();
        Iterator infosTx = getTransactionnalOids(sessionId).values().iterator();
        while (infosTx.hasNext()) {
            OidInfo info = (OidInfo) infosTx.next();
            if (info.getFloid().getClid() == clidShort) {
                if (info.getActionCaller().isDelete()) {
                    removedOids.add(info.getFloid());
                } else if (info.getActionCaller().isInsert()) {
                    newOids.add(info.getFloid());
                }
            }
        }

        LogicalOid lastFloid = extent.getLastFloid();
        if (lastFloid != null) {
            newOids.tailSet(lastFloid);
            newOids.remove(lastFloid);
        }

        TreeSet workingExtent = new TreeSet();
        String startingAddress = extent.getStartingAddress();

        Iterator oidIfaList = physicalAccess.getKeysStartingWith(IFA_PREFIXE + clid, true).iterator();
        boolean continueSearching = true;
        String currentOidPageAddress = "";
        int currentIndex;
        while (oidIfaList.hasNext() && continueSearching) {
            currentOidPageAddress = (String) oidIfaList.next();
            if (currentOidPageAddress.compareTo(startingAddress) >= 0) {
                InFileAddress ifa = new InFileAddress(currentOidPageAddress);
                ifa.setFileIndex(JalistoProperties.OID_INDEX);
                OidPage oidPage = (OidPage) physicalAccess.readFileObjectAt(ifa);
                Object[] infos = oidPage.getAllDatas();
                for (currentIndex = 0; currentIndex < infos.length; currentIndex++) {
                    OidInfo info = (OidInfo) infos[currentIndex];
                    if (info != null) {
                        LogicalOid floid = info.getFloid();
                        if ((lastFloid == null) || (floid.compareTo(lastFloid) > 0)) {
                            try {
                                LogicalOid firstNewOid = (LogicalOid) newOids.first();
                                while ((firstNewOid.compareTo(floid) < 0) && continueSearching) {
                                    workingExtent.add(firstNewOid);
                                    if (workingExtent.size() >= number) {
                                        continueSearching = false;
                                    }
                                    newOids.remove(firstNewOid);
                                    firstNewOid = (LogicalOid) newOids.first();
                                }
                            } catch (NoSuchElementException nsee) {
                            }
                            if ((!removedOids.contains(floid)) && continueSearching) {
                                workingExtent.add(floid);
                                if (workingExtent.size() >= number) {
                                    continueSearching = false;
                                }
                            }
                        }
                    }
                }
            }
        }

        extent.setStartingAddress(currentOidPageAddress);
        return workingExtent;
    }

    public PhysicalOid getFpoid(Object sessionId, LogicalOid floid) {
        trace.println(Trace.OIDTABLE, "OidTableImpl : getFpoid({0})", floid);
        OidInfo info = (OidInfo) getTransactionnalOids(sessionId).get(floid);
        if (info != null) {
            if (info.getActionCaller().isDelete()) {
                throw new IdentityException("the given oid " + floid + " has been deleted from this Jalisto datastore");
            }
        } else {
            info = (OidInfo) floidTable.get(floid);
            if (info == null) {
                if (loadOidPage(sessionId, floid)) {
                    info = (OidInfo) floidTable.get(floid);
                    if (info == null) {
                        throw new IdentityException("null info");
                    }
                } else {
                    throw new IdentityException("the given oid " + floid + " doesn't exist in this Jalisto datastore");
                }
            }
        }
        return info.getFpoid();
    }

    public void markAsCreated(Object sessionId, LogicalOid floid) {
        trace.println(Trace.OIDTABLE, "OidTableImpl : markAsCreated({0})", floid);
        OidInfo info = (OidInfo) getTransactionnalOids(sessionId).get(floid);
        if (info == null) {
            info = (OidInfo) floidTable.get(floid);
            if (info == null) {
                if (loadOidPage(sessionId, floid)) {
                    info = (OidInfo) floidTable.get(floid);
                    if (info == null) {
                        throw new IdentityException("null info");
                    }
                } else {
                    throw new IdentityException("the given oid " + floid + " doesn't exist in this Jalisto datastore");
                }
            }
        }
        info.markAsCreated();
    }

    public boolean containsFloid(Object sessionId, LogicalOid floid) {
        trace.println(Trace.OIDTABLE, "OidTableImpl : containsFloid({0})", floid);
        OidInfo info = (OidInfo) getTransactionnalOids(sessionId).get(floid);
        if (info != null) {
            return (!info.getActionCaller().isDelete());
        } else {
            info = (OidInfo) floidTable.get(floid);
            if (info == null) {
                return loadOidPage(sessionId, floid);
            }
            return true;
        }
    }

    public PhysicalOid getDeletedFpoid(Object sessionId, LogicalOid floid) {
        trace.println(Trace.OIDTABLE, "OidTableImpl : getDeletedFpoid({0})", floid);
        OidInfo info = (OidInfo) getTransactionnalOids(sessionId).get(floid);
        if ((info != null) && info.getActionCaller().isDelete()) {
            return info.getFpoid();
        }
        throw new IdentityException("the given oid " + floid + " has not been deleted from this Jalisto datastore");
    }

    public void begin(Object sessionId) {
    }

    public void commit(Object sessionId) {
        oidProvider.commit();

        Map txOids = getTransactionnalOids(sessionId);
        Iterator oids = txOids.keySet().iterator();
        while (oids.hasNext()) {
            LogicalOid floid = (LogicalOid) oids.next();
            OidInfo info = (OidInfo) txOids.get(floid);
            info.getActionCaller().callCommitAction(this, floid, info, floidTable);
        }
        txOids.clear();
    }

    public void rollback(Object sessionId) {
        getTransactionnalOids(sessionId).clear();
    }

    public void close(Object sessionId) {
    }

    public void open(Object sessionId) {
    }

    public void insertOidInfoInBase(OidInfo info) {
        setOidIfa(info.getFloid());
        OidPage oidPage = getOidPage();

        Object old = oidPage.getDataAt(oidIfaBuffer);
        if (old != null) {
            throw new IdentityException("the given oid " + info.getFloid() + " already exist in this Jalisto datastore");
        }
        oidPage.setDataAt(oidIfaBuffer, info);
        physicalAccess.updateFileObject(oidPage);
    }

    public void updateOidInfoInBase(OidInfo info) {
        setOidIfa(info.getFloid());
        OidPage oidPage = getOidPage();

        oidPage.setDataAt(oidIfaBuffer, info);
        physicalAccess.updateFileObject(oidPage);
    }

    public void deleteOidInfoInBase(LogicalOid floid) {
        setOidIfa(floid);
        OidPage oidPage = getOidPage();

        oidPage.setDataAt(oidIfaBuffer, null);
        if (oidPage.getUsedSpace() == 0) {
            physicalAccess.deleteFileObject(oidIfaBuffer.getClone());
        } else {
            physicalAccess.updateFileObject(oidPage);
        }
    }

    /**
     * *********************************    private    **********************************************
     */

    public IdentityProvider getIdentityProvider() {
        return oidProvider;
    }

    private void setOidIfa(LogicalOid floid) {
        long instanceNumber = floid.getInstanceNumber();
        oidIfaBuffer.setAddress(IFA_PREFIXE + floid.getClid() + "-" + (instanceNumber / oidPageSize));
        oidIfaBuffer.setIndex((int) (instanceNumber % oidPageSize));
    }

    private OidPage getOidPage() {
        try {
            // could raise exception if oidPage don't exist
            return (OidPage) physicalAccess.readFileObjectAt(oidIfaBuffer);
        } catch (JalistoException fde) {
            if (fde.getMessage().startsWith("could not read object at ifa(" + IFA_PREFIXE)) {
                // create a new OidPage. Don't happends if we are deleting
                return createOidPage();
            } else {
                throw fde;
            }
        }
    }

    private OidPage createOidPage() {
        OidPage oidPage = new OidPage(oidPageSize);
        oidPage.setIfa(oidIfaBuffer);
        physicalAccess.insertFileObject(oidPage);
        return oidPage;
    }

    protected boolean loadOidPage(Object sessionId, LogicalOid floid) {
        int index = (int) (floid.getInstanceNumber() % oidPageSize);
        setOidIfa(floid);
        try {
            // could raise exception if oidPage don't exist
            OidPage oidPage = (OidPage) physicalAccess.readFileObjectAt(oidIfaBuffer);
            Object[] infos = oidPage.getAllDatas();
            Map txOids = getTransactionnalOids(sessionId);
            for (int j = 0; j < infos.length; j++) {
                OidInfo info = (OidInfo) infos[j];
                if (info != null) {
                    LogicalOid readedFloid = info.getFloid();
                    if (!txOids.containsKey(readedFloid)) {
                        floidTable.put(readedFloid, info);
                    }
                }
            }
            return (infos[index] != null);
        } catch (Exception e) {         // TODO amliorer le catch
            trace.println(Trace.OIDTABLE, "OidTableImpl : loadOidPage : oidPage doesn't exist");
        }
        return false;
    }

    protected void readAtTable() {
        trace.println(Trace.OIDTABLE, "OidTableImpl : readAtTable()");
        int dataRead = 0;
        Iterator oidIfaList = physicalAccess.getKeysStartingWith(IFA_PREFIXE, true).iterator();
        while (oidIfaList.hasNext()) {
            InFileAddress ifa = new InFileAddress((String) oidIfaList.next());
            ifa.setFileIndex(JalistoProperties.OID_INDEX);
            OidPage oidPage = (OidPage) physicalAccess.readFileObjectAt(ifa);
            Object[] infos = oidPage.getAllDatas();
            for (int i = 0; i < infos.length; i++) {
                OidInfo info = (OidInfo) infos[i];
                if (info != null) {
                    floidTable.put(info.getFloid(), info);
                    dataRead++;
                }
                if (dataRead == oidTableSize) {       // don't read more than the oidtable size
                    return;
                }
            }
        }
    }


    protected InternalPhysicalFileAccess physicalAccess;
    protected IdentityProvider oidProvider;
    protected Map floidTable;
    protected int oidPageSize;
    protected int oidTableSize;
    protected Trace trace;

    private InFileAddress oidIfaBuffer;

    private static final String IFA_PREFIXE = "id";
}
