/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.impl.server.socket;

import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.api.internal.SessionInternal;
import org.objectweb.jalisto.se.api.internal.InternalMetaRepository;
import org.objectweb.jalisto.se.api.remote.ServerCommunicationAgent;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.exception.remote.RemoteException;
import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.impl.meta.InternalMetaRepositoryImpl;
import org.objectweb.jalisto.se.impl.trace.Trace;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.net.SocketException;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

public class ServerCommunicationAgentSocketImpl extends Thread implements ServerCommunicationAgent {

    public ServerCommunicationAgentSocketImpl(Socket socket) {
        try {
            this.socket = socket;
            this.socketInfos = socket.toString();
            this.fromClient = new ObjectInputStream(socket.getInputStream());
            this.toClient = new ObjectOutputStream(socket.getOutputStream());
        } catch (IOException ioe) {
            throw new RemoteException(ioe);
        }
    }

    private void initSession() throws IOException {
        String propertiesPath = (String) readObject();
        session = (SessionInternal) JalistoFactory.getSession(propertiesPath);
        trace = JalistoFactory.getInternalFactory().getTracer(session.getProperties());
        trace.println(Trace.REMOTE, "get session with properties path = {0}", propertiesPath);
    }

    public SessionInternal getSession() {
        return session;
    }

    public void run() {
        try {
            initSession();
            trace.println(Trace.REMOTE, "Run ServerCommunicationAgentSocketImpl on " + socketInfos);
        } catch (IOException ioe) {
            throw new JalistoException(ioe);
        }
        try {
            while (decode) {
                try {
                    int code = readInt();
                    decodeMethodCode(code);
                } catch (SocketException se) {
                    if (se.getMessage().equals("Connection reset")) {
                        trace.println(Trace.REMOTE, "{0} closed", socketInfos);
                        return;
                    } else {
                        throw new JalistoException(se);
                    }
                }
            }
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } finally {
            closeConnection();
        }
    }

    private void closeConnection() {
        trace.println(Trace.REMOTE, "cleaning connection");
        try {
            if (fromClient != null) {
                fromClient.close();
            }
            if (toClient != null) {
                toClient.close();
            }
            if (socket != null) {
                trace.println(Trace.REMOTE, "---> closing {0}", socket.toString());
                socket.close();
            }
        } catch (Exception e) {
        }
        if (session != null) {
            if (session.currentTransaction().isActive()) {
                session.currentTransaction().rollback();
            }
            if (session.isOpen()) {
                session.closeSession();
            }
        }
    }

    private int readInt() throws IOException {
        while (true) {
            int request = fromClient.read();
            if (request > 0) {
                trace.println(Trace.REMOTE, "read int = " + request);
                return request;
            }
        }
    }

    private Object readObject() throws IOException {
        try {
            Object request = null;
            while (request == null) {
                request = fromClient.readObject();
            }
            return request;
        } catch (ClassNotFoundException cnfe) {
            throw new JalistoException(cnfe);
        }
    }

    private void sendException(Exception e) throws IOException {
        RemoteException re = new RemoteException(e);
        toClient.writeObject(re);
        toClient.flush();
//        re.printStackTrace();
    }

    private void sendBack(Object reply) throws IOException {
        toClient.writeObject(reply);
        toClient.flush();
    }

    private void decodeMethodCode(int code) throws IOException {
        if (code < 6) {
            switch (code) {
                case 1:
                    try {
                        session.closeSession();
                        sendBack("ok");
//                        decode = false;
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                case 2:
                    try {
                        Collection toRead = (Collection) readObject();
                        sendBack(session.readObjectsByOids(toRead));
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                case 3:
                    try {
                        LogicalOid oid = (LogicalOid) readObject();
                        Object[] objectToCreate = (Object[]) readObject();
                        session.makeNewFileOid(oid);
                        Object result = session.createObject(oid, objectToCreate);
                        sendBack(result);
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                case 4:
                    try {
                        ClassDescription meta = (ClassDescription) readObject();
                        session.defineClass(meta);
                        sendBack("ok");
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                case 5:
                    try {
                        Object oid = readObject();
                        session.deleteObjectByOid(oid);
                        sendBack("ok");
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;

            }
        } else if (code < 11) {
            switch (code) {
                case 6:
                    try {
                        String fullClassName = (String) readObject();
                        Object result = session.getExtent(fullClassName);
                        sendBack(result);
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                case 7:
                    try {
                        String fullClassName = (String) readObject();
                        Object result = session.makeNewFileOid(fullClassName);
                        sendBack(result);
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                case 8:
                    try {
                        session.openSession();
                        sendBack("ok");
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                case 9:
                    try {
                        String fullClassName = (String) readObject();
                        session.removeClass(fullClassName);
                        sendBack("ok");
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                case 10:
                    try {
                        Object oid = readObject();
                        Object[] objectToUpdate = (Object[]) readObject();
                        Object result = session.updateObjectByOid(oid, objectToUpdate);
                        sendBack(result);
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                case 11:
                    try {
                        Object oid = readObject();
                        Object result = session.refreshObjectByOid(oid);
                        sendBack(result);
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
            }
        } else if (code < 24) {
            switch (code) {
                case 20:
                    try {
                        session.currentTransaction().begin();
                        sendBack("ok");
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                case 21:
                    try {
                        session.currentTransaction().commit();
                        sendBack("ok");
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                case 22:
                    try {
                        Object oid = readObject();
                        Boolean withCache = (Boolean) readObject();
                        Object result = session.readObjectByOid(oid, withCache.booleanValue());
                        sendBack(result);
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                case 23:
                    try {
                        session.currentTransaction().rollback();
                        sendBack("ok");
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                default :
                    throw new JalistoException("unkown code : " + code);
            }
        } else if (code < 28) {
            switch (code) {
                case 24:
                    try {
                        sendBack(session.getProperties());
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                case 25:
                    try {
                        sendBack(session.getSessionId());
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                case 26:
                    try {
                        session.reorganize();
                        sendBack("ok");
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                case 27:
                    try {
                        session.eraseStorage();
                        sendBack("ok");
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
            }
        } else if (code < 35) {
            switch (code) {
                case 30:
                    try {
                        Object result = ((InternalMetaRepositoryImpl) session.getMetaRepository()).getClassMetas();
                        sendBack(result);
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                case 31:
                    try {
                        Object result = ((InternalMetaRepositoryImpl) session.getMetaRepository()).getClassTable();
                        sendBack(result);
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                case 32:
                    try {
                        String className = (String) readObject();
                        Object result = ((InternalMetaRepository)session.getMetaRepository()).getClidFromClassName(className);
                        sendBack(result);
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                case 33:
                    try {
                        Map createdObjects = (Map) readObject();
                        Iterator oids = createdObjects.keySet().iterator();
                        while (oids.hasNext()) {
                            LogicalOid oid = (LogicalOid) oids.next();
                            Object[] objectToCreate = (Object[]) createdObjects.get(oid);
                            if (!session.getOidTable().containsFloid(session.getSessionId(), oid)) {
                                session.makeNewFileOid(oid);
                            }
                            session.createObject(oid, objectToCreate);
                        }
                        sendBack("ok");
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                case 34:
                    try {
                        Map updatedObjects = (Map) readObject();
                        Iterator oids = updatedObjects.keySet().iterator();
                        while (oids.hasNext()) {
                            Object oid = oids.next();
                            Object[] objectToUpdate = (Object[]) updatedObjects.get(oid);
                            session.updateObjectByOid(oid, objectToUpdate);
                        }
                        sendBack("ok");
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                default :
                    throw new JalistoException("unkown code : " + code);
            }
        } else {
            switch (code) {
                case 35:
                    try {
                        Iterator oids = ((Collection) readObject()).iterator();
                        while (oids.hasNext()) {
                            Object oid = oids.next();
                            session.deleteObjectByOid(oid);
                        }
                        sendBack("ok");
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                case 36:
                    try {
                        Object sessionId = readObject();
                        Object clid = readObject();
                        Object result = session.getFloidsFromClid(sessionId, clid);
                        sendBack(result);
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                case 37:
                    try {
                        short clid = ((Short) readObject()).shortValue();
                        int number = ((Integer) readObject()).intValue();
                        ;
                        Long result = session.reserveFloids(clid, number);
                        sendBack(result);
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                case 38:
                    try {
                        Iterator oids = ((Collection) readObject()).iterator();
                        while (oids.hasNext()) {
                            LogicalOid oid = (LogicalOid) oids.next();
                            session.makeNewFileOid(oid);
                        }
                        sendBack("ok");
                    } catch (Exception e) {
                        sendException(e);
                    }
                    break;
                default :
                    throw new JalistoException("unkown code : " + code);
            }
        }
    }


    private boolean decode = true;
    private Socket socket;
    private String socketInfos;
    private ObjectInputStream fromClient;
    private ObjectOutputStream toClient;

    private SessionInternal session;
    private Trace trace;
}
