/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.storage.memory;

import org.objectweb.jalisto.se.api.internal.JalistoObject;
import org.objectweb.jalisto.se.api.physical.PluggablePhysicalFileAccess;
import org.objectweb.jalisto.se.api.JalistoProperties;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.impl.InFileAddress;
import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.impl.trace.Trace;

import java.util.*;

public class PhysicalFileAccessMemoryImpl implements PluggablePhysicalFileAccess {

    public PhysicalFileAccessMemoryImpl() {
    }

    public void init(JalistoProperties properties) {
        file = new HashMap();
        this.trace = JalistoFactory.getInternalFactory().getTracer(properties);
        trace.println(Trace.TRACE_ON, "USE IN MEMORY PHYSICAL IMPLEMENTATION");
    }

    public boolean isNewBase() {
        return true;
    }

    public void reorganize() {
    }

    public void deleteFileObject(InFileAddress ifa) {
        trace.println(Trace.PHYSICAL, "memory deleteFileObject : ifa = {0}", ifa);
        if (!file.containsKey(ifa.getAddress())) {
            throw new JalistoException("could not delete object at " + ifa);
        }
        file.remove(ifa.getAddress());
    }

    public void insertFileObject(InFileAddress ifa, JalistoObject fo) {
        trace.println(Trace.PHYSICAL, "memory insertFileObject : ifa = {0}", ifa);
        if (file.containsKey(ifa.getAddress())) {
            throw new JalistoException("already contains object at " + ifa + " : could not insert value");
        }
        file.put(ifa.getAddress(), fo);
    }

    public JalistoObject readFileObjectAt(InFileAddress ifa) {
        trace.println(Trace.PHYSICAL, "memory readFileObjectAt : ifa = {0}", ifa);
        if (file.containsKey(ifa.getAddress())) {
            return (JalistoObject) file.get(ifa.getAddress());
        }
        throw new JalistoException("could not read object at " + ifa);
    }

    public void updateFileObject(InFileAddress ifa, JalistoObject fo) {
        trace.println(Trace.PHYSICAL, "memory updateFileObject : ifa = {0}", ifa);
        if (!file.containsKey(ifa.getAddress())) {
            throw new JalistoException("could not update object " + fo);
        }
        file.put(ifa.getAddress(), fo);
    }

    public void startCommit() {
    }

    public void commit() {
    }

    public void rollback() {
        throw new JalistoException("don't support rollback with Memory physical access");
    }

    public void close() {
    }

    public void open() {
    }

    public Collection getKeysStartingWith(String filter) {
        trace.println(Trace.PHYSICAL, "memory getKeysStartingWith : filter = {0}", filter);
        Collection result = new ArrayList();
        Iterator keys = file.keySet().iterator();
        while (keys.hasNext()) {
            String key = (String) keys.next();
            if (key.startsWith(filter)) {
                result.add(key);
            }
        }
        return result;
    }


    private Map file;
    private Trace trace;
}
