/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.storage.raf.log.asynraf;

import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.impl.trace.Trace;

import java.util.List;

public class LogDemon implements Runnable {
    public LogDemon(AsynrafLogger minilog, PhysicalFileAccessLogAsynrafImpl physical) {
        this.minilog = minilog;
        this.physical = physical;
        trace = physical.getTrace();
    }

    public void run() {
        while (!mustClose) {
            try {
                Thread.yield();
                if (isWorking) {
                    List list = minilog.getSortedInfos();
                    if (!list.isEmpty()) {
                        while (!list.isEmpty()) {
                            LogInfo info = (LogInfo) list.remove(0);
                            if ((!list.isEmpty()) && (info.toString().equals(list.get(0).toString()))) {
                                throw new JalistoException("pbm during test");
                            }
                            short type = info.getActionType();
                            if (type == LogInfo.INSERT_TYPE) {
                                byte[] datas = minilog.readBytes(info.getValueAddress(),
                                                                 (int) info.getValueLength());
                                physical.internalInsert(info.getIfa(), datas);
                            } else if (type == LogInfo.UPDATE_TYPE) {
                                byte[] datas = minilog.readBytes(info.getValueAddress(),
                                                                 (int) info.getValueLength());
                                physical.internalUpdateFileObject(info.getIfa(), datas);
                            } else if (type == LogInfo.DELETE_TYPE) {
                                physical.internalDeleteFileObject(info.getIfa());
                            } else {
                                throw new JalistoException("unknow action type : " + type);
                            }
                            minilog.removeInfo(info);
                        }
                        minilog.setLength(2);
                        minilog.writeStatus(AsynrafLogger.NOBODY_WORKING);
                        isWorking = false;
                    } else {
                        if (isWorking) {
                            isWorking = false;
                        }
                    }
                }
            } catch (Exception e) {
                isWorking = false;
                throw new JalistoException(e);
            }
        }
    }

    public boolean isWorking() {
        return isWorking;
    }

    public void setWorking() {
        isWorking = true;
    }

    public void open() {
        mustClose = false;
        demon = new Thread(this);
        demon.start();
    }

    public void close() {
        mustClose = true;
        while (demon.isAlive()) {
            Thread.yield();
        }
        demon = null;
    }

    private PhysicalFileAccessLogAsynrafImpl physical;
    private AsynrafLogger minilog;
    private Trace trace;
    private boolean mustClose = false;
    private boolean isWorking = false;
    private Thread demon;
}
