/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.storage.raf.log.backup;

import org.objectweb.jalisto.se.storage.raf.log.synraf.action.LogRecord;

import java.io.*;

public abstract class LogObject implements Serializable {

    public abstract void writeTo(DataOutput out) throws IOException;

    public abstract Object internalReadFrom(DataInput in) throws IOException;

    public abstract void undoOn(RandomAccessFile raf) throws IOException;

    public abstract void corrupt(RandomAccessFile raf) throws IOException;

    public static Object readFrom(DataInput in) throws IOException {
        short classCode = in.readShort();
        if (classCode == FILELENGTH_SERIAL_CODE) {
            return logFileLengthModification.internalReadFrom(in);
        } else if (classCode == RECORD_SERIAL_CODE) {
            return logRecord.internalReadFrom(in);
        } else if (classCode == BEGIN_SERIAL_CODE) {
            return new LogBoundary(true);
        } else if (classCode == END_SERIAL_CODE) {
            return new LogBoundary(false);
        }
        return null;
    }

    public static void writeLogRecord(DataOutput out, long address, byte[] before, int newDatasLength)
            throws IOException {
        out.writeShort(RECORD_SERIAL_CODE);
        out.writeLong(address);
        out.writeInt(newDatasLength);
        out.writeInt(before.length);
        out.write(before);
    }

    public static void writeLogFileLengthModification(DataOutput out, byte[] deletedBytes)
            throws IOException {
        out.writeShort(FILELENGTH_SERIAL_CODE);
        out.writeBoolean(true);
        out.writeLong(deletedBytes.length);
        out.write(deletedBytes);
    }

    public static void writeLogFileLengthModification(DataOutput out, long sizeModification)
            throws IOException {
        out.writeShort(FILELENGTH_SERIAL_CODE);
        out.writeBoolean(false);
        out.writeLong(sizeModification);
    }

    public static void writeLogBoundary(DataOutput out, boolean isBegin)
            throws IOException {
        if (isBegin) {
            out.writeShort(BEGIN_SERIAL_CODE);
        } else {
            out.writeShort(END_SERIAL_CODE);
        }
    }

    private static LogRecord logRecord = new LogRecord();
    private static LogFileLengthModification logFileLengthModification = new LogFileLengthModification();


    public static final short RECORD_SERIAL_CODE = 1;
    public static final short FILELENGTH_SERIAL_CODE = 2;
    public static final short BEGIN_SERIAL_CODE = 3;
    public static final short END_SERIAL_CODE = 4;

    static final long serialVersionUID = -7699636669241161460L;
}
