/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.test.threads;

import org.objectweb.jalisto.se.test.data.Book;
import org.objectweb.jalisto.se.test.workbench.JalistoTimer;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class BookClientBasic extends BookClientThread {

    public BookClientBasic(String jalistoPropertiesFilename, int numClient,
                           int nbrBook, int nbrAction, int nbrIteration) {
        super(jalistoPropertiesFilename, numClient, nbrBook, nbrAction, nbrIteration);
    }


    public void stabilityTest() {
        System.out.println("BookClientBasic " + numClient + " starts stability test");

        String timerNameCreate = "BookClientBasic - create " + nbrCreate + " - " + numClient;
        String timerNameRead = "BookClientBasic - read " + nbrRead + " - " + numClient;
        String timerNameUpdate = "BookClientBasic - update " + nbrUpdate + " - " + numClient;
        String timerNameDelete = "BookClientBasic - delete " + nbrDelete + " - " + numClient;
        if (withTimer) {
            if (nbrCreate > 0) {
                JalistoTimer.createTimer(timerNameCreate);
            }
            if (nbrDelete > 0) {
                JalistoTimer.createTimer(timerNameDelete);
            }
            if (nbrRead > 0) {
                JalistoTimer.createTimer(timerNameRead);
            }
            if (nbrUpdate > 0) {
                JalistoTimer.createTimer(timerNameUpdate);
            }
        }

        Random random = new Random();

        for (int c = 0; ((nbrIteration < 0) || (c < nbrIteration)) && continueWorking; c++) {
            if (withTimer && (c != 0) && ((c % 50) == 0)) {
                System.out.println("");
                if (nbrCreate > 0) {
                    JalistoTimer.summary(timerNameCreate);
                    JalistoTimer.clean(timerNameCreate);
                }
                if (nbrRead > 0) {
                    JalistoTimer.summary(timerNameRead);
                    JalistoTimer.clean(timerNameRead);
                }
                if (nbrUpdate > 0) {
                    JalistoTimer.summary(timerNameUpdate);
                    JalistoTimer.clean(timerNameUpdate);
                }
                if (nbrDelete > 0) {
                    JalistoTimer.summary(timerNameDelete);
                    JalistoTimer.clean(timerNameDelete);
                }
            }

            if (withExtent) {
                tx.begin();
                oids = new ArrayList();
                oids.addAll(session.getExtent(Book.class).readFully().collection());
                tx.commit();
            }

            if (nbrCreate > 0) {
                if (withTimer) {
                    JalistoTimer.timerStart(timerNameCreate);
                }
                List newOids = new ArrayList();
                tx.begin();
                for (int i = 0; i < nbrCreate; i++) {
                    Object[] bookInArray = Book.newBook().toArray();
                    Object oid = session.createObject(bookInArray, Book.class);
                    newOids.add(oid);
                }
                tx.commit();
                if (withTimer) {
                    JalistoTimer.timerStop(timerNameCreate, false);
                }
                synchronized (oids) {
                    oids.addAll(newOids);
                }
            }

            if (nbrRead > 0) {
                if (withTimer) {
                    JalistoTimer.timerStart(timerNameRead);
                }
                tx.begin();
                Object oid;
                for (int i = 0; i < nbrRead; i++) {
                    synchronized (oids) {
                        int index = random.nextInt(oids.size());
                        oid = oids.get(index);
                        session.readObjectByOid(oid);
                    }
                }
                tx.commit();
                if (withTimer) {
                    JalistoTimer.timerStop(timerNameRead, false);
                }
            }

            if (nbrUpdate > 0) {
                if (withTimer) {
                    JalistoTimer.timerStart(timerNameUpdate);
                }
                tx.begin();
                Object oid;
                for (int i = 0; i < nbrUpdate; i++) {
                    synchronized (oids) {
                        int index = random.nextInt(oids.size());
                        oid = oids.get(index);
                        Object[] book = session.readObjectByOid(oid);
                        book[0] = book[0] + "u_" + numClient;
                        session.updateObjectByOid(oid, book);
                    }
                }
                tx.commit();
                if (withTimer) {
                    JalistoTimer.timerStop(timerNameUpdate, false);
                }
            }

            if (nbrDelete > 0) {
                if (withTimer) {
                    JalistoTimer.timerStart(timerNameDelete);
                }
                tx.begin();
                Object oid;
                for (int i = 0; i < nbrDelete; i++) {
                    synchronized (oids) {
                        int index = random.nextInt(oids.size());
                        oid = oids.remove(index);
                    }
                    session.deleteObjectByOid(oid);
                }
                tx.commit();
                if (withTimer) {
                    JalistoTimer.timerStop(timerNameDelete, false);
                }
            }
        }

        System.out.println("BookClientBasic " + numClient + " ends stability test");
    }


}
