/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.test.workbench;

import java.io.PrintStream;
import java.util.*;

public class JalistoTimer {
    public JalistoTimer(String name) {
        this.name = name;
        this.sum = 0;
        this.count = 0;
        out = System.out;
        mask = new BitSet(NEWLINE + 1);
        mask.set(0, NEWLINE + 1, true);
        verbose = true;
    }

    public void start() {
        start = System.currentTimeMillis();
    }

    public void stop(String name, boolean show) {
        long end = System.currentTimeMillis();
        long result = (end - start);
        if (show) {
            out.println(name + " elapsed time : " + result + " ms");
        }
        sum += result;
        count++;
    }

    public float getAverage() {
        return sum / count;
    }

    private String getSummary() {
        StringBuffer sb = new StringBuffer();
        if (mask.get(NAME)) {
            if (verbose) {
                sb.append("TIMER:");
            }
            sb.append(name).append(";");
        }
        if (mask.get(TIME)) {
            if (verbose) {
                sb.append("TIME:");
            }
            sb.append(System.currentTimeMillis()).append(";");
        }
        if (mask.get(SUM)) {
            if (verbose) {
                sb.append("SUM:");
            }
            sb.append(sum).append(";");
        }
        if (mask.get(COUNT)) {
            if (verbose) {
                sb.append("COUNT:");
            }
            sb.append(count).append(";");
        }
        if (mask.get(AVG) && (count != 0)) {
            if (verbose) {
                sb.append("AVG:");
            }
            sb.append(sum / count).append(";");
        }
        if (mask.get(NEWLINE)) {
            sb.append("\n");
        }
        return sb.toString();
    }

    private void clean() {
        this.sum = 0;
        this.count = 0;
    }


    /**
     * ****************************    STATIC    *****************************************
     */

    public static void createTimer(String name) {
        JalistoTimer timer = (JalistoTimer) timers.get(name);
        if (timer != null) {
            throw new UnsupportedOperationException("Create over an existing timer");
        }
        timer = new JalistoTimer(name);
        timers.put(name, timer);
        timerNames.add(name);
    }

    public static void removeTimer(String name) {
        timers.remove(name);
        timerNames.remove(name);
    }

    public static void timerStart(String name) {
        JalistoTimer timer = (JalistoTimer) timers.get(name);
        if (timer == null) {
            timer = new JalistoTimer(name);
            timers.put(name, timer);
            timerNames.add(name);
        }
        timer.start();
    }

    public static void timerStop(String name) {
        timerStop(name, true);
    }

    public static void timerStop(String name, boolean show) {
        JalistoTimer timer = (JalistoTimer) timers.get(name);
        if (timer == null) {
            throw new UnsupportedOperationException("Unable to stop an unexisting timer");
        }
        timer.stop(name, show);
    }

    public static void setOut(String name, PrintStream out) {
        JalistoTimer timer = (JalistoTimer) timers.get(name);
        if (timer == null) {
            throw new UnsupportedOperationException("Unable to set Out of an unexisting timer");
        }
        timer.out = out;
    }

    public static void setFormat(String name, int format) {
        JalistoTimer timer = (JalistoTimer) timers.get(name);
        if (timer == null) {
            throw new UnsupportedOperationException("Unable to set format of an unexisting timer");
        }
        if (timer.initialValues) {
            timer.mask.clear();
            timer.verbose = false;
            timer.initialValues = false;
        }
        timer.mask.set(format);
    }

    public static void unsetFormat(String name, int format) {
        JalistoTimer timer = (JalistoTimer) timers.get(name);
        if (timer == null) {
            throw new UnsupportedOperationException("Unable to unset format of an unexisting timer");
        }
        timer.mask.clear(format);
    }

    public static void setVerbose(String name, boolean verbose) {
        JalistoTimer timer = (JalistoTimer) timers.get(name);
        if (timer == null) {
            throw new UnsupportedOperationException("Unable to set verbose value of an unexisting timer");
        }
        if (timer.initialValues) {
            timer.mask.clear();
            timer.initialValues = false;
        }
        timer.verbose = verbose;
    }

    public static void summary() {
        for (int i = 0; i < timerNames.size(); i++) {
            JalistoTimer timer = (JalistoTimer) timers.get(timerNames.get(i));
            timer.out.print(timer.getSummary());
        }
    }

    public static void summary(String name) {
        JalistoTimer timer = (JalistoTimer) timers.get(name);
        if (timer == null) {
            throw new UnsupportedOperationException("Unable to print summary for an unexisting timer");
        }
        timer.out.print(timer.getSummary());
    }

    public static void clean(String name) {
        JalistoTimer timer = (JalistoTimer) timers.get(name);
        if (timer == null) {
            throw new UnsupportedOperationException("Unable to cleanAll an unexisting timer");
        }
        timer.clean();
    }

    public static void deleteAllTimers() {
        timers.clear();
        timerNames.clear();
    }

    public static float getAverage(String name) {
        JalistoTimer timer = (JalistoTimer) timers.get(name);
        if (timer == null) {
            throw new UnsupportedOperationException("Unable to get average on an unexisting timer");
        }
        return timer.getAverage();
    }


    private String name;
    private long start;
    private long sum;
    private long count;
    private PrintStream out;
    private BitSet mask;
    private boolean verbose;
    private boolean initialValues = true;


    private static Map timers = new HashMap();
    private static ArrayList timerNames = new ArrayList();

    public static final int TIME = 0;
    public static final int NAME = 1;
    public static final int SUM = 2;
    public static final int COUNT = 3;
    public static final int AVG = 4;
    public static final int NEWLINE = 5;
}
