/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.test.core.suite;

import junit.framework.Test;
import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.api.MetaRepository;
import org.objectweb.jalisto.se.api.query.FieldDescription;
import org.objectweb.jalisto.se.exception.SchemaException;
import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.impl.meta.InternalMetaRepositoryImpl;
import org.objectweb.jalisto.se.impl.meta.type.IntegerType;
import org.objectweb.jalisto.se.impl.meta.type.StringType;
import org.objectweb.jalisto.se.test.data.Book;
import org.objectweb.jalisto.se.test.workbench.JalistoTestCase;
import org.objectweb.jalisto.se.test.workbench.JalistoTestSuite;
import org.objectweb.jalisto.se.tool.JalistoViewer;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Random;

public class MetaRepositoryTestCase extends JalistoTestCase {

    public MetaRepositoryTestCase() {
        init();
    }

    public MetaRepositoryTestCase(String name) {
        super(name);
        init();
    }

    private void init() {
        meta0 = Book.getMetaDescription();
        meta1 = JalistoFactory.createClassDescription(Book.class.getName());
        meta2 = JalistoFactory.createClassDescription(Book.class.getName());

        meta1.addField(JalistoFactory.createFieldDescription("title", new StringType()));
        meta1.addField(JalistoFactory.createFieldDescription("comments", new StringType()));
        meta1.addField(JalistoFactory.createFieldDescription("pages", new IntegerType()));
        meta1.addField(JalistoFactory.createFieldDescription("price", new IntegerType()));
        meta1.addField(JalistoFactory.createFieldDescription("author", new StringType()));

        meta2.addField(JalistoFactory.createFieldDescription("title", new StringType()));
        meta2.addField(JalistoFactory.createFieldDescription("comments", new StringType()));
        meta2.addField(JalistoFactory.createFieldDescription("price", new IntegerType()));
        meta2.addField(JalistoFactory.createFieldDescription("author", new StringType()));
    }

    public static Test suite() {
        JalistoTestSuite suite = new JalistoTestSuite();
        MetaRepositoryTestCase tc = (MetaRepositoryTestCase) newTestCase(suite, new MetaRepositoryTestCase());

        tc.define();

        tc.populateTest(5000);

        tc.modifySchema();

        tc.modifySchemaDuringTransaction();

        tc.removePersistentClass();

        tc.removeUnknownClass();

        tc.defineAgain();
        tc.defineAgainWithNotSameMeta();

        tc.erase();
        tc.finishTests();

        return suite;
    }

    /**
     * **************************************************************************************
     */

    public void populateTest(int nbrBook) {
        super.populate(nbrBook);

        tx.begin();
        ArrayList list = new ArrayList(session.getExtent(Book.class).readFully().collection());
        tx.commit();
        int index = (new Random()).nextInt(list.size());
        oid = (LogicalOid) list.get(index);
    }

    public void modifySchema() {
        if (session.getInternalSession().isRemoteSession()) {
            System.out.println("---> can't execute this test with remote client");
            return;
        }

        Object[] o;
        MetaRepository metaRepository = session.getMetaRepository();
        String bookClassName = Book.class.getName();

        ClassDescription meta = metaRepository.getMetaData(bookClassName);
        assertTrue("must be equal", meta.equals(meta0));

        FieldDescription fieldDescription = JalistoFactory.createFieldDescription("comments", new StringType());
        fieldDescription.setIndex((short) 1);
        metaRepository.addField(session, bookClassName, fieldDescription);
        meta = metaRepository.getMetaData(bookClassName);
        assertTrue("must be equal", meta.equals(meta1));

        tx.begin();
        o = session.readObjectByOid(oid);
        assertNotNull(o[0]);
        assertNull(o[1]);
        assertNotNull(o[2]);
        assertNotNull(o[3]);
        assertNotNull(o[4]);
        tx.commit();

        metaRepository.removeField(session, bookClassName, "pages");
        meta = metaRepository.getMetaData(bookClassName);
        assertTrue("must be equal", meta.equals(meta2));

        tx.begin();
        o = session.readObjectByOid(oid);
        assertNotNull(o[0]);
        assertNull(o[1]);
        assertNotNull(o[2]);
        assertNotNull(o[3]);
        try {
            System.out.println(o[4]);
            assertTrue("Should raise exception", false);
        } catch (Exception e) {
            // normal behavior
        }
        tx.commit();
    }

    public void modifySchemaDuringTransaction() {
        if (session.getInternalSession().isRemoteSession()) {
            System.out.println("---> can't execute this test with remote client");
            return;
        }

        try {
            tx.begin();
            MetaRepository metaRepository = session.getMetaRepository();
            FieldDescription fieldDescription = JalistoFactory.createFieldDescription("author", new StringType());
            fieldDescription.setIndex((short) 1);
            metaRepository.addField(session, Book.class.getName(), fieldDescription);
            assertTrue("exception expected", false);
        } catch (SchemaException e) {
            assertTrue("not valide exception message",
                       e.getMessage().startsWith(InternalMetaRepositoryImpl.MODIFY_DURING_TRANSACTION_MESSAGE));
        } finally {
            tx.commit();
        }
    }

    public void removePersistentClass() {
        try {
            assertTrue(session.isClassDefined(Book.getMetaDescription().getClassName()));

            session.removeClass(Book.getMetaDescription().getClassName());

            tx.begin();
            try {
                session.createObject(Book.newBook().toArray(), Book.class);
            } catch (SchemaException jalistoSchemaExc) {
                assertEquals("Should be equals",
                             "Class org.objectweb.jalisto.se.test.data.Book is not define in base",
                             jalistoSchemaExc.getMessage());
            }
        } finally {
            if (tx.isActive()) {
                tx.commit();
            }
        }
    }

    public void removeUnknownClass() {
        try {
            session.removeClass(MetaRepositoryTestCase.class.getName());
            assertTrue("should raise exception", false);
        } catch (SchemaException jalistoSchemaExc) {
            assertEquals("Should be equals",
                         "Class org.objectweb.jalisto.test.core.suite.MetaRepositoryTestCase is not define in base",
                         jalistoSchemaExc.getMessage());
        }
    }

    public void defineAgain() {
        session.defineClass(Book.getMetaDescription());
        tx.begin();
        session.createObject(Book.newBook().toArray(), Book.class);
        tx.commit();
    }

    public void defineAgainWithNotSameMeta() {
        try {
            session.defineClass(meta2);
        } catch (SchemaException jalistoSchemaExc) {
            assertEquals("Should be equals",
                         "Class org.objectweb.jalisto.se.test.data.Book is already define in base with other metadescription",
                         jalistoSchemaExc.getMessage());
        }
    }

    public void showBase() {
        String args[] = {"-m", "-file", "jalisto.properties"};
        JalistoViewer viewer = new JalistoViewer(session, args, new PrintWriter(System.out));
        viewer.readBase();
    }

    public void erase() {
        endTransaction();
        closeSession();
        if (session.getInternalSession().getProperties().allowsSpecialFunctionnalities()) {
            session.eraseStorage();
        } else {
            System.out.println("not allows reorganize functionnality");
        }
    }


    private LogicalOid oid;

    private ClassDescription meta0;
    private ClassDescription meta1;
    private ClassDescription meta2;

    /**
     * ***********************************************************************************
     */

    public void define() {
        super.initSession(true);
        super.define(Book.getMetaDescription());
    }


    public void finishTests() {
        super.finishTests();
    }
}
