/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.test.core.suite;

import junit.framework.Test;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.exception.IdentityException;
import org.objectweb.jalisto.se.exception.TransactionException;
import org.objectweb.jalisto.se.api.*;
import org.objectweb.jalisto.se.test.data.Book;
import org.objectweb.jalisto.se.test.workbench.JalistoTestCase;
import org.objectweb.jalisto.se.test.workbench.JalistoTestSuite;
import org.objectweb.jalisto.se.JalistoFactory;

import java.util.ArrayList;
import java.util.Iterator;

public class MultiOptimisticTestCase extends JalistoTestCase {
    public MultiOptimisticTestCase() {
    }

    public MultiOptimisticTestCase(String name) {
        super(name);
    }

    public static Test suite() {
        JalistoTestSuite suite = new JalistoTestSuite();
        MultiOptimisticTestCase tc = (MultiOptimisticTestCase) newTestCase(suite, new MultiOptimisticTestCase());

        tc.define(Book.getMetaDescription());

        tc.createSessions(5);

        tc.createTest();
        tc.updateTest();
        tc.deleteTest();
        tc.extentTest(20);

        tc.refreshTest();

        tc.testMakeOidRollback01();
        tc.testMakeOidRollback02();

        tc.restructureMultiTest();

        return suite;
    }

    public void createSessions(int nbr) {
        if (!canRun) {
            System.out.println("can't execute this test with implementation other than 'multi'");
            return;
        }

        ArrayList sessions = new ArrayList();
        JalistoProperties properties = JalistoFactory.getInternalFactory().getProperties(getJalistoPropertiesFilename());
        overrideJalistoProperties(properties);
        for (int i = 0; i < nbr; i++) {
            Session s = JalistoFactory.getSession(getJalistoPropertiesFilename());
            s.openSession();
            s.currentTransaction().begin();
            s.currentTransaction().commit();
            sessions.add(s);
        }
        for (int i = 0; i < nbr; i++) {
            Session s = (Session) sessions.get(i);
            s.closeSession();
        }
    }

    public void restructureMultiTest() {
        if (!canRun) {
            System.out.println("can't execute this test with implementation other than 'multi'");
            return;
        }

        JalistoProperties properties = JalistoFactory.getInternalFactory().getProperties(getJalistoPropertiesFilename());
        overrideJalistoProperties(properties);
        Session s1 = JalistoFactory.getSession(getJalistoPropertiesFilename());
        Session s2 = JalistoFactory.getSession(getJalistoPropertiesFilename());
        s1.openSession();
        s1.currentTransaction().begin();
        s2.openSession();
        s2.currentTransaction().begin();
        s1.currentTransaction().commit();
        s1.closeSession();
        try {
            s1.reorganize();
            assertTrue("must raise an exception", true);
        } catch (JalistoException e) {
            assertTrue("must begin with", e.getMessage().startsWith(
                    "There are active sessions during attempt to reorganize datastore"));
        }
        s2.currentTransaction().commit();
        try {
            s1.reorganize();
            assertTrue("must raise an exception", true);
        } catch (JalistoException e) {
            assertTrue("must begin with", e.getMessage().startsWith(
                    "There are active sessions during attempt to reorganize datastore"));
        }
        s2.closeSession();
        s1.reorganize();
    }

    public void createTest() {
        if (!canRun) {
            System.out.println("can't execute this test with implementation other than 'multi'");
            return;
        }

        JalistoProperties properties = JalistoFactory.getInternalFactory().getProperties(getJalistoPropertiesFilename());
        overrideJalistoProperties(properties);
        Session session1 = JalistoFactory.getSession(getJalistoPropertiesFilename());
        session1.openSession();
        Transaction tx1 = session1.currentTransaction();
        Session session2 = JalistoFactory.getSession(getJalistoPropertiesFilename());
        session2.openSession();
        Transaction tx2 = session2.currentTransaction();

        {
            tx1.begin();
            tx2.begin();
            Object oid = session1.createObject(Book.newBook().toArray(), Book.class);
            try {
                session2.readObjectByOid(oid);
                assertTrue("should raise a IdentityException", true);
            } catch (IdentityException e) {
            }
            tx1.commit();
            session2.readObjectByOid(oid);
            tx2.commit();
        }

        {
            tx1.begin();
            tx2.begin();
            Object oid = session1.createObject(Book.newBook().toArray(), Book.class);
            try {
                session2.readObjectByOid(oid);
                assertTrue("should raise a IdentityException", true);
            } catch (IdentityException e) {
            }
            tx1.rollback();
            try {
                session2.readObjectByOid(oid);
                assertTrue("should raise a IdentityException", true);
            } catch (IdentityException e) {
            }
            tx2.commit();
        }

        session1.closeSession();
        session2.closeSession();
    }

    public void updateTest() {
        if (!canRun) {
            System.out.println("can't execute this test with implementation other than 'multi'");
            return;
        }

        JalistoProperties properties = JalistoFactory.getInternalFactory().getProperties(getJalistoPropertiesFilename());
        overrideJalistoProperties(properties);
        Session session1 = JalistoFactory.getSession(getJalistoPropertiesFilename());
        session1.openSession();
        Transaction tx1 = session1.currentTransaction();
        Session session2 = JalistoFactory.getSession(getJalistoPropertiesFilename());
        session2.openSession();
        Transaction tx2 = session2.currentTransaction();

        tx1.begin();
        Object oid = session1.createObject(Book.newBook().toArray(), Book.class);
        tx1.commit();

        {
            tx1.begin();
            tx2.begin();
            Object[] o1 = session1.readObjectByOid(oid);
            Object[] o2 = session2.readObjectByOid(oid);
            assertTrue("must not be identical", o1 != o2);
            assertTrue("must be equals", equals(o1, o2));
            o1[0] = o1[0] + " (update 01 by s1)";
            session1.updateObjectByOid(oid, o1);
            tx1.commit();
            Object[] o3 = session2.readObjectByOid(oid);
            assertTrue("must not be identical", o2 != o3);
            assertTrue("must be equals", equals(o2, o3));
            tx2.commit();
            tx2.begin();
            Object[] o4 = session2.readObjectByOid(oid);
            assertTrue("must be equals", equals(o1, o4));
            tx2.commit();
        }

        {
            tx1.begin();
            Object[] o1 = session1.readObjectByOid(oid);
            o1[0] = o1[0] + " (update 02 by s1)";
            session1.updateObjectByOid(oid, o1);
            tx2.begin();
            Object[] o2 = session2.readObjectByOid(oid);
            assertTrue("must not be identical", o1 != o2);
            assertTrue("must not be equals", !equals(o1, o2));
            tx1.commit();
            tx2.commit();
        }

        {
            tx1.begin();
            tx2.begin();
            Object[] o1 = session1.readObjectByOid(oid);
            Object[] o2 = session2.readObjectByOid(oid);
            assertTrue("must not be identical", o1 != o2);
            assertTrue("must be equals", equals(o1, o2));
            o1[0] = o1[0] + " (update 03 by s1)";
            session1.updateObjectByOid(oid, o1);
            tx1.commit();
            o2[0] = o2[0] + " (update 01 by s2)";
            session2.updateObjectByOid(oid, o2);
            try {
                tx2.commit();
                assertTrue("must raise optimistic exception", false);
            } catch (TransactionException jalistoTransactionExc) {
                assertTrue("must be a optimistic/pessimistic exception",
                           jalistoTransactionExc.isOptimisticException() || jalistoTransactionExc.isPessimisticException());
            } finally {
                if (tx2.isActive()) {
                    tx2.rollback();
                }
            }
        }

        {
            tx1.begin();
            tx2.begin();
            Object[] o1 = session1.readObjectByOid(oid);
            Object[] o2 = session2.readObjectByOid(oid);
            assertTrue("must not be identical", o1 != o2);
            assertTrue("must be equals", equals(o1, o2));
            o1[0] = o1[0] + " (update 04 by s1)";
            session1.updateObjectByOid(oid, o1);
            tx1.commit();
            Object[] o3 = session2.readObjectByOid(oid);          // with read before update
            o3[0] = o3[0] + " (update 02 by s2)";
            session2.updateObjectByOid(oid, o3);
            try {
                tx2.commit();
                assertTrue("must raise optimistic exception", false);
            } catch (TransactionException jalistoTransactionExc) {
                assertTrue("must be a optimistic/pessimistic exception",
                           jalistoTransactionExc.isOptimisticException() || jalistoTransactionExc.isPessimisticException());
            } finally {
                if (tx2.isActive()) {
                    tx2.rollback();
                }
            }
        }

        session1.closeSession();
        session2.closeSession();
    }

    public void deleteTest() {
        if (!canRun) {
            System.out.println("can't execute this test with implementation other than 'multi'");
            return;
        }

        JalistoProperties properties = JalistoFactory.getInternalFactory().getProperties(getJalistoPropertiesFilename());
        overrideJalistoProperties(properties);
        Session session1 = JalistoFactory.getSession(getJalistoPropertiesFilename());
        session1.openSession();
        Transaction tx1 = session1.currentTransaction();
        Session session2 = JalistoFactory.getSession(getJalistoPropertiesFilename());
        session2.openSession();
        Transaction tx2 = session2.currentTransaction();

        tx1.begin();
        Object oid = session1.createObject(Book.newBook().toArray(), Book.class);
        tx1.commit();

        {
            tx1.begin();
            tx2.begin();
            session1.deleteObjectByOid(oid);
            Object[] o2 = session2.readObjectByOid(oid);
            assertNotNull(o2);
            try {
                session1.readObjectByOid(oid);
                assertTrue("should raise exception", true);
            } catch (IdentityException e) {
            }
            tx1.rollback();
            tx1.begin();
            Object[] o1 = session1.readObjectByOid(oid);
            assertNotNull(o1);
            tx1.commit();
            tx2.commit();
        }

        session1.closeSession();
        session2.closeSession();
    }

    public void extentTest(int number) {
        if (!canRun) {
            System.out.println("can't execute this test with implementation other than 'multi'");
            return;
        }

        JalistoProperties properties = JalistoFactory.getInternalFactory().getProperties(getJalistoPropertiesFilename());
        overrideJalistoProperties(properties);
        Session session1 = JalistoFactory.getSession(getJalistoPropertiesFilename());
        session1.openSession();
        Transaction tx1 = session1.currentTransaction();
        Session session2 = JalistoFactory.getSession(getJalistoPropertiesFilename());
        session2.openSession();
        Transaction tx2 = session2.currentTransaction();
        ArrayList oids = new ArrayList();

        {
            tx1.begin();
            Iterator extent = session1.getExtent(Book.class).readFully().iterator();
            while(extent.hasNext()) {
                session1.deleteObjectByOid(extent.next());
            }
            for (int i = 0; i < number; i++) {
                oids.add(session1.createObject(Book.newBook().toArray(), Book.class));
            }
            tx1.commit();
        }

        {
            tx1.begin();
            tx2.begin();
            Object createdOid = session1.createObject(Book.newBook().toArray(), Book.class);
            Object removedOid = oids.get(0);
            session2.deleteObjectByOid(removedOid);
            {
                Extent extent1 = session1.getExtent(Book.class).readFully();
                assertEquals("must be equals", number + 1, extent1.size());
                assertTrue("must contains", extent1.contains(createdOid));
                assertTrue("must contains", extent1.contains(removedOid));

                Extent extent2 = session2.getExtent(Book.class).readFully();
                assertEquals("must be equals", number - 1, extent2.size());
                assertTrue("must not contains", ! extent2.contains(createdOid));
                assertTrue("must not contains", ! extent2.contains(removedOid));
            }
            tx1.commit();
            tx1.begin();
            {
                Extent extent1 = session1.getExtent(Book.class).readFully();
                assertEquals("must be equals", number + 1, extent1.size());
                assertTrue("must contains", extent1.contains(createdOid));
                assertTrue("must contains", extent1.contains(removedOid));

                Extent extent2 = session2.getExtent(Book.class).readFully();
                assertEquals("must be equals", number, extent2.size());
                assertTrue("must contains", extent2.contains(createdOid));
                assertTrue("must not contains", ! extent2.contains(removedOid));
            }
            tx2.commit();
            tx2.begin();
            {
                Extent extent1 = session1.getExtent(Book.class).readFully();
                assertEquals("must be equals", number, extent1.size());
                assertTrue("must contains", extent1.contains(createdOid));
                assertTrue("must not contains", ! extent1.contains(removedOid));

                Extent extent2 = session2.getExtent(Book.class).readFully();
                assertEquals("must be equals", number, extent2.size());
                assertTrue("must contains", extent2.contains(createdOid));
                assertTrue("must not contains", ! extent2.contains(removedOid));
            }
            tx1.commit();
            tx2.commit();
        }

        session1.closeSession();
        session2.closeSession();
    }

    public void refreshTest() {
        if (!canRun) {
            System.out.println("can't execute this test with implementation other than 'multi'");
            return;
        }

        Session session1 = JalistoFactory.getSession(getJalistoPropertiesFilename());
        session1.openSession();
        Transaction tx1 = session1.currentTransaction();
        Session session2 = JalistoFactory.getSession(getJalistoPropertiesFilename());
        session2.openSession();
        Transaction tx2 = session2.currentTransaction();

        tx1.begin();
        Object oid = session1.createObject(Book.newBook().toArray(), Book.class);
        tx1.commit();

        {
            tx1.begin();
            tx2.begin();
            Object[] o11 = session1.readObjectByOid(oid);
            Object[] o21 = session2.readObjectByOid(oid);
            assertNotNull(o11);
            assertNotNull(o21);
            assertTrue("should not be same reference", o11 != o21);
            String title = (String) o11[0];
            assertEquals("must be equals", title, o21[0]);
            o11[0] = title + "u1";
            session1.updateObjectByOid(oid, o11);
            tx1.commit();
            tx1.begin();
            Object[] o12 = session1.readObjectByOid(oid);
            assertTrue("should not be same reference", o11 != o12);
            assertEquals("must be equals", title+"u1", o12[0]);
            Object[] o22 = session2.readObjectByOid(oid);
            assertEquals("must be equals", title, o22[0]);
            Object[] o23 = session2.refreshObjectByOid(oid);
            assertTrue("should not be same reference", o21 != o23);
            assertEquals("must be equals", title+"u1", o23[0]);
            Object[] o24 = session2.readObjectByOid(oid);
            assertEquals("must be equals", title+"u1", o24[0]);
            tx1.commit();
            tx2.commit();
        }

        {
            tx1.begin();
            tx2.begin();
            Object[] o11 = session1.readObjectByOid(oid);
            Object[] o21 = session2.readObjectByOid(oid);
            assertNotNull(o11);
            assertNotNull(o21);
            assertTrue("should not be same reference", o11 != o21);
            String title = (String) o11[0];
            assertEquals("must be equals", title, o21[0]);
            o11[0] = title + "u1";
            o21[0] = title + "u2";
            session1.updateObjectByOid(oid, o11);
            session2.updateObjectByOid(oid, o21);
            Object[] o12 = session1.readObjectByOid(oid);
            Object[] o22 = session2.readObjectByOid(oid);
            assertEquals("must be equals", title+"u1", o12[0]);
            assertEquals("must be equals", title+"u2", o22[0]);
            Object[] o13 = session1.refreshObjectByOid(oid);
            assertEquals("must be equals", title, o13[0]);
            o13[0] = title + "u3";
            session1.updateObjectByOid(oid, o13);
            tx1.commit();
            Object[] o23 = session2.readObjectByOid(oid);
            assertEquals("must be equals", title+"u2", o23[0]);
            Object[] o24 = session2.refreshObjectByOid(oid);
            assertEquals("must be equals", title+"u3", o24[0]);
            tx2.commit();
        }

        {
            tx1.begin();
            tx2.begin();
            Object[] o11 = session1.readObjectByOid(oid);
            Object[] o21 = session2.readObjectByOid(oid);
            assertNotNull(o11);
            assertNotNull(o21);
            assertTrue("should not be same reference", o11 != o21);
            session1.deleteObjectByOid(oid);
            try {
                session1.refreshObjectByOid(oid);
                assertTrue("should raise exception", false);
            } catch (IdentityException fie) {
            }
            tx1.commit();
            tx2.commit();
        }

        session1.closeSession();
        session2.closeSession();
    }

    public void testMakeOidRollback01() {
        if (!canRun) {
            System.out.println("can't execute this test with implementation other than 'multi'");
            return;
        }

        Session session1 = JalistoFactory.getSession(getJalistoPropertiesFilename());
        session1.openSession();
        Transaction tx1 = session1.currentTransaction();
        Session session2 = JalistoFactory.getSession(getJalistoPropertiesFilename());
        session2.openSession();
        Transaction tx2 = session2.currentTransaction();

        int nbr = session1.getInternalSession().getProperties().getOidPageSize();
        ArrayList oids1 = new ArrayList();

        try {
            tx1.begin();
            tx2.begin();
            for (int i = 0; i < nbr*5; i++) {
                oids1.add(session1.makeNewFileOid(Book.class));
            }
            tx1.commit();
            tx2.commit();

            tx2.begin();
            for (int i = 0; i < oids1.size(); i++) {
                session2.deleteObjectByOid(oids1.get(i));
            }
            tx2.commit();
        } finally {
            if (tx1.isActive()) {
                tx1.rollback();
            }
            if (tx2.isActive()) {
                tx2.rollback();
            }
            session1.closeSession();
            session2.closeSession();
        }
    }

    public void testMakeOidRollback02() {
        if (!canRun) {
            System.out.println("can't execute this test with implementation other than 'multi'");
            return;
        }

        Session session1 = JalistoFactory.getSession(getJalistoPropertiesFilename());
        session1.openSession();
        Transaction tx1 = session1.currentTransaction();
        Session session2 = JalistoFactory.getSession(getJalistoPropertiesFilename());
        session2.openSession();
        Transaction tx2 = session2.currentTransaction();

        int nbr = session1.getInternalSession().getProperties().getOidPageSize();
        ArrayList oids1 = new ArrayList();
        ArrayList oids2 = new ArrayList();

        try {
            tx1.begin();
            tx2.begin();
            for (int i = 0; i < nbr/2; i++) {
                oids1.add(session1.makeNewFileOid(Book.class));
                oids2.add(session2.makeNewFileOid(Book.class));
            }
            tx1.commit();
            tx1.begin();
            tx2.rollback();
            Object[] o = Book.newBook().toArray();
            for (int i = 0; i < oids2.size(); i++) {
                Object oid = oids2.get(i);
                try {
                    session1.createObject(oid, o);
                    assertTrue("must raise exception", false);
                } catch (IdentityException jalistoIdentityExc) {
                    String message = "the given oid "+oid+" doesn't exist in this Jalisto datastore";
                    assertEquals("must be equals", message, jalistoIdentityExc.getMessage());
                }
            }
            tx1.rollback();
            tx1.begin();
            for (int i = 0; i < nbr*4; i++) {
                oids1.add(session1.makeNewFileOid(Book.class));
            }
            tx1.commit();

            tx2.begin();
            for (int i = 0; i < oids1.size(); i++) {
                session2.deleteObjectByOid(oids1.get(i));
            }
            tx2.commit();
        } finally {
            if (tx1.isActive()) {
                tx1.rollback();
            }
            if (tx2.isActive()) {
                tx2.rollback();
            }
            session1.closeSession();
            session2.closeSession();
        }
    }

    /**
     * **************************************************************************************
     */

    private boolean equals(Object[] o1, Object[] o2) {
        for (short i = 0; i < o1.length; i++) {
            if (i >= o2.length) {
                return false;
            }
            if (!o1[i].equals(o2[i])) {
                return false;
            }
        }
        return true;
    }

    public void overrideJalistoProperties(JalistoProperties jalistoProperties) {
        jalistoProperties.setProperty(JalistoProperties.CONCURRENCY_MODE_KEY, "optimistic");
    }

    public void define(ClassDescription classDescription) {
        JalistoProperties properties = JalistoFactory.getInternalFactory().getProperties(getJalistoPropertiesFilename());
        if (!properties.isMultiImplementation()) {
            canRun = false;
            return;
        }
        overrideJalistoProperties(properties);
        Session s = JalistoFactory.getSession(getJalistoPropertiesFilename());
        s.openSession();
        s.defineClass(classDescription);
        s.closeSession();
    }

    public void finishTests() {
        super.finishTests();
    }


    private boolean canRun = true;
}
