/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.test.core.suite;

import junit.framework.Test;
import org.objectweb.jalisto.se.test.data.Book;
import org.objectweb.jalisto.se.test.workbench.JalistoTestCase;
import org.objectweb.jalisto.se.test.workbench.JalistoTestSuite;
import org.objectweb.jalisto.se.api.internal.SessionInternal;
import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.impl.meta.type.MetaTypeFactory;

public class PopulateTestCase extends JalistoTestCase {
    public PopulateTestCase() {
    }

    public PopulateTestCase(String name) {
        super(name);
    }

    public static Test suite() {
        JalistoTestSuite suite = new JalistoTestSuite();
        PopulateTestCase tc = (PopulateTestCase) newTestCase(suite, new PopulateTestCase());

        tc.define();
        tc.cleanUp();

        tc.populate(500);     // should be 500 for the test suite validity

        tc.eraseTest();

//        tc.overheadFloids();

        tc.finishTests();

        return suite;
    }

    public void eraseTest() {
        session.closeSession();
        session.eraseStorage();
        SessionInternal internal = (SessionInternal) session;
        long[] floids = internal.getOidTable().getIdentityProvider().getFloidCounters();
        for (int i = 0; i < floids.length; i++) {
            assertEquals("must be null", 0, floids[i]);
        }
        short clid = internal.getOidTable().getIdentityProvider().getClidCounter();
        assertEquals("must be empty", -1, clid);
        session.openSession();
    }

    public void overheadFloids() {
        boolean continueTest = true;
        String className = "java.Vomir";
        Object[] data = new Object[2];
        data[0] = "tralala";
        data[1] = new Integer(465);
        int c = 0;
        while(continueTest) {
            try {
                ClassDescription meta = JalistoFactory.createClassDescription(className+c);
                meta.addField(JalistoFactory.createFieldDescription("title", MetaTypeFactory.StringType));
                meta.addField(JalistoFactory.createFieldDescription("pages", MetaTypeFactory.IntegerType));
                session.defineClass(meta);
                tx.begin();
                LogicalOid floid = (LogicalOid) session.createObject(data, className+c);
                assertEquals("must be equals", c, floid.getClid());
                assertEquals("must be equals", 0, floid.getInstanceNumber());
                assertTrue("must be > 0, c = "+c, floid.getFloidAsLong() >= 0);
                tx.commit();
            } catch (Exception e) {
                continueTest = false;
                System.out.println("c = "+c);
                System.out.println(e.getMessage());
            }
            if ((c % 100) == 0) {
                System.out.println("c = "+c);
            }
            c++;
        }
    }

    /**
     * ***********************************************************************************
     */

    public void define() {
        super.initSession(false);
        super.define(Book.getMetaDescription());
    }

    public void cleanUp() {
        super.cleanUp(Book.class);
    }

    public void populate(int nbr) {
        super.populate(nbr);
    }

    public void finishTests() {
        super.finishTests();
    }
}
