/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.test.core.suite;

import junit.framework.Test;
import org.objectweb.jalisto.se.test.data.Book;
import org.objectweb.jalisto.se.test.workbench.JalistoTestCase;
import org.objectweb.jalisto.se.test.workbench.JalistoTestSuite;
import org.objectweb.jalisto.se.test.workbench.JalistoTimer;

import java.util.ArrayList;
import java.util.Random;

public class StabilityTestCase extends JalistoTestCase {

    public StabilityTestCase() {
    }

    public StabilityTestCase(String name) {
        super(name);
    }

    public static Test suite() {
        JalistoTestSuite suite = new JalistoTestSuite();
        StabilityTestCase tc = (StabilityTestCase) newTestCase(suite, new StabilityTestCase());

        tc.define();
        tc.cleanUp();
        tc.populate(5000);
        tc.stabilityTest(200, 10000);
        tc.cleanUp();

        return suite;
    }

    /**
     * **************************************************************************************
     */

    public void populate(int nbr) {
        tx.begin();
        for (int i = 0; i < nbr; i++) {
            oids.add(session.createObject(Book.newBook().toArray(), Book.class));
        }
        tx.commit();
    }

    public void stabilityTest(int nbrCreate, int nbrIt) {
        Random random = new Random();
        String timerName = "Stability test execution cycle";
        JalistoTimer.createTimer(timerName);

        for (int c = 0; c < nbrIt; c++) {
            if ((c != 0) && ((c % 50) == 0)) {
                session.reorganize();
                JalistoTimer.summary();
                JalistoTimer.clean(timerName);
            }

            JalistoTimer.timerStart(timerName);
            tx.begin();
            for (int i = 0; i < nbrCreate; i++) {
                Object[] bookInArray = Book.newBook().toArray();
                Object oid = session.createObject(bookInArray, Book.class);
                oids.add(oid);
            }
            tx.commit();

            tx.begin();
            for (int i = 0; i < nbrCreate; i++) {
                int index = random.nextInt(oids.size());
                Object oid = oids.get(index);
                session.readObjectByOid(oid);
            }
            tx.commit();

            tx.begin();
            for (int i = 0; i < nbrCreate; i++) {
                int index = random.nextInt(oids.size());
                Object oid = oids.get(index);
                session.deleteObjectByOid(oid);
                oids.remove(index);
            }
            tx.commit();
            JalistoTimer.timerStop(timerName, false);
        }
    }


    private ArrayList oids = null;

    /**
     * ***********************************************************************************
     */

    public void define() {
        super.initSession(false);
        super.define(Book.getMetaDescription());
    }

    public void cleanUp() {
        super.cleanUp(Book.class);
    }
}
