/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.jca;

import org.objectweb.jalisto.se.exception.JcaException;
import org.objectweb.jalisto.se.impl.trace.Trace;

import javax.naming.Reference;
import javax.resource.ResourceException;
import javax.resource.spi.ConnectionManager;
import java.io.ObjectStreamException;
import java.io.PrintWriter;
import java.io.Serializable;

public class JalistoCfSerializationWrapper implements Serializable {

    public JalistoCfSerializationWrapper() {
    }

    public void setConnectionManager(ConnectionManager cm) {
        this.connectionManager = cm;
    }

    public void setManagedConnectionFactory(JalistoManagedConnectionFactory jdomcf) {
        try {
            logger = jdomcf.getLogWriter();
        } catch (ResourceException e) {
        }

        connectionURL = jdomcf.getConnectionURL();
        connectionDriverName = jdomcf.getConnectionDriverName();
        hashCode = jdomcf.hashCode();
    }

    public void setReference(Reference ref) {
        reference = ref;
    }

    public Object readResolve() throws ObjectStreamException {
        if (trace != null) {
            trace.println(Trace.JCA, TRACE_ID + " readResolve");
        }

        JalistoConnectionFactory result = null;

        if (reference != null) {
            result = (JalistoConnectionFactory) JalistoObjectFactory.getConnectionFactory(reference);
        }

        if (result == null) {
            // fully rebuild a new identical JDOConnectionFactory (cluster case)
            try {
                JalistoManagedConnectionFactory managedConnectionFactory = new JalistoManagedConnectionFactory();
                managedConnectionFactory.setLogWriter(logger);
                managedConnectionFactory.setConnectionURL(connectionURL);
                managedConnectionFactory.setConnectionDriverName(connectionDriverName);
                managedConnectionFactory.setHashCode(hashCode);
                result = (JalistoConnectionFactory) managedConnectionFactory.createConnectionFactory(connectionManager);
            } catch (ResourceException e) {
                throw new JcaException("Cannot create new ConnectionFactory instance", e);
            }
        }

        return result;
    }


    private ConnectionManager connectionManager = null;
    private String connectionDriverName = null;
    private String connectionURL = null;

    private PrintWriter logger = null;
    private Reference reference = null;

    private Trace trace = null;
    private int hashCode = 0;


    private static final String TRACE_ID = "[JalistoCfSerializationWrapper]";

    static final long serialVersionUID = -7589377087964761669L;
}
