/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.jca;

import org.objectweb.jalisto.se.exception.JcaException;
import org.objectweb.jalisto.se.impl.trace.Trace;
import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.jca.data.JalistoConnectionRequestInfo;

import javax.naming.NamingException;
import javax.naming.Reference;
import javax.naming.StringRefAddr;
import javax.resource.NotSupportedException;
import javax.resource.ResourceException;
import javax.resource.cci.*;
import javax.resource.spi.ConnectionManager;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.ManagedConnectionFactory;
import java.io.ObjectStreamException;

public class JalistoConnectionFactory implements ResourceAdapterMetaData, ConnectionFactory {

    public JalistoConnectionFactory(ConnectionManager connectionManager, JalistoManagedConnectionFactory mcFactory)
            throws Exception {
        super();
        this.trace = mcFactory.getTrace();
        trace.println(Trace.JCA, TRACE_ID + " constructor");
        this.connectionManager = connectionManager;
        this.mcf = mcFactory;
    }

    public JalistoConnectionFactory(ManagedConnectionFactory mcf, ConnectionManager cm) throws Exception {
        this(cm, (JalistoManagedConnectionFactory) mcf);
    }


    public Session getSession(String userid, String password) {
        synchronized (this) {
            Session result;
            try {
                JalistoConnectionSpec spec = new JalistoConnectionSpec();
                spec.setUserName(userid);
                spec.setPassword(password);
                result = (Session) getConnection(spec);
            } catch (ResourceException re) {
                throw new JcaException(PMGET_EXCEPTION_MSG, re);
            }
            return result;
        }
    }

    public synchronized Session getSession() {
        synchronized (this) {
            Session result;
            try {
                result = (Session) getConnection();
            } catch (ResourceException re) {
                throw new JcaException(PMGET_EXCEPTION_MSG, re);
            }
            return result;
        }
    }


    private Object writeReplace() throws ObjectStreamException {
        trace.println(Trace.JCA, TRACE_ID + " writeReplace");
        JalistoCfSerializationWrapper wrapper = new JalistoCfSerializationWrapper();
        try {
            wrapper.setReference(this.getReference());
            wrapper.setManagedConnectionFactory(mcf);
            wrapper.setConnectionManager(connectionManager);
        } catch (NamingException e) {
            throw new JcaException("Cannot save reference before serialization", e);
        }
        return wrapper;
    }


    /****************************   properties methods *****************************************/

    public void setConnectionDriverName(String connectionDriverName) {
        this.connectionDriverName = connectionDriverName;
    }

    public void setConnectionPassword(String connectionPassword) {
        this.connectionPassword = connectionPassword;
    }

    public void setConnectionURL(String connectionURL) {
        this.connectionURL = connectionURL;
    }

    public void setConnectionUserName(String connectionUserName) {
        this.connectionUserName = connectionUserName;
    }

    public String getConnectionDriverName() {
        return connectionDriverName;
    }

    public String getConnectionPassword() {
        return connectionPassword;
    }

    public String getConnectionURL() {
        return connectionURL;
    }

    public String getConnectionUserName() {
        return connectionUserName;
    }

    /*****************************   ConnectionFactory implementation ***************************/

    public javax.resource.cci.Connection getConnection() throws ResourceException {
        synchronized (this) {
            return getConnection(null);
        }
    }

    public javax.resource.cci.Connection getConnection(ConnectionSpec spec) throws ResourceException {
        synchronized (this) {
            trace.println(Trace.JCA, TRACE_ID + " getConnection({0})", spec);
            ConnectionRequestInfo infos = new JalistoConnectionRequestInfo(spec);
            return (javax.resource.cci.Connection) connectionManager.allocateConnection(mcf, infos);
        }
    }

    public RecordFactory getRecordFactory() throws ResourceException {
        synchronized (this) {
            try {
                throw new NotSupportedException("getRecordFactory()");
            } catch (Exception e) {
                e.printStackTrace();
            }
            return null;
        }
    }

    public ResourceAdapterMetaData getMetaData() throws ResourceException {
        synchronized (this) {
            trace.println(Trace.JCA, TRACE_ID + " getMetaData()");
            return this;
        }
    }


    /*****************************   ResourceAdapterMetaData implementation ***************************/

    public String getAdapterName() {
        synchronized (this) {
            return null;
        }
    }

    public String getAdapterShortDescription() {
        synchronized (this) {
            return RESDESC;
        }
    }

    public String getAdapterVendorName() {
        synchronized (this) {
            return null;
        }
    }

    public String getAdapterVersion() {
        synchronized (this) {
            return null;
        }
    }

    public String getSpecVersion() {
        return null;
    }

    public boolean supportsExecuteWithInputAndOutputRecord() {
        return false;
    }

    public boolean supportsExecuteWithInputRecordOnly() {
        return false;
    }

    public boolean supportsLocalTransactionDemarcation() {
        return false;
    }

    public String[] getInteractionSpecsSupported() {
        synchronized (this) {
            return null;
        }
    }

    /******************************   Referenceable implementation ************************************/

    public void setReference(Reference reference) {
        synchronized (this) {
            trace.println(Trace.JCA, TRACE_ID + " set new reference = {0}", reference);
        }
    }

    public Reference getReference() throws NamingException {
        synchronized (this) {
            JalistoObjectFactory.setTrace(trace);
            trace.println(Trace.JCA, TRACE_ID + " get reference");

            Reference ref = new Reference(this.getClass().getName(), DRIVER_NAMING_FACTORY_NAME, null);

            // These values are used by ObjectFactory
            ref.add(new StringRefAddr(this.NAMINGPROPNAME, this.RESNAME));
            ref.add(new StringRefAddr(JalistoObjectFactory.JALISTOID, this.toString()));
            JalistoObjectFactory.addConnectionFactory(this.toString(), this);
            trace.println(Trace.JCA, TRACE_ID + " the reference is \n{0}", ref);

            return ref;
        }
    }


    private ConnectionManager connectionManager = null;
    private JalistoManagedConnectionFactory mcf = null;
    private transient Trace trace;

    private String connectionDriverName;
    private String connectionURL;           // properties file name
    private String connectionUserName;
    private String connectionPassword;

    private static final String PMGET_EXCEPTION_MSG = "Error while getting a JalistoSession";
    private static final String TRACE_ID = "[JalistoConnectionFactory]";
    private static final String DRIVER_NAMING_FACTORY_NAME = "org.objectweb.jalisto.se.jca.JalistoObjectFactory";
    private static final String NAMINGPROPNAME = "xcalia.jalisto";
    private static final String RESNAME = "Xcalia Jalisto Connector Adapter 1.0";
    private static final String RESDESC = "This is a generic Xcalia Connector adapter for Jalisto";
}
