/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.jca;

import org.objectweb.jalisto.se.impl.trace.Trace;
import org.objectweb.jalisto.se.impl.trace.TraceNull;
import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.api.JalistoProperties;

import javax.resource.ResourceException;
import javax.resource.spi.*;
import javax.security.auth.Subject;
import java.io.PrintWriter;
import java.util.Iterator;
import java.util.Set;

public class JalistoManagedConnectionFactory implements ManagedConnectionFactory {

    public void setHashCode(int code) {
        hashCode = code;
    }

    public Trace getTrace() {
        return trace;
    }

    private String getCfIdentification() {
        return String.valueOf(connectionDriverName) +
               String.valueOf(connectionURL) +
               String.valueOf(connectionUserName) +
               String.valueOf(connectionPassword);
    }

    private void initTrace() {
        if (connectionURL != null) {
            JalistoProperties prop = JalistoFactory.getInternalFactory().getProperties(connectionURL);
            trace = JalistoFactory.getInternalFactory().getTracer(prop);
        } else {
            trace = new TraceNull();
        }
        JalistoXARPool.setTrace(trace);
        JalistoObjectFactory.setTrace(trace);
    }

    private void out(Object o) {
        String message = TRACE_ID + " " + o.toString();
        trace.println(Trace.JCA, message);
    }


    /*********************************** Properties methods *******************************************/

    public void setConnectionDriverName(String s) {
        connectionDriverName = s;
        if (jalistoCf != null) {
            jalistoCf.setConnectionDriverName(s);
        }
    }

    public String getConnectionDriverName() {
        return jalistoCf.getConnectionDriverName();
    }

    public void setConnectionPassword(String connectionPassword) {
        this.connectionPassword = connectionPassword;
        if (jalistoCf != null) {
            jalistoCf.setConnectionPassword(connectionPassword);
        }
    }

    public String getConnectionPassword() {
        return connectionPassword;
    }

    public void setConnectionURL(String s) {
        connectionURL = s;

        if (jalistoCf != null) {
            jalistoCf.setConnectionURL(s);
        }
    }

    public String getConnectionURL() {
        return jalistoCf.getConnectionURL();
    }

    public void setConnectionUserName(String connectionUserName) {
        this.connectionUserName = connectionUserName;
        if (jalistoCf != null) {
            jalistoCf.setConnectionUserName(connectionUserName);
        }
    }

    public String getConnectionUserName() {
        return connectionUserName;
    }

    /*************************   ManagedConnectionFactory implementation *******************************/

    public Object createConnectionFactory() throws ResourceException {
        return createConnectionFactory(new JalistoConnectionManager());
    }

    public Object createConnectionFactory(ConnectionManager manager) throws ResourceException {
        try {
            initTrace();

            if (jalistoCf == null) {
                jalistoCf = new JalistoConnectionFactory(manager, this);

                if (connectionDriverName != null) {
                    jalistoCf.setConnectionDriverName(connectionDriverName);
                }
                if (connectionURL != null) {
                    jalistoCf.setConnectionURL(connectionURL);
                }
                if (connectionUserName != null) {
                    jalistoCf.setConnectionUserName(connectionUserName);
                }
                if (connectionPassword != null) {
                    jalistoCf.setConnectionPassword(connectionPassword);
                }
            }
            out("createConnectionFactory(ConnectionManager)");
            out("connectionDriverName = " + connectionDriverName);
            out("connectionURL        = " + connectionURL);
            out("connectionUserName   = " + connectionUserName);
        } catch (Exception e) {
            e.printStackTrace();
            throw new ResourceException("Cannot create ConnectionFactory");
        }
        JalistoXARPool.setJdoConnectionFactory(getCfIdentification(), jalistoCf);
        return jalistoCf;
    }

    // info may be null
    public ManagedConnection createManagedConnection(Subject subject, ConnectionRequestInfo info)
            throws ResourceException {
        out("createManagedConnection");
        out("with Subject : " + String.valueOf(subject));
        out("with info : " + String.valueOf(info));
        JalistoManagedConnection managedConnection = new JalistoManagedConnection(connectionURL, info, trace);
        managedConnection.setLogWriter(getLogWriter());
        managedConnection.setCfIdentification(getCfIdentification());
        return managedConnection;
    }

    public ManagedConnection matchManagedConnections(Set set, Subject subject, ConnectionRequestInfo info)
            throws ResourceException {
        out("matchManagedConnections");
        out("with info : " + String.valueOf(info));
        out("with set : " + String.valueOf(set));
        JalistoManagedConnection jdoManagedConnection;
        for (Iterator iterator = set.iterator(); iterator.hasNext();) {
            jdoManagedConnection = (JalistoManagedConnection) iterator.next();
            if (jdoManagedConnection.matchConnectionRequestInfo(info) && jdoManagedConnection.isAvalaible()) {
                jdoManagedConnection.setNotAvalaible();
                out("matchManagedConnections : return " + String.valueOf(jdoManagedConnection));
                return jdoManagedConnection;
            }
        }
        out("matchManagedConnections : return null");
        return null;
    }

    public void setLogWriter(PrintWriter writer) throws ResourceException {
        if (writer != null) {
            logger = writer;
        }
    }

    public PrintWriter getLogWriter() throws ResourceException {
        return logger;
    }

    // hashcode devrait tre gal si les connection... le sont
    public int hashCode() {
        if (hashCode == 0) {
            hashCode = super.hashCode();
        }
        return hashCode;
    }

    public boolean equals(Object o) {
        if (o instanceof JalistoManagedConnectionFactory) {
            JalistoManagedConnectionFactory candidat = (JalistoManagedConnectionFactory) o;
            return (connectionURL.equals(candidat.connectionURL) &&
                    connectionDriverName.equals(candidat.connectionDriverName) &&
                    connectionUserName.equals(candidat.connectionUserName) &&
                    connectionPassword.equals(candidat.connectionPassword));
        }
        return false;
    }


    private JalistoConnectionFactory jalistoCf = null;
    private PrintWriter logger = null;
    private Trace trace = new TraceNull();

    private String connectionDriverName;
    private String connectionURL;           // properties file name
    private String connectionUserName;
    private String connectionPassword;

    private int hashCode = 0;

    private static final String TRACE_ID = "[JalistoManagedConnectionFactory]";
}
