/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.jca;

import org.objectweb.jalisto.se.impl.trace.Trace;
import org.objectweb.jalisto.se.jca.session.ManagedSession;
import org.objectweb.jalisto.se.jca.session.ManagedTransactionImpl;

import javax.transaction.xa.XAException;
import javax.transaction.xa.Xid;

public class JalistoXAResource {

    public JalistoXAResource(ManagedSession mpm, Trace trace) {
        this.managedJalistoSession = mpm;
        this.trace = trace;
        this.transaction = (ManagedTransactionImpl) mpm.currentTransaction();
    }

    public void setCurrentXid(Xid currentXid) {
        this.currentXid = currentXid;
    }

    public Xid getCurrentXid() {
        return currentXid;
    }

    public void setManagedJalistoSession(ManagedSession managedJalistoSession) {
        this.managedJalistoSession = managedJalistoSession;
    }

    public ManagedSession getManagedJalistoSession() {
        return managedJalistoSession;
    }

    public boolean isSameJDOXAR(JalistoXAResource jdoxar) {
        if ((managedJalistoSession == null) || (jdoxar == null)) {
            return false;
        }

        return managedJalistoSession.equals(jdoxar.managedJalistoSession);
    }

    public void commit(Xid xid, boolean b) throws XAException {
        getTrace().println(Trace.JCA,
                           TRACE_ID + " commit XAResource : {0} current xid : {1} call with xid : {2}",
                           this, currentXid, xid);
        getTrace().println(Trace.JCA, TRACE_ID + " hashcodes : {0}, {1}",
                           String.valueOf(currentXid.hashCode()),
                           String.valueOf(xid.hashCode()));

        if (transaction.isActive()) {
            if ((currentXid == null) || (currentXid.hashCode() != xid.hashCode()) || !xaUsed) {
                if (currentXid == null) {
                    throw new XAException("Cannot mix transactions (currentXid == null)");
                }
                if (currentXid.hashCode() != xid.hashCode()) {
                    throw new XAException("Cannot mix transactions (currentXid.hashCode() != xid.hashCode())");
                }
                if (!xaUsed) {
                    throw new XAException("Cannot mix transactions (!xaUsed)");
                }
            }

            transaction.commit();
            managedJalistoSession.finishTransaction();
            JalistoXARPool.releaseXAResource(this);
            currentXid = null;
            xaUsed = false;
        }
    }

    public void rollback(Xid xid) throws XAException {
        getTrace().println(Trace.JCA,
                           TRACE_ID + " commit XAResource : {0} current xid : {1} call with xid : {2}",
                           this, currentXid, xid);
        getTrace().println(Trace.JCA, TRACE_ID + " hashcodes : {0}, {1}",
                           String.valueOf(currentXid.hashCode()),
                           String.valueOf(xid.hashCode()));

        if (transaction.isActive()) {
            if ((currentXid == null) || (currentXid.hashCode() != xid.hashCode()) || !xaUsed) {
                if (currentXid == null) {
                    throw new XAException("Cannot mix transactions (currentXid == null)");
                }
                if (currentXid.hashCode() != xid.hashCode()) {
                    throw new XAException("Cannot mix transactions (currentXid.hashCode() != xid.hashCode())");
                }
                if (!xaUsed) {
                    throw new XAException("Cannot mix transactions (!xaUsed)");
                }
            }

            transaction.rollback();
            managedJalistoSession.finishTransaction();
            JalistoXARPool.releaseXAResource(this);
            currentXid = null;
            xaUsed = false;
        }
    }

    public void start(Xid xid, JalistoManagedConnection mc) throws XAException {
        if (xaUsed) {
            if (currentXid == null) {
                throw new XAException("XA START: xaUsed && currentXid == null");
            }
        }
        xaUsed = true;
        currentXid = xid;
        try {
            mc.setManagedJalistoSession(managedJalistoSession);
            if (!transaction.isActive()) {
                transaction.begin();
                managedJalistoSession.startTransaction(xid);
            }
        } catch (Throwable t) {
            t.printStackTrace();
            throw new XAException("ERROR during IES association : " + t.getMessage());
        }
        getTrace().println(Trace.JCA, TRACE_ID + " start");
    }

    private Trace getTrace() {
        return trace;
    }


    private ManagedSession managedJalistoSession;
    private ManagedTransactionImpl transaction;
    private Xid currentXid = null;
    private boolean xaUsed = false;
    int timeout;
    private Trace trace;

    private static final String TRACE_ID = "[JalistoXAResource]";

}
