/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.jmx;

import javax.management.*;


public class MemoryAdmin extends JalistoAbstractDynamicMBean {

    public MemoryAdmin(String propertiesPath) {
    }

    public float getFreeMemorySize() {
        return ((float) Runtime.getRuntime().freeMemory()) / mo;
    }

    public float getTotalMemorySize() {
        return ((float) Runtime.getRuntime().totalMemory()) / mo;
    }

    public float getUsedMemorySize() {
        return ((float) (Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory())) / mo;
    }


    /*****************************   DYNAMIC MBEAN IMPLEMENTATION   *****************************/

    public MBeanInfo getMBeanInfo() {
        if (infos == null) {
            MBeanAttributeInfo[] attributesInfos = new MBeanAttributeInfo[3];
            attributesInfos[0] = new MBeanAttributeInfo(
                    "FreeMemorySize",
                    "java.lang.Float",
                    "Size in Mo of free memory allocated to the JVM.",
                    true,
                    false,
                    false);
            attributesInfos[1] = new MBeanAttributeInfo(
                    "TotalMemorySize",
                    "java.lang.Float",
                    "Size in Mo of memory allocated to the JVM.",
                    true,
                    false,
                    false);
            attributesInfos[2] = new MBeanAttributeInfo(
                    "UsedMemorySize",
                    "java.lang.Float",
                    "Size in Mo of used memory allocated to the JVM.",
                    true,
                    false,
                    false);

            infos = new MBeanInfo(this.getClass().getName(),
                                  "Administration MBean to watch memory used by Jalisto",
                                  attributesInfos,
                                  new MBeanConstructorInfo[0],
                                  new MBeanOperationInfo[0],
                                  new MBeanNotificationInfo[0]);
        }
        return infos;
    }

    protected Object internalGetAttribute(String attributeName)
            throws AttributeNotFoundException, MBeanException, ReflectionException {
        if (attributeName.equals("FreeMemorySize")) {
            return new Float(getFreeMemorySize());
        }
        if (attributeName.equals("TotalMemorySize")) {
            return new Float(getTotalMemorySize());
        }
        if (attributeName.equals("UsedMemorySize")) {
            return new Float(getUsedMemorySize());
        }

        throw(new AttributeNotFoundException(
                "Cannot find " + attributeName + " attribute in " + this.getClass().getName()));
    }

    protected boolean internalSetAttribute(String name, Object value)
            throws AttributeNotFoundException, InvalidAttributeValueException, MBeanException, ReflectionException {
        if (name.equals("FreeMemorySize")) {
            throw(new AttributeNotFoundException(
                    "Cannot set attribute " + name + " because it is read-only"));
        } else if (name.equals("TotalMemorySize")) {
            throw(new AttributeNotFoundException(
                    "Cannot set attribute " + name + " because it is read-only"));
        } else if (name.equals("UsedMemorySize")) {
            throw(new AttributeNotFoundException(
                    "Cannot set attribute " + name + " because it is read-only"));
        }
        return false;
    }

    protected Object internalInvoke(String operationName, Object[] params, String[] signature)
            throws MBeanException, ReflectionException {
        throw new ReflectionException(new NoSuchMethodException(operationName),
                                      "Cannot find the operation " + operationName +
                                      " in " + this.getClass().getName());
    }


    private MBeanInfo infos;

    private static final int mo = 1024 * 1024;
}
