/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.jmx.server.mx4j;

import mx4j.tools.adaptor.http.HttpAdaptor;
import org.objectweb.jalisto.se.jmx.exception.MBeanAdminException;
import org.objectweb.jalisto.se.jmx.MemoryAdmin;
import org.objectweb.jalisto.se.api.jmx.JalistoMBeanServer;
import org.objectweb.jalisto.se.api.JalistoProperties;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.jmx.CacheAdmin;
import org.objectweb.jalisto.se.jmx.ClassDescriptionAdmin;
import org.objectweb.jalisto.se.JalistoFactory;

import javax.management.Attribute;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.ObjectName;
import javax.management.remote.JMXConnectorServer;
import javax.management.remote.JMXConnectorServerFactory;
import javax.management.remote.JMXServiceURL;


public class JalistoMbeanServerMx4jImpl extends Thread implements JalistoMBeanServer {

    public static void main(String[] args) {
        String propertiesPath = "jalisto.properties";
        try {
            propertiesPath = args[0];
        } catch (Exception e) {
            System.out.println("Cannot find properties path argument, use default value = " + propertiesPath);
        }

        JalistoMbeanServerMx4jImpl jalistoServer = new JalistoMbeanServerMx4jImpl();
        jalistoServer.init(propertiesPath);
        jalistoServer.run();
    }

    public JalistoMbeanServerMx4jImpl() {
    }

    public void init(String propertiesPath) {
        this.propertiesPath = propertiesPath;
        this.mbeanServer = MBeanServerFactory.createMBeanServer();
        this.props = JalistoFactory.getInternalFactory().getProperties(propertiesPath);

        enlistHttpAdaptor(props.getMBeanJmxPort());
        enlistJmxRmiConnector();
    }

    public void run() {
        registerMBean(mbeanServer, CacheAdmin.class.getName());
        registerMBean(mbeanServer, MemoryAdmin.class.getName());
        registerMBean(mbeanServer, ClassDescriptionAdmin.class.getName());

        System.out.println("Jalisto Mx4j MBean server is OK");
    }

    public void addSession(String s) {
    }

    private void enlistHttpAdaptor(int port) {
        try {
            ObjectName name = new ObjectName(mbeanServer.getDefaultDomain()+":name=HttpAdaptor");
            ObjectName processorName = new ObjectName(mbeanServer.getDefaultDomain()+":name=XSLTProcessor");
            System.out.println("name = " + name);
            System.out.println("processorName = " + processorName);

            HttpAdaptor adaptor = new HttpAdaptor();
            mbeanServer.registerMBean(adaptor, name);
            adaptor.setPort(port);
            adaptor.setHost("localhost");
            System.out.println("uses localhost on port "+port+" for HttpAdaptor...");

            mbeanServer.createMBean("mx4j.tools.adaptor.http.XSLTProcessor", processorName, null);
            mbeanServer.setAttribute(name, new Attribute("ProcessorName", processorName));

            adaptor.start();
        } catch (JalistoException jalistoExc) {
            throw jalistoExc;
        } catch (Exception e) {
            throw new MBeanAdminException("Could not create the Jalisto Mx4j HtmlAdaptor", e);
        }
        System.out.println("Jalisto Mx4j HtmlAdaptor started...");
    }

    private void enlistJmxRmiConnector() {
        try {
            JMXServiceURL address = new JMXServiceURL("service:jmx:rmi://localhost");
            JMXConnectorServer connectorServer = JMXConnectorServerFactory.newJMXConnectorServer(address, null, null);

            ObjectName connectorServerName = ObjectName.getInstance("connectors:protocol=rmi");
            mbeanServer.registerMBean(connectorServer, connectorServerName);

            connectorServer.start();

        } catch (Exception e) {
            throw new MBeanAdminException("Could not create Jalisto jmx connector", e);
        }
    }

    private void registerMBean(MBeanServer server, String mbeanClassName) {
        try {
            Object[] params = new Object[1];
            params[0] = propertiesPath;
            String[] signature = new String[1];
            signature[0] = "java.lang.String";
            ObjectName mbeanObjectName =
//                    new ObjectName(server.getDefaultDomain() + ":type=" + mbeanClassName);
                    new ObjectName(domain + ":type=" + mbeanClassName);
            server.createMBean(mbeanClassName,
                               mbeanObjectName,
                               null,
                               params,
                               signature);
        } catch (Exception e) {
            throw new MBeanAdminException(e);
        }
    }

    private String domain = "Jalisto";
    private String propertiesPath;
    private JalistoProperties props;
    private MBeanServer mbeanServer;
}
