/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.query;

import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.impl.InFileAddress;
import org.objectweb.jalisto.se.api.internal.InternalPhysicalFileAccess;
import org.objectweb.jalisto.se.api.query.Index;
import org.objectweb.jalisto.se.api.internal.SessionInternal;
import org.objectweb.jalisto.se.api.internal.LogicalSystemPageAccess;
import org.objectweb.jalisto.se.api.query.IndexManager;
import org.objectweb.jalisto.se.query.exception.QueryEngineException;
import org.objectweb.jalisto.se.impl.meta.ClassDescriptionImpl;
import org.objectweb.jalisto.se.impl.meta.IndexDescription;
import org.objectweb.jalisto.se.impl.server.page.Page;
import org.objectweb.jalisto.se.query.btree.BTree;
import org.objectweb.jalisto.se.query.btree.BTreeNode;
import org.objectweb.jalisto.se.query.btree.OidCollection;
import org.objectweb.jalisto.se.query.index.IndexBTreeImpl;
import org.objectweb.jalisto.se.query.index.IndexHashMapImpl;

import java.util.*;

public class IndexManagerImpl implements IndexManager {

    private IndexManagerImpl(SessionInternal session) {
        this.session = session;
        this.sessionId = session.getSessionId();
        this.logical = session.getFileAccess();
        this.physical = logical.getPhysicalAccess();
        this.indexes = new HashMap();
        this.indexesIfaList = new LinkedList();
        this.counter = 0;
    }

    public Index getIndex(ClassDescription meta, int fieldIndex) {
        if (!session.currentTransaction().isActive()) {
            throw new QueryEngineException("session must be active to call resolveLeafOnIndex method");
        }
        InFileAddress ifa = ((ClassDescriptionImpl) meta).getIndexIfa(fieldIndex);
        if (ifa == null) {
            throw new QueryEngineException("no index for this field");
        }
        Index index = (Index) indexes.get(ifa);
        if (index == null) {
            return loadIndex(ifa);
        }
        return index;
    }

    public Index getIndex(InFileAddress ifa) {
        if (!session.currentTransaction().isActive()) {
            throw new QueryEngineException("session must be active to call resolveLeafOnIndex method");
        }
        if (ifa == null) {
            throw new QueryEngineException("no index for this field");
        }
        Index index = (Index) indexes.get(ifa);
        if (index == null) {
            return loadIndex(ifa);
        }
        return index;
    }

    public Index buildIndexOnField(ClassDescription meta, int fieldIndex) {
        if (!session.currentTransaction().isActive()) {
            throw new QueryEngineException("session must be active to call buildIndexOnField method");
        }
        ClassDescriptionImpl metaImpl = (ClassDescriptionImpl) meta;
        InFileAddress ifa = metaImpl.getIndexIfa(fieldIndex);
        if (ifa != null) {
            return getIndex(metaImpl, fieldIndex);
        }
        ifa = new InFileAddress(IFA_PREFIXE + counter);
        counter++;
        indexesIfaList.add(ifa);
        updateStateInBase(true);
        metaImpl.setIndexIfa(fieldIndex, ifa);
        metaImpl.updateInBase();

        Index index;
        if (metaImpl.getIndexType(fieldIndex) == IndexHashMapImpl.TYPE) {
            index = new IndexHashMapImpl(ifa, fieldIndex, metaImpl);
        } else {     // default
            index = new IndexBTreeImpl(ifa, fieldIndex, metaImpl, this);
        }
        physical.writeObjectInBase(ifa, index, false);
        indexes.put(ifa, index);

        Iterator extent = session.getExtent(metaImpl.getClassName()).readFully().iterator();
        while (extent.hasNext()) {
            Object oid = extent.next();
            Object[] o = session.readObjectByOid(oid, true);
            index.put(o[fieldIndex], (LogicalOid) oid);
        }

        return index;
    }

    public void updateIndexInBase(Index index) {
        physical.writeObjectInBase(index.getIfa(), index, true);
    }

    public void deleteIndexOnField(ClassDescription meta, int fieldIndex) {
        if (!session.currentTransaction().isActive()) {
            throw new QueryEngineException("session must be active to call deleteIndexOnField method");
        }
        ClassDescriptionImpl metaImpl = (ClassDescriptionImpl) meta;
        InFileAddress ifa = metaImpl.getIndexIfa(fieldIndex);
        if (ifa != null) {
            Index index = getIndex(metaImpl, fieldIndex);
            InFileAddress indexIfa = index.getIfa();
            index.deleteIndex();
            indexes.remove(indexIfa);
            metaImpl.setIndexIfa(fieldIndex, null);
            metaImpl.updateInBase();
//            session.getMetaRepository().updateStateInBase(true);
            physical.deleteFileObject(ifa);
        }
    }

    public void deleteIndexesOnClass(ClassDescription meta) {
        Iterator descriptions = meta.getIndexedFieldDescription();
        while (descriptions.hasNext()) {
            IndexDescription indexDescription = (IndexDescription) descriptions.next();
            deleteIndexOnField(meta, indexDescription.getFieldDescription().getIndex());
        }
    }

    /**
     * ************************    insert/update/remove datas    **********************************
     */

    public void createObject(ClassDescription meta, LogicalOid floid, Object[] values) {
        Iterator descriptions = meta.getIndexedFieldDescription();
        while (descriptions.hasNext()) {
            IndexDescription indexDescription = (IndexDescription) descriptions.next();
            InFileAddress ifa = indexDescription.getValueIndexIfa();
            getIndex(ifa).put(values[indexDescription.getFieldDescription().getIndex()], floid);
        }
    }

    public void removeObject(ClassDescription meta, LogicalOid floid, Object[] values) {
        Iterator descriptions = meta.getIndexedFieldDescription();
        while (descriptions.hasNext()) {
            IndexDescription indexDescription = (IndexDescription) descriptions.next();
            InFileAddress ifa = indexDescription.getValueIndexIfa();
            getIndex(ifa).remove(values[indexDescription.getFieldDescription().getIndex()], floid);
        }
    }

    public void updateObject(ClassDescription meta, LogicalOid floid, Object[] oldValues, Object[] newValues) {
        Iterator descriptions = meta.getIndexedFieldDescription();
        while (descriptions.hasNext()) {
            IndexDescription indexDescription = (IndexDescription) descriptions.next();
            int index = indexDescription.getFieldDescription().getIndex();
            if (!oldValues[index].equals(newValues[index])) {
                getIndex(meta, index).remove(oldValues[index], floid);
                getIndex(meta, index).put(oldValues[index], floid);
            }
        }
    }

    /**
     * ************************    read/write for btrees    **************************************
     */

    public OidCollection readOidsInBase(BTree tree, InFileAddress ifa) {
        OidCollection oids = (OidCollection) tree.getLoadedElements().get(ifa);
        if (oids == null) {
            oids = (OidCollection) logical.readLeaf(sessionId, ifa);
            oids.setIfa(ifa);
            tree.getLoadedElements().put(ifa, oids);
        }
        return oids;
    }

    public InFileAddress getNewOidsCollIfaAndInsert(BTree tree, OidCollection oids) {
        InFileAddress leafIfa = logical.allocateLeafAddressAndInsert(
                sessionId, tree.getFieldDescription(), oids);
        tree.getLoadedElements().put(leafIfa, oids);
        return leafIfa;
    }

    public void updateLeaf(OidCollection oids) {
        logical.updateLeaf(sessionId, oids.getIfa(), oids);
    }

    public void removeLeaf(BTree tree, InFileAddress ifa) {
        logical.removeLeaf(sessionId, ifa);
        tree.getLoadedElements().remove(ifa);
    }

    public BTreeNode readNodeInBase(BTree tree, InFileAddress ifa) {
        BTreeNode node = (BTreeNode) tree.getLoadedElements().get(ifa);
        if (node == null) {
            node = (BTreeNode) logical.readNode(sessionId, ifa);
            node.setTree(tree);
            node.setIfa(ifa);
            tree.getLoadedElements().put(ifa, node);
        }
        return node;
    }

    public InFileAddress getNewNodeIfaAndInsert(BTree tree, BTreeNode node) {
        InFileAddress nodeIfa = logical.allocateNodeAddressAndInsert(sessionId, tree.getFieldDescription(), node);
        tree.getLoadedElements().put(nodeIfa, node);
        return nodeIfa;
    }

    public void updateNode(BTreeNode node) {
        logical.updateNode(sessionId, node.getIfa(), node);
    }

    public void removeNode(BTree tree, InFileAddress ifa) {
        logical.removeNode(sessionId, ifa);
        tree.getLoadedElements().remove(ifa);
    }

    public Page getPageForIfa(InFileAddress ifa) {
        return (Page) physical.readFileObjectAt(ifa);
    }

    /**
     * **************************    PRIVATE    *****************************************
     */

    private Index loadIndex(InFileAddress ifa) {
        Index index = (Index) physical.readFileObjectAt(ifa).getData();
        index.setIndexManager(this);
        indexes.put(ifa, index);
        return index;
    }

    private void readStateFromBase() {
        Object[] o = (Object[]) physical.readFileObjectAt(INDEX_MANAGER_STATE_IFA).getData();
        this.counter = ((Integer) o[0]).intValue();
        this.nodeCounter = ((Integer) o[1]).intValue();
        this.oidsCounter = ((Integer) o[2]).intValue();
        this.indexesIfaList = (List) o[3];
    }

    private void updateStateInBase(boolean isUpdate) {
        Object[] datas = new Object[4];
        datas[0] = new Integer(counter);
        datas[1] = new Integer(nodeCounter);
        datas[2] = new Integer(oidsCounter);
        datas[3] = indexesIfaList;
        physical.writeObjectInBase(INDEX_MANAGER_STATE_IFA, datas, isUpdate);
    }


    private HashMap indexes;
    private SessionInternal session;
    private Object sessionId;
    private InternalPhysicalFileAccess physical;
    private LogicalSystemPageAccess logical;
    private List indexesIfaList;
    private int counter;
    private int nodeCounter;
    private int oidsCounter;


    /**
     * ************************************    STATIC    *********************************************
     */

    public static IndexManager getAnIndexManagerImpl(SessionInternal session) {
        IndexManagerImpl indexManager = new IndexManagerImpl(session);
        try {
            indexManager.readStateFromBase();
        } catch (Exception e) {
            indexManager.indexesIfaList = new LinkedList();
            indexManager.counter = 0;
            indexManager.updateStateInBase(false);
        }
        return indexManager;
    }

    protected static final String IFA_PREFIXE = "ind";
    protected static final InFileAddress INDEX_MANAGER_STATE_IFA = new InFileAddress("lstidx");
}
