/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.query;

import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.api.query.QueryManager;
import org.objectweb.jalisto.se.api.query.Query;
import org.objectweb.jalisto.se.api.query.Constraint;
import org.objectweb.jalisto.se.api.internal.SessionInternal;
import org.objectweb.jalisto.se.query.exception.QueryEngineException;
import org.objectweb.jalisto.se.query.constraint.*;
import org.objectweb.jalisto.se.query.execution.ExecutionTree;
import org.objectweb.jalisto.se.query.parameter.QueryParameter;
import org.objectweb.jalisto.se.query.parameter.QueryValue;
import org.objectweb.jalisto.se.query.parameter.SubQuery;
import org.objectweb.jalisto.se.query.result.QueryResultWrapper;
import org.objectweb.jalisto.se.query.result.ObjectSetImpl;
import org.objectweb.jalisto.se.api.query.ObjectSet;

import java.util.*;

public class QueryImpl implements Query {

    public QueryImpl(QueryManager queryManager, Object classConstraint) {
        this.queryManager = queryManager;
        this.session = (SessionInternal) queryManager.getSession();
        this.constraints = new HashMap();
        this.parameters = new HashMap();
        this.subQueries = new ArrayList();
        this.subFieldNames = new ArrayList();
        if (queriedClass != null) {
            try {
                this.queriedClass = (Class) classConstraint;
            } catch (ClassCastException e) {
                this.queriedClass = classConstraint.getClass();
            }
        }
    }

    public Constraint constrain(Object constraintObject) {
        if (queriedClass == null) {
            try {
                this.queriedClass = (Class) constraintObject;
            } catch (ClassCastException e) {
                this.queriedClass = constraintObject.getClass();
            }
            return null;
        }
        Constraint constraint;
        String key = getSubFieldNamesAsKey();
        ArrayList constraintList = (ArrayList) constraints.get(key);
        if (constraintList == null) {
            constraintList = new ArrayList();
            constraints.put(key, constraintList);
        }
        if (constraintObject instanceof Query) {
            constraintObject = new SubQuery((Query) constraintObject);
            subQueries.add(constraintObject);
            constraint = new SubQueryConstraintImpl((SubQuery) constraintObject, queriedClass.getName(), subFieldNames);
        } else {
            if (constraintObject instanceof QueryParameter) {
                QueryParameter parameter = (QueryParameter) constraintObject;
                parameters.put(parameter.getParameterName(), parameter);
            } else if (!(constraintObject instanceof QueryValue)) {
                constraintObject = new QueryValue(constraintObject);
            }
            constraint = new ValueConstraintImpl(queriedClass.getName(), subFieldNames, (QueryValue) constraintObject);
        }
        constraintList.add(constraint);
        subFieldNames.clear();
        return constraint;
    }

    private String getSubFieldNamesAsKey() {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < subFieldNames.size(); i++) {
            sb.append(subFieldNames.get(i));
            if (i != subFieldNames.size() - 1) {
                sb.append("@");
            }
        }
        return sb.toString();
    }

    public List constraints() {
        throw new QueryEngineException();
    }

    public Query descend(String fieldName) {
        subFieldNames.add(fieldName);
        return this;
    }

    public Query orderAscending() {
        throw new QueryEngineException();
    }

    public Query orderDescending() {
        throw new QueryEngineException();
    }

    public Constraint getBinaryTree() {
        HashSet roots = new HashSet();
        Iterator constraintLists = constraints.values().iterator();
        while (constraintLists.hasNext()) {
            ArrayList constraintList = (ArrayList) constraintLists.next();
            for (int i = 0; i < constraintList.size(); i++) {
                ConstraintImpl constraint = (ConstraintImpl) constraintList.get(i);
                roots.addAll(constraint.getRootFathers());
            }
        }
        // roots contains binConstraint and valueConstraint
        if (roots.size() > 0) {
            Iterator rootsIterator = roots.iterator();
            Constraint left = (Constraint) rootsIterator.next();
            while (rootsIterator.hasNext()) {
                Constraint right = (Constraint) rootsIterator.next();
                left = left.or(right);
            }
            return left;
        }
        return null;
    }

    public ObjectSet execute() {
        if (!session.currentTransaction().isActive()) {
            throw new QueryEngineException("transaction is not active during execution of a query");
        }

        checkBindedParameters();
        checkSubQueries();

        if (tree == null) {
            String queriedClass = getQueriedClassName();
            tree = new ExecutionTree(getBinaryTree(), session.getMetaRepository(), queriedClass);
        }

        Set resultSet = tree.resolve(session, queryManager.getIndexManager());

        tree.cleanLeaf();
        cleanSubQueries();

        ObjectSet result = new ObjectSetImpl(resultSet);

        return result;
    }

    public Set getExtent(ClassDescription meta) {
        session.currentTransaction().begin();
        Iterator extent = session.getExtent(meta.getClassName()).readFully().iterator();
        Set resultSet = new HashSet();
        while (extent.hasNext()) {
            LogicalOid floid = (LogicalOid) extent.next();
            Object[] values = session.readObjectByOid(floid, true);
            resultSet.add(new QueryResultWrapper(floid, values));
        }
        session.currentTransaction().commit();
        return resultSet;
    }

    private void checkBindedParameters() {
        Iterator parametersIt = parameters.values().iterator();
        while (parametersIt.hasNext()) {
            QueryParameter parameter = (QueryParameter) parametersIt.next();
            if (!parameter.isBind()) {
                throw new QueryEngineException("Parameter " + parameter.getParameterName() +
                                                  " is not bind during query execution");
            }
        }
    }

    private void checkSubQueries() {
        for (short i = 0; i < subQueries.size(); i++) {
            SubQuery subQuery = (SubQuery) subQueries.get(i);
            subQuery.getTree(session.getMetaRepository());
        }
    }

    private void cleanSubQueries() {
        for (short i = 0; i < subQueries.size(); i++) {
            SubQuery subQuery = (SubQuery) subQueries.get(i);
            subQuery.deleteTree();
        }
    }

    public void bind(String parameterName, Object value) {
        QueryParameter parameter = (QueryParameter) parameters.get(parameterName);
        if (parameter == null) {
            throw new QueryEngineException("Try to bind value to non existing parameter : " + parameterName);
        }
        parameter.bind(value);
    }

    public HashMap getParameters() {
        return parameters;
    }

    public ArrayList getSubQueries() {
        return subQueries;
    }

    public String getQueriedClassName() {
        return queriedClass.getName();
    }

    private SessionInternal session;
    private QueryManager queryManager;
    private Class queriedClass;
    private HashMap constraints;
    private HashMap parameters;
    private ArrayList subQueries;
    private ExecutionTree tree;
    private ArrayList subFieldNames;
}
