/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.query.btree;

import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.impl.InFileAddress;
import org.objectweb.jalisto.se.api.query.Index;
import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.api.query.FieldDescription;
import org.objectweb.jalisto.se.api.query.*;
import org.objectweb.jalisto.se.query.IndexManagerImpl;

import java.io.Serializable;
import java.util.*;

public class BTree implements Serializable {

    public BTree(Index index, int fieldIndex, ClassDescription meta,
                 IndexManagerImpl indexManager) {
        this.index = index;
        this.comparator = meta.getComparator(fieldIndex);
        this.fieldIndex = fieldIndex;
        this.indexManager = indexManager;
        this.meta = meta;
        loadedElements = new HashMap();
        BTreeNode root = new BTreeNode(this, null);
        rootIfa = root.getIfa();
        firstIfa = rootIfa;
    }

    public Set getOidsFor(Object value) {
        BTreeNode root = indexManager.readNodeInBase(this, rootIfa);
        OidCollection oids = indexManager.readOidsInBase(this, root.getOidsIfa(value, comparator));
        return oids.asSet();
    }

    public void addOid(Object value, LogicalOid oid) {
        BTreeNode root = indexManager.readNodeInBase(this, rootIfa);
        root.insertOid(comparator, value, oid);
    }

    public void removeOid(Object value, LogicalOid oid) {
        BTreeNode root = indexManager.readNodeInBase(this, rootIfa);
        root.removeOid(value, oid, comparator);
    }

    public Set getKeys() {
        Set set = new HashSet();
        InFileAddress nextIfa = firstIfa;
        while (nextIfa != null) {
            BTreeNode node = indexManager.readNodeInBase(this, nextIfa);
            nextIfa = node.getKeys(set);
        }
        return set;
    }

    public IndexManagerImpl getIndexManager() {
        return indexManager;
    }

    public void setIndexManager(IndexManagerImpl indexManager) {
        this.indexManager = indexManager;
        loadedElements = new HashMap();
    }

    public void setRootIfa(InFileAddress rootIfa) {
        this.rootIfa = rootIfa;
        updateInBase();
    }

    public FielComparator getComparator() {
        return comparator;
    }

    public InFileAddress getFirstIfa() {
        return firstIfa;
    }

    public void setFirstIfa(InFileAddress firstIfa) {
        this.firstIfa = firstIfa;
        updateInBase();
    }

    public InFileAddress getLastIfa() {
        return lastIfa;
    }

    public void setLastIfa(InFileAddress lastIfa) {
        this.lastIfa = lastIfa;
        updateInBase();
    }

    public FieldDescription getFieldDescription() {
        return meta.getFieldDescription(fieldIndex);
    }

    public Map getLoadedElements() {
        return loadedElements;
    }

    public void deleteTree() {
        BTreeNode root = indexManager.readNodeInBase(this, rootIfa);
        root.deleteYourself();
    }

    private void updateInBase() {
        indexManager.updateIndexInBase(index);
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("BTree on ").append(indexManager).append(" with comparator ").append(comparator);
        sb.append("\n    root ifa : ").append(rootIfa);
        sb.append("\n    firstIfa ifa : ").append(firstIfa);
        sb.append("\n    lastIfa ifa : ").append(lastIfa);
        if (loadedElements != null) {
            sb.append("\n    loadedElements size : ").append(loadedElements.size());
        }
        sb.append("\n");
        if (indexManager != null) {
            LinkedList toPrintNode = new LinkedList();
            LinkedList toPrintOids = new LinkedList();
            toPrintNode.add(rootIfa);
            while (!toPrintNode.isEmpty()) {
                BTreeNode node = indexManager.readNodeInBase(this, (InFileAddress) toPrintNode.removeFirst());
                sb.append(node.toFullString(toPrintNode, toPrintOids));
            }
            while (!toPrintOids.isEmpty()) {
                OidCollection oids = indexManager.readOidsInBase(this, (InFileAddress) toPrintOids.removeFirst());
                sb.append(oids.toFullString()).append("\n");
            }
        }
        return sb.toString();
    }

    private Index index;
    private ClassDescription meta;
    private int fieldIndex;
    private InFileAddress rootIfa;
    private InFileAddress firstIfa;
    private InFileAddress lastIfa;
    private FielComparator comparator;

    private transient IndexManagerImpl indexManager;
    private transient Map loadedElements;


    static final long serialVersionUID = -7589377901514761459L;

}
