/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.query.btree;

import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.impl.InFileAddress;
import org.objectweb.jalisto.se.api.query.FielComparator;
import org.objectweb.jalisto.se.api.query.Operator;
import org.objectweb.jalisto.se.query.exception.QueryEngineException;
import org.objectweb.jalisto.se.impl.server.DataWrapperImpl;
import org.objectweb.jalisto.se.query.operator.OperatorFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

public class BTreeNode extends DataWrapperImpl {
    public BTreeNode(BTree tree, InFileAddress fatherIfa) {
        this.cells = new ArrayList();
        this.fatherIfa = fatherIfa;
        this.tree = tree;
        this.overSize = tree.getFieldDescription().getIndexDescription().getNodeSize();
        if ((overSize % 2) != 0) {
            overSize++;
        }
        setIfa(tree.getIndexManager().getNewNodeIfaAndInsert(tree, this));
    }

    public InFileAddress getOidsIfa(Object value, FielComparator comparator) {
        for (int i = 0; i < cells.size(); i++) {
            BTreeCell cell = (BTreeCell) cells.get(i);
            if (equal.executeSingle(comparator, value, cell.getValue())) {
                return cell.getOidsIfa();
            } else if (greater.executeSingle(comparator, value, cell.getValue())) {
                InFileAddress subIfa = cell.getSubIfa();
                if (subIfa != null) {
                    BTreeNode node = tree.getIndexManager().readNodeInBase(tree, subIfa);
                    return node.getOidsIfa(value, comparator);
                } else {     // leaf
                    return null;
                }
            }
        }
        return null;
    }

    public void insertOid(FielComparator comparator, Object value, LogicalOid floid) {
        if (cells.isEmpty()) {
            createAndAddNewCell(0, floid, value);
        } else {
            boolean isInserted = false;
            for (int i = 0; i < cells.size(); i++) {
                BTreeCell cell = (BTreeCell) cells.get(i);
                if (equal.executeSingle(comparator, value, cell.value)) {
                    OidCollection oids = tree.getIndexManager().readOidsInBase(tree, cell.oidsIfa);
                    oids.addOid(floid);
                    tree.getIndexManager().updateLeaf(oids);
                    return;
                } else if (greater.executeSingle(comparator, value, cell.value)) {
                    if (cell.getSubIfa() != null) {         // not a leaf node
                        BTreeNode subNode = tree.getIndexManager().readNodeInBase(tree, cell.getSubIfa());
                        subNode.insertOid(comparator, value, floid);
                        return;
                    }
                    // on a leaf node
                    createAndAddNewCell(i, floid, value);
                    isInserted = true;
                    i = cells.size();
                }
            }
            if (!isInserted) {   // we are working on root node -> new max element
                OidCollection oids = new OidCollection();
                oids.addOid(floid);
                InFileAddress oidsIfa = tree.getIndexManager().getNewOidsCollIfaAndInsert(tree, oids);
                changeMaxValueDown(value, oidsIfa, comparator);
                return;
            }
            if (cells.size() >= overSize) {
                split(comparator);
            }
        }
        tree.getIndexManager().updateNode(this);
    }

    public void removeOid(Object value, LogicalOid oid, FielComparator comparator) {
        for (int i = 0; i < cells.size(); i++) {
            BTreeCell cell = (BTreeCell) cells.get(i);
            if (equal.executeSingle(comparator, value, cell.getValue())) {
                OidCollection oids = tree.getIndexManager().readOidsInBase(tree, cell.getOidsIfa());
                oids.removeOid(oid);
                if (oids.isEmpty()) {
                    tree.getIndexManager().removeLeaf(tree, oids.getIfa());
                    getNode(value, comparator).removeCell(value, comparator);
                } else {
                    tree.getIndexManager().updateLeaf(oids);
                    //                    tree.getIndexManager().updateNode(this);
                }
                return;
            } else if (greater.executeSingle(comparator, value, cell.getValue())) {
                InFileAddress subIfa = cell.getSubIfa();
                if (subIfa != null) {
                    BTreeNode node = tree.getIndexManager().readNodeInBase(tree, subIfa);
                    node.removeOid(value, oid, comparator);
                    return;
                } else {     // leaf
                    throw new QueryEngineException("remove oid for a non existing value : " + value);
                }
            }
        }
    }

    public InFileAddress getKeys(Set in) {
        for (int i = 0; i < cells.size(); i++) {
            BTreeCell cell = (BTreeCell) cells.get(i);
            if (cell.getSubIfa() == null) {
                in.add(cell.getValue());
            }
        }
        return nextIfa;
    }

    public void deleteYourself() {
        for (int i = 0; i < cells.size(); i++) {
            BTreeCell cell = (BTreeCell) cells.get(i);
            if (cell.getSubIfa() != null) {
                BTreeNode node = tree.getIndexManager().readNodeInBase(tree, cell.getSubIfa());
                node.deleteYourself();
            } else {
                tree.getIndexManager().removeLeaf(tree, cell.getOidsIfa());
            }
        }
        tree.getIndexManager().removeNode(tree, ifa);
    }

    /**
     * ****************************   PRIVATE   *********************************************
     */

    public BTreeNode getNode(Object value, FielComparator comparator) {
        for (int i = 0; i < cells.size(); i++) {
            BTreeCell cell = (BTreeCell) cells.get(i);
            if (equal.executeSingle(comparator, value, cell.getValue())) {
                InFileAddress subIfa = cell.getSubIfa();
                if (subIfa != null) {
                    BTreeNode node = tree.getIndexManager().readNodeInBase(tree, subIfa);
                    return node.getNode(value, comparator);
                } else {     // leaf
                    return this;
                }
            } else if (greater.executeSingle(comparator, value, cell.getValue())) {
                InFileAddress subIfa = cell.getSubIfa();
                if (subIfa != null) {
                    BTreeNode node = tree.getIndexManager().readNodeInBase(tree, subIfa);
                    return node.getNode(value, comparator);
                } else {     // leaf
                    return null;
                }
            }
        }
        return null;
    }

    private void split(FielComparator comparator) {
        BTreeNode newNode = new BTreeNode(tree, fatherIfa);
        InFileAddress newIfa = newNode.getIfa();

        // change first ifa if we are splitting first sub node
        if (ifa.equals(tree.getFirstIfa())) {
            tree.setFirstIfa(newIfa);
        }

        // copy first cells in new node
        for (int i = 0; i < overSize / 2; i++) {
            BTreeCell c = (BTreeCell) cells.remove(0);      // keep the order
            newNode.cells.add(c);
            if (c.getSubIfa() != null) {                    // maintain father ifa in sub node
                tree.getIndexManager().readNodeInBase(tree, c.getSubIfa()).setFatherIfa(newIfa);
            }
        }

        // maintain chaining between node
        if (lastIfa != null) {
            tree.getIndexManager().readNodeInBase(tree, lastIfa).setNextIfa(newIfa);
            newNode.setLastIfa(lastIfa);
        }
        newNode.setNextIfa(ifa);
        lastIfa = newIfa;

        if (fatherIfa != null) {
            BTreeNode fatherNode = tree.getIndexManager().readNodeInBase(tree, fatherIfa);
            BTreeCell c = newNode.getLastCell();
            fatherNode.insertCell(c.value, c.oidsIfa, newIfa, comparator);   // insert last cell of new node in father
//            fatherNode.getCellForValue(getLastCell().value, comparator).setSubIfa(ifa);   // really needed ?
            tree.getIndexManager().updateNode(fatherNode);
        } else {   // new root
            BTreeNode newRootNode = new BTreeNode(tree, null);
            InFileAddress newRootIfa = newRootNode.getIfa();
            fatherIfa = newRootIfa;
            newNode.setFatherIfa(newRootIfa);
            BTreeCell c = newNode.getLastCell();
            newRootNode.insertCell(c.value, c.oidsIfa, newIfa, comparator);
            c = getLastCell();
            newRootNode.insertCell(c.value, c.oidsIfa, ifa, comparator);
            tree.setRootIfa(newRootIfa);
            tree.getIndexManager().updateNode(newRootNode);
        }
    }

    private void createAndAddNewCell(int i, LogicalOid floid, Object value) {
        OidCollection oids = new OidCollection();
        oids.addOid(floid);
        InFileAddress oidsIfa = tree.getIndexManager().getNewOidsCollIfaAndInsert(tree, oids);
        BTreeCell newCell = new BTreeCell();
        newCell.setValue(value);
        newCell.setOidsIfa(oidsIfa);
        cells.add(i, newCell);
    }

    private void insertCell(Object value, InFileAddress oidsIfa, InFileAddress subIfa, FielComparator comparator) {
        BTreeCell newCell = new BTreeCell();
        newCell.setValue(value);
        newCell.setOidsIfa(oidsIfa);
        newCell.setSubIfa(subIfa);
        for (int i = 0; i < cells.size(); i++) {
            BTreeCell cell = (BTreeCell) cells.get(i);
            if (greater.executeSingle(comparator, value, cell.value)) {
                cells.add(i, newCell);
                return;
            }
        }
        cells.add(newCell);
    }

    private void removeCell(Object value, FielComparator comparator) {
        BTreeCell cell = getCellForValue(value, comparator);
        if (cell == null) {
            throw new QueryEngineException("no cell for value " + value);
        }
        int index = cells.indexOf(cell);
        if (index != -1) {
            int size = cells.size();
            cells.remove(index);
            if (size == 1) {  // last cell => remove node
                if (nextIfa != null) {
                    BTreeNode nextCell = tree.getIndexManager().readNodeInBase(tree, nextIfa);
                    nextCell.setLastIfa(lastIfa);
                    tree.getIndexManager().updateNode(nextCell);
                }
                if (lastIfa != null) {
                    BTreeNode lastCell = tree.getIndexManager().readNodeInBase(tree, lastIfa);
                    lastCell.setNextIfa(nextIfa);
                    tree.getIndexManager().updateNode(lastCell);
                }
                if (tree.getFirstIfa().equals(ifa)) {
                    if (nextIfa != null) {
                        tree.setFirstIfa(nextIfa);
                    } else if (fatherIfa != null) {
                        tree.setFirstIfa(fatherIfa);
                    } // else : root node
                }
                if (fatherIfa != null) {
                    BTreeNode father = tree.getIndexManager().readNodeInBase(tree, fatherIfa);
                    father.removeCell(value, comparator);
                    tree.getIndexManager().removeNode(tree, ifa);     // we don't delete root node
                }
            } else if (index == (size - 1)) {       // cell at the end of the collection
                if (fatherIfa != null) {
                    BTreeNode father = tree.getIndexManager().readNodeInBase(tree, fatherIfa);
                    BTreeCell last = getLastCell();
                    father.changeMaxValueUp(cell.getValue(), last.getValue(),
                                            cell.getOidsIfa(), last.getOidsIfa(), comparator);
                }
            }
        }
        tree.getIndexManager().updateNode(this);
    }

    private BTreeCell getCellForValue(Object value, FielComparator comparator) {
        for (int i = 0; i < cells.size(); i++) {
            BTreeCell cell = (BTreeCell) cells.get(i);
            if (equal.executeSingle(comparator, value, cell.value)) {
                return cell;
            }
        }
        return null;
    }

    private void changeMaxValueDown(Object value, InFileAddress oidsIfa, FielComparator comparator) {
        BTreeCell cell = getLastCell();
        InFileAddress subIfa = cell.getSubIfa();
        if (subIfa != null) {
            cell.setValue(value);
            cell.setOidsIfa(oidsIfa);
            tree.getIndexManager().updateNode(this);
            tree.getIndexManager().readNodeInBase(tree, subIfa).changeMaxValueDown(value, oidsIfa, comparator);
        } else {
            BTreeCell newCell = new BTreeCell();
            newCell.setValue(value);
            newCell.setOidsIfa(oidsIfa);
            cells.add(newCell);
            if (cells.size() >= overSize) {
                split(comparator);
            }
            tree.getIndexManager().updateNode(this);
        }
    }

    private void changeMaxValueUp(Object oldValue, Object newValue,
                                  InFileAddress oldOidsIfa, InFileAddress newOidsIfa,
                                  FielComparator comparator) {
        BTreeCell cell = getCellForValue(oldValue, comparator);
        if (cell == null) {
            throw new QueryEngineException("don't find cell with value " + oldValue);
        }
        BTreeCell last = getLastCell();
        if (last.equals(cell)) {
            if (fatherIfa != null) {
                BTreeNode father = tree.getIndexManager().readNodeInBase(tree, fatherIfa);
                father.changeMaxValueUp(oldValue, newValue,
                                        oldOidsIfa, newOidsIfa, comparator);
            }
        }
        cell.setValue(newValue);
        cell.setOidsIfa(newOidsIfa);
        tree.getIndexManager().updateNode(this);

    }

    /**
     * ****************************   GETTER SETTER   ********************************************
     */

    public InFileAddress getLastIfa() {
        return lastIfa;
    }

    public void setLastIfa(InFileAddress lastIfa) {
        this.lastIfa = lastIfa;
    }

    public InFileAddress getNextIfa() {
        return nextIfa;
    }

    public void setNextIfa(InFileAddress nextIfa) {
        this.nextIfa = nextIfa;
    }

    public BTreeCell getLastCell() {
        return (BTreeCell) cells.get(cells.size() - 1);
    }

    public InFileAddress getFatherIfa() {
        return fatherIfa;
    }

    public void setFatherIfa(InFileAddress fatherIfa) {
        this.fatherIfa = fatherIfa;
    }

    public void setTree(BTree tree) {
        this.tree = tree;
    }

    public InFileAddress getIfa() {
        return ifa;
    }

    public void setIfa(InFileAddress ifa) {
        this.ifa = ifa;
    }

    public String toString() {
        return "BTreeNode(" + cells.size() + ")";
    }

    public String toFullString(List nodes, List oids) {
        StringBuffer sb = new StringBuffer();
        sb.append(ifa).append(" : ");
        sb.append("BTree(");
        sb.append("lastIfa(").append(lastIfa).append("),");
        sb.append("nextIfa(").append(nextIfa).append("),");
        sb.append("fatherIfa(").append(fatherIfa).append("),");
        sb.append("cells:").append(cells).append(")\n");
        for (int i = 0; i < cells.size(); i++) {
            BTreeCell cell = (BTreeCell) cells.get(i);
            if (cell.getSubIfa() != null) {
                nodes.add(cell.getSubIfa());
            } else {
                oids.add(cell.getOidsIfa());
            }
        }
        return sb.toString();
    }


    protected short overSize;     // must be pair
    protected ArrayList cells;
    private InFileAddress fatherIfa;
    protected transient BTree tree;

    private InFileAddress lastIfa;
    private InFileAddress nextIfa;
    private InFileAddress ifa;

    protected static final Operator greater = OperatorFactory.getGreaterOperator();
    protected static final Operator equal = OperatorFactory.getEqualOperator();
}
