/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.query.constraint;

import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.api.query.FielComparator;
import org.objectweb.jalisto.se.api.query.Constraint;
import org.objectweb.jalisto.se.api.query.Operator;
import org.objectweb.jalisto.se.query.operator.OperatorFactory;
import org.objectweb.jalisto.se.query.parameter.QueryValue;
import org.objectweb.jalisto.se.query.result.WrapperSet;

import java.util.ArrayList;
import java.util.Collection;

public class ValueConstraintImpl extends ConstraintImpl {

    public ValueConstraintImpl(String className, ArrayList subFieldNames, QueryValue value) {
        this.className = className;
        this.fieldNames = new String[subFieldNames.size()];
        for (int i = 0; i < fieldNames.length; i++) {
            fieldNames[i] = (String) subFieldNames.get(i);
        }
        this.value = value;
        this.resolvedValues = new WrapperSet();
        this.isResolved = false;
    }

    public Constraint contains() {
        operator = OperatorFactory.getContainsOperator();
        return this;
    }

    public Constraint equal() {
        operator = OperatorFactory.getEqualOperator();
        return this;
    }

    public Object getObject() {
        return value.getValue();
    }

    public Constraint greater() {
        operator = OperatorFactory.getGreaterOperator();
        return this;
    }

    public Constraint identity() {
        operator = OperatorFactory.getIdentityOperator();
        return this;
    }

    public Constraint like() {
        throw new UnsupportedOperationException();
    }

    public Constraint not() {
        not = !not;
        return this;
    }

    public Constraint smaller() {
        operator = OperatorFactory.getSmallerOperator();
        return this;
    }

    public boolean execute(Session session, FielComparator comparator, Object candidate) {
        ArrayList candidates = new ArrayList();
        candidates.add(candidate);
        if (fieldNames.length > 1) {
            candidates.clear();
            if (Collection.class.isAssignableFrom(candidate.getClass())) {
                candidates.addAll((Collection) candidate);
            } else {
                candidates.add(candidate);
            }
            for (short i = 1; i < fieldNames.length; i++) {
                // all object in collection are of the same class
                String currentClassName = session.getClassNameFor(candidates.get(0));
                ClassDescription meta = session.getMetaRepository().getMetaData(currentClassName);
                int index = meta.getIndex(fieldNames[i]);

                ArrayList newCandidates = new ArrayList();
                for (int j = 0; j < candidates.size(); j++) {
                    newCandidates.add(session.readObjectByOid(candidates.get(j))[index]);
                }
                candidates.clear();
                if (Collection.class.isAssignableFrom(newCandidates.get(0).getClass())) {
                    for (int j = 0; j < newCandidates.size(); j++) {
                        candidates.addAll((Collection) newCandidates.get(i));
                    }
                } else {
                    candidates.addAll(newCandidates);
                }
            }
        }
        if (not) {
            return (!operator.execute(comparator, candidates, value.getValue()));
        } else {
            return operator.execute(comparator, candidates, value.getValue());
        }
    }

    public String getFieldName() {
        return fieldNames[0];
    }

    public String[] getFieldNames() {
        return fieldNames;
    }

    public String getClassName() {
        return className;
    }

    public boolean isResolved() {
        return isResolved;
    }

    public void setResolved(boolean resolved) {
        isResolved = resolved;
    }

    public WrapperSet getResolvedValues() {
        return resolvedValues;
    }

    public void clean() {
        resolvedValues.clear();
        isResolved = false;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("VCon").append(fieldNames[0]).append(operator).append(value.getValue());
        return sb.toString();
    }


    private String className;
    private String[] fieldNames;
    private QueryValue value;
    private Operator operator;
    private boolean not = false;

    protected WrapperSet resolvedValues;
    protected boolean isResolved;
}
