/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.query.execution;

import org.objectweb.jalisto.se.api.MetaRepository;
import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.api.query.IndexManager;
import org.objectweb.jalisto.se.query.result.QueryResultWrapper;
import org.objectweb.jalisto.se.query.result.WrapperSet;

import java.util.Iterator;
import java.util.List;
import java.util.Set;

public class ExecutionElement implements Comparable {

    public ExecutionElement(ExecutionTree tree, ExecutionElement father, int depth) {
        this.tree = tree;
        this.resolvedValues = new WrapperSet();
        this.isResolved = false;
        this.father = (ExecutionNode) father;
        this.depth = depth;
        this.toLeaf = -1;
    }

    public void setToLeaf(int toLeaf) {
        if (this.toLeaf < toLeaf) {
            this.toLeaf = toLeaf;
            if (father != null) {
                father.setToLeaf(toLeaf + 1);
            }
        }
    }

    public boolean onlyOneToResolve() {
        return false;
    }

    public boolean isAndNode() {
        return false;
    }

    public boolean isInOrBranch(IndexManager indexManager) {
        return true;
    }

    public ExecutionNode getFather() {
        return father;
    }

    public boolean isResolved() {
        return isResolved;
    }

    public void setResolved(boolean resolved) {
        isResolved = resolved;
    }

    public WrapperSet getResolvedValues() {
        return resolvedValues;
    }

    public int getToLeaf() {
        return toLeaf;
    }

    public int getDepth() {
        return depth;
    }

    // do not clean resolved values in the constraints
    public void cleanSubExecutionTree() {
        resolvedValues.clear();
        isResolved = false;
    }

    public void cleanAll() {
        resolvedValues.clear();
        isResolved = false;
    }

    public void defineMeta(MetaRepository repository) {
    }

    public int compareTo(Object o) {
        ExecutionElement candidate = (ExecutionElement) o;
        if (this.toLeaf < candidate.toLeaf) {
            return -1;
        } else if (candidate.toLeaf == this.toLeaf) {
            return 0;
        } else {
            return 1;
        }
    }

    /**
     * ******************************   RESOLVE   *******************************************
     * used to propagate back results of resolved nodes, without reading objects
     * return the father of the node if this node can be resolved, else return null
     */

    // used to propagate back results of resolved nodes, without reading objects
    // return the father of the node if this node can be resolved, else return null
    public ExecutionNode resolveBack() {
        if (isResolved()) {
            return father;
        }
        return null;
    }

    // used to resolve first nearest from root and nodes
    public void resolveBeforeAndNodeNearRoot(Session session, WrapperSet candidates, List toResolve) {
        resolveWithCandidates(session, candidates);
        toResolve.remove(this);
    }

    // used to resolve node, with a set of resolved value
    public WrapperSet resolveWithCandidates(Session session, WrapperSet candidates) {
        return new WrapperSet();
    }

    // resolve on a specific readed element
    public boolean resolveOnElement(Session session, QueryResultWrapper wrapper) {
        return false;
    }

    // resolve this leaf if field has an index
    public void resolveLeafOnIndex(Session session, IndexManager indexManager) {
    }

    /**
     * **************************   UTIL   **************************************************
     * s1 most important
     */

    // s1 most important
    public WrapperSet doIntersection(WrapperSet in, WrapperSet s1, WrapperSet s2) {
        if (s1.isEmpty()) {
            in.addAll(s2);
            return in;
        }
        if (s2.isEmpty()) {
            in.addAll(s1);
            return in;
        }
        Iterator i = s1.iterator();
        while (i.hasNext()) {
            QueryResultWrapper w = (QueryResultWrapper) i.next();
            QueryResultWrapper w2 = s2.get(w.getLogicalOid());
            if (w2 != null) {
                if (w.isReaded()) {
                    in.addToRead(w);
                } else {
                    in.add(w2);
                }
            }
        }
        return in;
    }

    // intersection done in s1
    public WrapperSet doIntersection(WrapperSet s1, WrapperSet s2) {
        if (s1.isEmpty()) {
            s1.addAll(s2);
            return s1;
        }
        if (s2.isEmpty()) {
            return s1;
        }
        Iterator i = s1.iterator();
        while (i.hasNext()) {
            QueryResultWrapper w = (QueryResultWrapper) i.next();
            QueryResultWrapper w2 = s2.get(w.getLogicalOid());
            if (w2 != null) {
                if ((!w.isReaded()) && w2.isReaded()) {
                    s1.remove(w);
                    s1.addToRead(w2);
                }
            }
        }
        return s1;
    }

    public Set readValues(Session session, Set set) {
        Iterator wrappers = set.iterator();
        session.currentTransaction().begin();
        while (wrappers.hasNext()) {
            QueryResultWrapper wrapper = (QueryResultWrapper) wrappers.next();
            if (wrapper.getValue() == null) {
                wrapper.setValue(session.readObjectByOid(wrapper.getLogicalOid()));
            }
        }
        session.currentTransaction().commit();
        return set;
    }


    private ExecutionNode father;
    private int depth;
    private int toLeaf;
    protected WrapperSet resolvedValues;
    protected boolean isResolved;
    protected ExecutionTree tree;
}
