/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.se.query.execution;

import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.api.Session;
import org.objectweb.jalisto.se.api.MetaRepository;
import org.objectweb.jalisto.se.api.query.IndexManager;
import org.objectweb.jalisto.se.api.query.Constraint;
import org.objectweb.jalisto.se.api.internal.SessionInternal;
import org.objectweb.jalisto.se.api.internal.InternalMetaRepository;
import org.objectweb.jalisto.se.query.constraint.*;
import org.objectweb.jalisto.se.query.result.QueryResultWrapper;
import org.objectweb.jalisto.se.query.result.WrapperSet;

import java.util.*;

public class ExecutionTree {

    public ExecutionTree(Constraint rootConstraint, MetaRepository repository, String className) {
        toResolveAndNodes = new ArrayList();

        readedValues = new HashMap();
        leafs = new HashSet();

        this.baseClassName = className;

        root = buildElement(null, rootConstraint, 0);
        toResolveAndNodesBackup = new ArrayList(toResolveAndNodes);
        root.defineMeta(repository);
    }

    public ExecutionElement buildElement(ExecutionElement father, Constraint constraint, int depth) {
        ExecutionElement element = null;
        if (constraint instanceof BinaryBooleanOperatorConstraint) {
            BinaryBooleanOperatorConstraint binConstraint = (BinaryBooleanOperatorConstraint) constraint;
            ExecutionNode node = new ExecutionNode(this, father, binConstraint, depth);
            node.setLeft(buildElement(node, binConstraint.getLeft(), depth + 1));
            node.setRight(buildElement(node, binConstraint.getRight(), depth + 1));
            if (binConstraint.isAndOperator()) {
                toResolveAndNodes.add(node);
            }
            element = node;
        } else if (constraint instanceof ValueConstraintImpl) {
            element = new ValueExecutionLeaf(this, father, (ValueConstraintImpl) constraint, depth);
            leafs.add(element);
        } else if (constraint instanceof SubQueryConstraintImpl) {
            element = new SubQueryExecutionLeaf(this, father, (SubQueryConstraintImpl) constraint, depth);
            leafs.add(element);
        }
        return element;
    }

    public Collection getToResolveAndNodes() {
        return toResolveAndNodes;
    }

    public Set getLeafs() {
        return leafs;
    }

    public Map getReadedValues() {
        return readedValues;
    }

    public void cleanLeaf() {
        root.cleanAll();
        readedValues.clear();
        toResolveAndNodes.clear();
        toResolveAndNodes.addAll(toResolveAndNodesBackup);
    }

    /**
     * *******************************    RESOLVE    ***************************************
     */

    public Set resolve(SessionInternal session, IndexManager indexManager) {
        if (manageIndexes(session, indexManager)) {
            resolveWithIndex(session);
        }
        if (root.isResolved()) {
            return root.getResolvedValues().getCopyInSet();
        }
        return resolveOnElements(session);
    }

    public Set resolveOnElements(SessionInternal session) {
        Iterator extent = session.getExtent(baseClassName).readFully().iterator();
        Set result = new HashSet();
        while (extent.hasNext()) {
            LogicalOid floid = (LogicalOid) extent.next();
            Object[] values = (Object[]) readedValues.get(floid);
            if (values == null) {
                values = session.readObjectByOid(floid, true);
                readedValues.put(floid, values);
            }
            QueryResultWrapper wrapper = new QueryResultWrapper(floid, values);
            if (root.resolveOnElement(session, wrapper)) {
                result.add(wrapper);
            }
        }
        return result;
    }

    public boolean resolveOnOneElement(SessionInternal session, QueryResultWrapper wrapper) {
        return root.resolveOnElement(session, wrapper);
    }

    private boolean manageIndexes(Session session, IndexManager indexManager) {
        boolean useIndexes = false;
        Set toResolveBack = new HashSet();
        Iterator leafIterator = leafs.iterator();
        while (leafIterator.hasNext()) {
            ExecutionElement leaf = (ExecutionElement) leafIterator.next();
            leaf.resolveLeafOnIndex(session, indexManager);
            if (leaf.isResolved()) {
                ExecutionElement resBack = leaf.resolveBack();
                if (resBack != null) {
                    toResolveBack.add(resBack);
                }
            }
        }

        Set toResolveBackNext = new HashSet();
        while (!toResolveBack.isEmpty()) {
            useIndexes = true;
            Iterator it = toResolveBack.iterator();
            while (it.hasNext()) {
                ExecutionElement element = (ExecutionElement) it.next();
                ExecutionNode node = element.resolveBack();
                if (node != null) {
                    toResolveBackNext.add(node);
                }
            }
            toResolveBack.clear();
            toResolveBack.addAll(toResolveBackNext);
            toResolveBackNext.clear();
        }

        return useIndexes;
    }

    private void resolveWithIndex(SessionInternal session) {
        if (root.isResolved()) {
            return;
        }

        LinkedList toResolve = new LinkedList();
        Iterator ands = toResolveAndNodes.iterator();
        while (ands.hasNext()) {
            ExecutionElement e = (ExecutionElement) ands.next();
            if (e.onlyOneToResolve()) {
                toResolve.add(e);
            }
        }
        Collections.sort(toResolve);

        while (!toResolve.isEmpty()) {
            ExecutionNode andNode = (ExecutionNode) toResolve.removeFirst();

            boolean before = false;
            while ((andNode.getFather() != null) &&
                   (andNode.getFather().isAndNode()) &&
                   (andNode.getFather().onlyOneToResolve())) {
                andNode = andNode.getFather();
                before = true;
            }

            if (before) {
                andNode.resolveBeforeAndNodeNearRoot(session, null, toResolve);
            } else {
                andNode.resolveWithCandidates(session, new WrapperSet());
            }

            ExecutionElement son = andNode;
            ExecutionElement father = andNode.resolveBack();
            while (father != null) {
                son = father;
                father = father.resolveBack();
            }

            if ((son == root) && root.isResolved) {
                return;
            }

            if (son.isAndNode() && son.onlyOneToResolve()) {
                toResolve.add(son);
                Collections.sort(toResolve);
            }
        }
    }

    public boolean detectOrBranch(IndexManager indexManager) {
        return root.isInOrBranch(indexManager);
    }


    private ArrayList toResolveAndNodes;
    private ArrayList toResolveAndNodesBackup;
    private Set leafs;
    private ExecutionElement root;
    private String baseClassName;
    private Map readedValues;
}
