/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.test.query;

import junit.framework.Test;
import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.api.query.Query;
import org.objectweb.jalisto.se.api.query.Constraint;
import org.objectweb.jalisto.se.exception.JalistoException;
import org.objectweb.jalisto.se.test.data.*;
import org.objectweb.jalisto.se.test.workbench.JalistoTestSuite;
import org.objectweb.jalisto.se.test.workbench.JalistoTimer;

import java.util.ArrayList;

public class AdvancedQueryTestCase extends JalistoQueryTestCase {
    public AdvancedQueryTestCase() {
    }

    public AdvancedQueryTestCase(String name) {
        super(name);
    }

    public static Test suite() {
        JalistoTestSuite suite = new JalistoTestSuite();
        AdvancedQueryTestCase tc = (AdvancedQueryTestCase) newTestCase(suite, new AdvancedQueryTestCase());

        tc.prepare();

        tc.defineInvalidIndex();

        tc.complexQuery("Without index", 50);

        tc.buildIndex(Author.class, "lastName");

        tc.complexQuery("With index on lastname of Author", 50);

        tc.deleteIndex(Author.class, "lastName");

        tc.finishTests();

        return suite;
    }

    /**
     * **************************************************************************************
     */

    public void prepare() {
        initSession();
        define(BookWithAuthor.getMetaDescription());
        define(BookWithFormat.getMetaDescription());
        define(Author.getMetaDescription());
        define(Library.getMetaDescription());
        define(Client.getMetaDescription());

        BookWithFormat.counter = -1;
        Library.counter = -1;
        Client.counter = -1;
        Author.counter = -1;

        cleanUp(Library.class);
        cleanUp(Client.class);
        cleanUp(Author.class);
        cleanUp(BookWithFormat.class);
        populateTestLibrary(25, 2, 10);
    }

    public void defineInvalidIndex() {
        try {
            ClassDescription meta = session.getMetaRepository().getMetaData(Library.class.getName());
            int fieldIndex = meta.getIndex("books");
            tx.begin();
            indexManager.buildIndexOnField(meta, fieldIndex);
            assertTrue("must raise error", false);
        } catch (JalistoException jalistoExc) {
        } finally {
            tx.commit();
        }
    }

    public void complexQuery(String message, int nbr) {
        String timerName = String.valueOf(nbr) + " executions " + message;
        Query query = getQuery();
        int resultSize = -1;
        JalistoTimer.timerStart(timerName);
        for (int i = 0; i < nbr; i++) {
            tx.begin();
            resultSize = query.execute().size();
            tx.commit();
        }
        JalistoTimer.timerStop(timerName, false);
        assertEquals("must be equals", 13, resultSize);
        JalistoTimer.summary(timerName);
    }

    private Query getQuery() {
        String authorLastName1 = Author.lastNames[0];
        String authorLastName2 = Author.lastNames[1];
        String authorFirstName = Author.firstNames[0];

        Query query = queryManager.query();
        query.constrain(Library.class);
        Constraint c1 = query.descend("books").descend("author").descend("lastName").constrain(authorLastName1).equal();
        Constraint c2 = query.descend("books").descend("author").descend("lastName").constrain(authorLastName2).equal();
        Constraint c3 = query.descend("books").descend("author").descend("firstName").constrain(authorFirstName).equal();
        (c1.or(c2)).and(c3);

        return query;
    }

    public void populateTestLibrary(int nbrLibrary, int nbrBook, int nbrClient) {
        tx.begin();
        for (int i = 0; i < nbrLibrary; i++) {
            Library library = Library.newLibrary();
            ArrayList books = library.getBooks();
            ArrayList clients = library.getClients();
            for (int j = 0; j < nbrBook; j++) {
                Object[] book = BookWithFormat.newBook().toArray();
                Author author = Author.newAuthor();
                book[3] = session.createObject(author.toArray(), Author.class);
                books.add(session.createObject(book, BookWithFormat.class));
            }
            for (int j = 0; j < nbrClient; j++) {
                Client client = Client.newClient();
                Object clientOid = session.createObject(client.toArray(), Client.class);
                clients.add(clientOid);
            }
            Author a = Author.newAuthor();
            Object[] author = a.toArray();
            library.setAuthorMostWanted(session.createObject(author, Author.class));
            session.createObject(library.toArray(), Library.class);
        }
        tx.commit();
    }


    /**
     * ************************* need to overwrite for suite() method ********************************************
     */

    public void initSession() {
        super.initSession(false);
    }

    public void finishTests() {
        super.finishTests();
    }

    public void define(ClassDescription classDescription) {
        super.define(classDescription);
    }

    public void cleanUp(Class aClass) {
        super.cleanUp(aClass);
    }

    public void buildIndex(Class aClass, String fieldName) {
        super.buildIndex(aClass, fieldName);
    }

    public void deleteIndex(Class aClass, String fieldName) {
        super.deleteIndex(aClass, fieldName);
    }

    public void printIndex(Class aClass, String fieldName) {
        super.printIndex(aClass, fieldName);
    }
}
