/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.test.query;

import junit.framework.Test;
import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.api.query.Query;
import org.objectweb.jalisto.se.api.query.Constraint;
import org.objectweb.jalisto.se.test.data.*;
import org.objectweb.jalisto.se.test.workbench.JalistoTestSuite;
import org.objectweb.jalisto.se.test.workbench.JalistoTimer;

import java.util.ArrayList;

public class CollectionQueryTestCase extends JalistoQueryTestCase {

    public CollectionQueryTestCase() {
    }

    public CollectionQueryTestCase(String name) {
        super(name);
    }

    public static Test suite() {
        JalistoTestSuite suite = new JalistoTestSuite();
        CollectionQueryTestCase tc = (CollectionQueryTestCase) newTestCase(suite, new CollectionQueryTestCase());

        tc.prepare();

        tc.executeQuery(0, "Without index", numberOfQueryExecute);
        tc.executeQuery(1, "Without index", numberOfQueryExecute);
        tc.executeQuery(2, "Without index", numberOfQueryExecute);

        tc.cleanUp(Library.class);

        tc.finishTests();

        return suite;
    }

    /**
     * **************************************************************************************
     */

    public void prepare() {
        initSession();
        define(BookWithAuthor.getMetaDescription());
        define(BookWithFormat.getMetaDescription());
        define(Author.getMetaDescription());
        define(Library.getMetaDescription());
        define(Client.getMetaDescription());

        BookWithFormat.counter = -1;
        Library.counter = -1;
        Client.counter = -1;
        Author.counter = -1;
        Format.counter = -1;

        cleanUp(Library.class);
        cleanUp(Client.class);
        cleanUp(Author.class);
        cleanUp(BookWithFormat.class);
        populateTestLibrary(10, 20, 10);
    }

    public void executeQuery(int q, String message, int nbr) {
        String timerName = String.valueOf(nbr) + " executions " + message;

        Query query;
        if (q == 0) {
            query = getQuery();
        } else if (q == 1) {
            query = getQueryWithSubQueryOnCollection();
        } else {
            query = getQueryWithSubQueryOnLink();
        }

        int resultSize = -1;

        JalistoTimer.timerStart(timerName);
        for (int i = 0; i < nbr; i++) {
            tx.begin();
            resultSize = query.execute().size();
            tx.commit();
        }
        JalistoTimer.timerStop(timerName, false);
        if (q == 0) {
            assertEquals("should be equal", 8, resultSize);
        } else if (q == 1) {
            assertEquals("should be equal", 3, resultSize);
        } else {
            assertEquals("should be equal", 4, resultSize);
        }


        JalistoTimer.summary(timerName);
        JalistoTimer.clean(timerName);
    }

    private Query getQuery() {
        String clientLastName = Client.lastNames[0];

        Query query = queryManager.query();
        query.constrain(Library.class);

        Constraint c1 = query.descend("books").constrain(bookToRetrieve).contains();
        Constraint c2 = query.descend("clients").descend("lastName").constrain(clientLastName).equal();
        c1.or(c2);

        return query;
    }

    private Query getQueryWithSubQueryOnCollection() {
        String clientLastName = Client.lastNames[0];
        String clientFirstName = Client.firstNames[0];

        Query query = queryManager.query();
        query.constrain(Library.class);

        Query clientQuery = queryManager.query();
        clientQuery.constrain(Client.class);

        Constraint c1 = query.descend("books").constrain(bookToRetrieve).contains();
        Constraint c2 = query.descend("clients").constrain(clientQuery);
        c1.or(c2);

        Constraint c3 = clientQuery.descend("lastName").constrain(clientLastName).equal();
        Constraint c4 = clientQuery.descend("firstName").constrain(clientFirstName).equal();
        c3.and(c4);

        return query;
    }

    private Query getQueryWithSubQueryOnLink() {
        String authorLastName = Author.lastNames[0];
        String authorFirstName = Author.firstNames[0];

        Query query = queryManager.query();
        query.constrain(Library.class);

        Query authorQuery = queryManager.query();
        authorQuery.constrain(Author.class);

        Constraint c1 = query.descend("books").constrain(bookToRetrieve).contains();
        Constraint c2 = query.descend("authorMostWanted").constrain(authorQuery);
        c1.or(c2);

        Constraint c3 = authorQuery.descend("lastName").constrain(authorLastName).equal();
        Constraint c4 = authorQuery.descend("firstName").constrain(authorFirstName).equal();
        c3.and(c4);

        return query;
    }

    public void populateTestLibrary(int nbrLibrary, int nbrBook, int nbrClient) {
        tx.begin();
        for (int i = 0; i < nbrLibrary; i++) {
            Library library = Library.newLibrary();
            ArrayList books = library.getBooks();
            ArrayList clients = library.getClients();
            for (int j = 0; j < nbrBook; j++) {
                BookWithFormat book = BookWithFormat.newBook();
                books.add(book);
                if ((i == (nbrLibrary / 2)) && (j == (nbrBook / 2))) {
                    bookToRetrieve = book;
                }
            }
            for (int j = 0; j < nbrClient; j++) {
                Client client = Client.newClient();
                Object clientOid = session.createObject(client.toArray(), Client.class);
                clients.add(clientOid);
            }
            Author a = Author.newAuthor();
            Object[] author = a.toArray();
            library.setAuthorMostWanted(session.createObject(author, Author.class));
            session.createObject(library.toArray(), Library.class);
        }
        tx.commit();
    }


    private BookWithFormat bookToRetrieve = null;

    /**
     * ************************* need to overwrite for suite() method ********************************************
     */

    public void initSession() {
        super.initSession(false);
    }

    public void define(ClassDescription classDescription) {
        super.define(classDescription);
    }

    public void cleanUp(Class aClass) {
        super.cleanUp(aClass);
    }

    public void finishTests() {
        super.finishTests();
    }
}
