/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.test.query;

import junit.framework.Test;
import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.api.query.Query;
import org.objectweb.jalisto.se.api.query.Constraint;
import org.objectweb.jalisto.se.test.data.BookWithAuthor;
import org.objectweb.jalisto.se.test.workbench.JalistoTestSuite;
import org.objectweb.jalisto.se.test.workbench.JalistoTimer;

public class QueryTestCase extends JalistoQueryTestCase {

    public QueryTestCase() {
    }

    public QueryTestCase(String name) {
        super(name);
    }

    public static Test suite() {
        JalistoTestSuite suite = new JalistoTestSuite();
        QueryTestCase tc = (QueryTestCase) newTestCase(suite, new QueryTestCase());

        tc.prepare();

        tc.executeQueryTests("Without index", numberOfQueryExecute);

        tc.buildIndex(BookWithAuthor.class, "title");
        tc.buildIndex(BookWithAuthor.class, "price");

        tc.executeQueryTests("With index", numberOfQueryExecute);

        tc.deleteIndex(BookWithAuthor.class, "title");
        tc.deleteIndex(BookWithAuthor.class, "price");

        tc.finishTests();

        return suite;
    }

    /**
     * **************************************************************************************
     */

    public void prepare() {
        initSession();
        BookWithAuthor.counter = -1;
        define(BookWithAuthor.getMetaDescription());
        deleteIndex(BookWithAuthor.class, "title");
        deleteIndex(BookWithAuthor.class, "price");
        cleanUp(BookWithAuthor.class);
        populateBookWithAuthor(1000);
    }

    public void executeQueryTests(String message, int nbr) {
        executeQuery(getQuery(), message, nbr, 16);
        executeQuery(getQueryWithOrBranch(), message + " or branch", nbr, 700);

//        evolutionExecutionTime(getQuery(), message, 10, nbr);
    }

    public void executeQuery(Query query, String message, int nbr, int expected) {
        String timerName = String.valueOf(nbr) + " executions " + message;

        int resultSize = -1;

        JalistoTimer.timerStart(timerName);
        for (int i = 0; i < nbr; i++) {
            tx.begin();
            resultSize = query.execute().size();
            tx.commit();
        }
        JalistoTimer.timerStop(timerName, false);
        assertEquals("Should be equals", expected, resultSize);

        JalistoTimer.summary(timerName);
        JalistoTimer.clean(timerName);
    }

    public void evolutionExecutionTime(Query query, String message, int nbrExec, int nbrTest) {
        String timerName = "Evolution of the " + nbrExec + " first execution time (average on " + nbrTest + " tests) - " + message;

        for (int j = 0; j < nbrTest; j++) {
//            Query query = getQuery();
            for (int i = 0; i < nbrExec; i++) {
                JalistoTimer.timerStart(timerName + " - " + i);
                tx.begin();
                query.execute();
                tx.commit();
                JalistoTimer.timerStop(timerName + " - " + i, false);
            }
        }

        JalistoTimer.summary();
        JalistoTimer.deleteAllTimers();
    }

    private Query getQuery() {
        String title1 = BookWithAuthor.titles[0];
        String title2 = BookWithAuthor.titles[1];
        String author1 = BookWithAuthor.authors[0];
        String author2 = BookWithAuthor.authors[1];
        Integer page1 = new Integer(500);
        Integer price1 = new Integer(9);
        Integer price2 = new Integer(26);


        Query query = queryManager.query();
        query.constrain(BookWithAuthor.class);
        Constraint c1 = query.descend("title").constrain(title1).equal();
        Constraint c2 = query.descend("title").constrain(title2).equal();
        Constraint c3 = query.descend("author").constrain(author1).equal();
        Constraint c4 = query.descend("author").constrain(author2).equal();
        Constraint c5 = query.descend("price").constrain(price1).smaller();
        Constraint c6 = query.descend("price").constrain(price2).greater();
        Constraint c7 = query.descend("pages").constrain(page1).greater();
        ((c1.and(c3)).or(c2.and(c4)))
                .and(
                        (c5.or(c6)).and(c7));

        return query;
    }

    private Query getQueryWithOrBranch() {
        String title1 = BookWithAuthor.titles[0];
        String title2 = BookWithAuthor.titles[1];
        String author1 = BookWithAuthor.authors[0];
        String author2 = BookWithAuthor.authors[1];
        Integer page1 = new Integer(500);
        Integer price1 = new Integer(9);
        Integer price2 = new Integer(26);


        Query query = queryManager.query();
        query.constrain(BookWithAuthor.class);
        Constraint c1 = query.descend("title").constrain(title1).equal();
        Constraint c2 = query.descend("title").constrain(title2).equal();
        Constraint c3 = query.descend("author").constrain(author1).equal();
        Constraint c4 = query.descend("author").constrain(author2).equal();
        Constraint c5 = query.descend("price").constrain(price1).smaller();
        Constraint c6 = query.descend("price").constrain(price2).greater();
        Constraint c7 = query.descend("pages").constrain(page1).greater();
        ((c1.and(c3)).or(c2.and(c4)))
                .or(
                        (c5.or(c6)).or(c7));

        return query;
    }


    /**
     * ************************* need to overwrite for suite() method ********************************************
     */

    public void initSession() {
        super.initSession(false);
    }

    public void define(ClassDescription classDescription) {
        super.define(classDescription);
    }

    public void cleanUp(Class aClass) {
        super.cleanUp(aClass);
    }

    public void populateBookWithAuthor(int nbr) {
        super.populateBookWithAuthor(nbr);
    }

    public void buildIndex(Class aClass, String fieldName) {
        super.buildIndex(aClass, fieldName);
    }

    public void deleteIndex(Class aClass, String fieldName) {
        super.deleteIndex(aClass, fieldName);
    }

    public void printIndex(Class aClass, String fieldName) {
        super.printIndex(aClass, fieldName);
    }

    public void finishTests() {
        super.finishTests();
    }
}
