/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.test.query.bench;

import org.objectweb.jalisto.test.query.JalistoQueryTestCase;
import junit.framework.Test;
import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.api.query.Index;
import org.objectweb.jalisto.se.api.query.Query;
import org.objectweb.jalisto.se.api.query.ObjectSet;
import org.objectweb.jalisto.se.exception.SchemaException;
import org.objectweb.jalisto.se.impl.InFileAddress;
import org.objectweb.jalisto.se.impl.LogicalOid;
import org.objectweb.jalisto.se.query.IndexManagerImpl;
import org.objectweb.jalisto.se.query.btree.BTree;
import org.objectweb.jalisto.se.query.index.IndexBTreeImpl;
import org.objectweb.jalisto.se.query.result.QueryResultWrapper;
import org.objectweb.jalisto.se.test.data.BookWithAuthor;
import org.objectweb.jalisto.se.test.data.DVD;
import org.objectweb.jalisto.se.test.workbench.JalistoTestSuite;

import java.util.Iterator;

public class BTreeTestCase extends JalistoQueryTestCase {
    public BTreeTestCase() {
    }

    public BTreeTestCase(String name) {
        super(name);
    }

    public static Test suite() {
        JalistoTestSuite suite = new JalistoTestSuite();
        BTreeTestCase tc = (BTreeTestCase) newTestCase(suite, new BTreeTestCase());

        tc.initSession();
        tc.define(BookWithAuthor.getMetaDescription());
        tc.define(DVD.getMetaDescription());

        tc.cleanUp(BookWithAuthor.class);
        tc.cleanUp(DVD.class);
        tc.populateBookWithAuthor(50);
        tc.populateTestDVD(50);

        tc.buildIndex(BookWithAuthor.class, "title");
        tc.printIndex(BookWithAuthor.class, "title");

        tc.buildIndex(DVD.class, "title");
        tc.printIndex(DVD.class, "title");

//        tc.removeOidsOnBook();
//        tc.printIndex(Book.class);
//
//        tc.deleteIndex(Book.class);
//        tc.cleanUp(Book.class);
//        tc.buildIndex(Book.class);
//        tc.populateTest(50, 50);
//        tc.printIndex(Book.class);
//        tc.deleteIndex(Book.class);
//        tc.printIndex(Book.class);

        tc.deleteClasses();

        tc.define(BookWithAuthor.getMetaDescription());
        tc.define(DVD.getMetaDescription());
        tc.buildIndex(BookWithAuthor.class, "title");
        tc.buildIndex(DVD.class, "title");
        tc.deleteClasses();

        return suite;
    }

    /**
     * **************************************************************************************
     */

    public void populateTestDVD(int nbr) {
        tx.begin();
        for (int i = 0; i < nbr; i++) {
            session.createObject(DVD.newDVD().toArray(), DVD.class);
        }
        tx.commit();
    }

    public void deleteClasses() {
        session.getMetaRepository().removeClass(session, DVD.class.getName());
        session.getMetaRepository().removeClass(session, BookWithAuthor.class.getName());
        session.reorganize();
        tx.begin();
        try {
            session.createObject(BookWithAuthor.newBook().toArray(), BookWithAuthor.class);
        } catch (SchemaException jalistoSchemaExce) {
            assertEquals("Should be equals",
                         "Class org.objectweb.jalisto.se.test.data.Book is not define in base...",
                         jalistoSchemaExce.getMessage());
        }
        tx.commit();
        tx.begin();
        try {
            session.createObject(DVD.newDVD().toArray(), DVD.class);
        } catch (SchemaException jalistoSchemaExce) {
            assertEquals("Should be equals",
                         "Class org.objectweb.jalisto.se.test.data.DVD is not define in base...",
                         jalistoSchemaExce.getMessage());
        }
        tx.commit();

        tx.begin();
        try {
            getIndex(BookWithAuthor.class, "title");
        } catch (SchemaException jalistoSchemaExce) {
            assertEquals("Should be equals",
                         "Class org.objectweb.jalisto.se.test.data.Book is not define in base...",
                         jalistoSchemaExce.getMessage());
        }
        tx.commit();

        tx.begin();
        try {
            getIndex(DVD.class, "title");
        } catch (SchemaException jalistoSchemaExce) {
            assertEquals("Should be equals",
                         "Class org.objectweb.jalisto.se.test.data.DVD is not define in base...",
                         jalistoSchemaExce.getMessage());
        }
        tx.commit();
    }

    private Index getIndex(Class aClass, String fieldName) {
        ClassDescription meta = session.getMetaRepository().getMetaData(aClass.getName());
        int fieldIndex = meta.getIndex(fieldName);
        return indexManager.getIndex(meta, fieldIndex);
    }

    public void insertInBtree(Class aClass) {
        ClassDescription meta = session.getMetaRepository().getMetaData(aClass.getName());
        int titleFieldIndex = meta.getIndex("title");

        tx.begin();
        InFileAddress ifa = new InFileAddress("bTreeTest");
        IndexBTreeImpl bTreeIndex = new IndexBTreeImpl(ifa, titleFieldIndex, meta,
                                                             (IndexManagerImpl) indexManager);
        BTree tree = bTreeIndex.getTree();
        tx.commit();

        tx.begin();
        Iterator extent = session.getExtent(aClass).readFully().iterator();
        while (extent.hasNext()) {
            LogicalOid oid = (LogicalOid) extent.next();
            Object[] array = session.readObjectByOid(oid);
            Object value = array[titleFieldIndex];
            tree.addOid(value, oid);
        }
        tx.commit();

        System.out.println(tree);
    }

    public void removeOidsOnBook() {
        ClassDescription meta = session.getMetaRepository().getMetaData(BookWithAuthor.class.getName());
        int titleFieldIndex = meta.getIndex("title");
        tx.begin();
        Index index = indexManager.getIndex(meta, titleFieldIndex);
        tx.commit();

        deleteTitleOnBook("Death Off Stage", index, titleFieldIndex);
        deleteTitleOnBook("Fingersmith", index, titleFieldIndex);
        deleteTitleOnBook("Bittersweet", index, titleFieldIndex);
        deleteTitleOnBook("A Cold Case of Murder", index, titleFieldIndex);
        deleteTitleOnBook("A Fountain Filled with Blood", index, titleFieldIndex);
        deleteTitleOnBook("Innocent Hearts", index, titleFieldIndex);
        deleteTitleOnBook("Metes and Bounds", index, titleFieldIndex);
        deleteTitleOnBook("The Jester", index, titleFieldIndex);
        deleteTitleOnBook("At the Edge", index, titleFieldIndex);
        deleteTitleOnBook("Tipping the Velvet", index, titleFieldIndex);
    }

    private void deleteTitleOnBook(String title, Index index, int titleIndex) {
        System.out.println("\n!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
        System.out.println("delete elements one by one for title : " + title);
        Query query = queryManager.query();
        query.constrain(BookWithAuthor.class);
        query.descend("title").constrain(title).equal();

        tx.begin();
        ObjectSet result = query.execute();
        tx.commit();

        tx.begin();
        while (result.hasNext()) {
            LogicalOid floid = ((QueryResultWrapper) result.next()).getLogicalOid();
            Object[] book = session.readObjectByOid(floid);
            index.remove(book[titleIndex], floid);
        }
        tx.commit();

        System.out.println("index : " + index.toFullString());
    }

    /**
     * ********************************************************************************
     */

    public void initSession() {
        super.initSession(false);
    }

    public void define(ClassDescription classDescription) {
        super.define(classDescription);
    }

    public void cleanUp(Class aClass) {
        super.cleanUp(aClass);
    }

    public void populateBookWithAuthor(int nbr) {
        super.populateBookWithAuthor(nbr);
    }

    public void buildIndex(Class aClass, String fieldName) {
        super.buildIndex(aClass, fieldName);
    }

    public void deleteIndex(Class aClass, String fieldName) {
        super.deleteIndex(aClass, fieldName);
    }

    public void printIndex(Class aClass, String fieldName) {
        super.printIndex(aClass, fieldName);
    }


}
