/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.test.query.bench;

import junit.framework.Test;
import org.objectweb.jalisto.se.api.Transaction;
import org.objectweb.jalisto.se.api.internal.SessionInternal;
import org.objectweb.jalisto.se.JalistoFactory;
import org.objectweb.jalisto.se.test.data.BookWithAuthor;
import org.objectweb.jalisto.se.test.workbench.JalistoTestCase;
import org.objectweb.jalisto.se.test.workbench.JalistoTestSuite;
import org.objectweb.jalisto.se.test.workbench.JalistoTimer;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Random;

public class BenchMonoMultiQuery extends JalistoTestCase {
    ArrayList oids = null;

    public BenchMonoMultiQuery() {
    }

    public BenchMonoMultiQuery(String name) {
        super(name);
    }

    public static Test suite() {
        JalistoTestSuite suite = new JalistoTestSuite();
        BenchMonoMultiQuery tc = (BenchMonoMultiQuery) newTestCase(suite, new BenchMonoMultiQuery());

        tc.stabilityTest(true, "mono", 5000, 500, 2000);

//        tc.stabilityTest(false, "multi", 5000, 200, 200);

        return suite;
    }

    /**
     * **************************************************************************************
     */

    public void stabilityTest(boolean isMono, String message, int nbrBook, int nbrAction, int nbrExecution) {
        SessionInternal workingSession = null;

        if (isMono) {
            workingSession = (SessionInternal) JalistoFactory.getSession("jalisto-query.properties");
            System.out.println("is mono : " + workingSession.getProperties().isMonoImplementation());
        } else {
            workingSession = (SessionInternal) JalistoFactory.getSession("jalisto-query-multi.properties");
            System.out.println("is mono : " + workingSession.getProperties().isMonoImplementation());
        }

        Transaction workingTransaction = workingSession.currentTransaction();
        workingSession.defineClass(BookWithAuthor.getMetaDescription());

        oids = new ArrayList();

        workingTransaction.begin();
        for (int i = 0; i < nbrBook; i++) {
            oids.add(workingSession.createObject(BookWithAuthor.newBook().toArray(), BookWithAuthor.class));
        }
        workingTransaction.commit();

        Random random = new Random();
        String createName = message + " : create time for " + nbrAction + " objects";
        String readName = message + " : read time for " + nbrAction + " objects";
        String updateName = message + " : update time for " + nbrAction + " objects";
        String deleteName = message + " : delete time for " + nbrAction + " objects";
        String extentName = message + " : get one extent for " + nbrAction + " objects";
        JalistoTimer.createTimer(createName);
        JalistoTimer.createTimer(readName);
        JalistoTimer.createTimer(updateName);
        JalistoTimer.createTimer(deleteName);
        JalistoTimer.createTimer(extentName);

        for (int c = 0; c < nbrExecution; c++) {
            if ((c != 0) && ((c % 50) == 0)) {
                workingSession.reorganize();
            }

//            Timer.timerStart(createName);
//            workingTransaction.begin();
//            for (int i = 0; i < nbrAction; i++) {
//                Object[] bookInArray = Book.newBook().toArray();
//                Object oid = workingSession.createObject(bookInArray, Book.class);
//                oids.add(oid);
//            }
//            workingTransaction.commit();
//            Timer.timerStop(createName, false);

            JalistoTimer.timerStart(readName);
            workingTransaction.begin();
            for (int i = 0; i < nbrAction; i++) {
                int index = random.nextInt(oids.size());
                Object oid = oids.get(index);
                workingSession.readObjectByOid(oid);
            }
            workingTransaction.commit();
            JalistoTimer.timerStop(readName, false);

//            Timer.timerStart(deleteName);
//            workingTransaction.begin();
//            for (int i = 0; i < nbrAction; i++) {
//                int index = random.nextInt(oids.size());
//                Object oid = oids.get(index);
//                workingSession.deleteObjectByOid(oid);
//                oids.remove(index);
//            }
//            workingTransaction.commit();
//            Timer.timerStop(deleteName, false);

//            Timer.timerStart(extentName);
//            workingTransaction.begin();
//            Iterator it = workingSession.getExtent(Book.class).iterator();
//            while (it.hasNext()) {
//                Timer.timerStart("pouet");
//                workingSession.readObjectByOid(it.next());
//                Timer.timerStop("pouet", false);
//            }
//            workingTransaction.commit();
//            Timer.timerStop(extentName, false);
        }

        workingTransaction.begin();
        Iterator extent = workingSession.getExtent(BookWithAuthor.class).readFully().iterator();
        while (extent.hasNext()) {
            workingSession.deleteObjectByOid(extent.next());
        }
        workingTransaction.commit();
        workingSession.reorganize();

        JalistoTimer.summary(createName);
        JalistoTimer.summary(readName);
        JalistoTimer.summary(updateName);
        JalistoTimer.summary(deleteName);
        JalistoTimer.summary(extentName);
        JalistoTimer.summary("pouet");
    }

}
