/*
 * Jalisto - JAva LIght STOrage
 * Copyright (C) 2000-2005 Xcalia http://www.xcalia.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Xcalia
 * 71, rue Desnouettes
 * 75014 Paris - France
 * http://www.xcalia.com
 */
package org.objectweb.jalisto.test.query.bench;

import org.objectweb.jalisto.test.query.JalistoQueryTestCase;
import junit.framework.Test;
import org.objectweb.jalisto.se.api.ClassDescription;
import org.objectweb.jalisto.se.test.data.BookWithAuthor;
import org.objectweb.jalisto.se.test.workbench.JalistoTestSuite;
import org.objectweb.jalisto.se.test.workbench.JalistoTimer;

import java.util.ArrayList;
import java.util.Random;

public class IndexCostTestCase extends JalistoQueryTestCase {
    static int call = 0;

    public IndexCostTestCase() {
    }

    public IndexCostTestCase(String name) {
        super(name);
    }

    public static Test suite() {
        JalistoTestSuite suite = new JalistoTestSuite();
        IndexCostTestCase tc = (IndexCostTestCase) newTestCase(suite, new IndexCostTestCase());

        tc.initSession();
        tc.define(BookWithAuthor.getMetaDescription());

        tc.deleteIndex(BookWithAuthor.class, "title");
        tc.cleanUp(BookWithAuthor.class);

        tc.populateBookWithAuthor(5000);
        tc.stabilityTest(200, 201);

        tc.buildIndex(BookWithAuthor.class, "title");
        tc.stabilityTest(200, 201);

        tc.printIndex(BookWithAuthor.class, "title");

        return suite;
    }

    /**
     * **************************************************************************************
     */

    public void stabilityTest(int nbrCreate, int nbrIt) {
        ArrayList oids = new ArrayList();
        Random random = new Random();
        String timerName = " time for call " + (call++);
        String createName = "create" + timerName;
        String deleteName = "delete" + timerName;
        JalistoTimer.createTimer(createName);
        JalistoTimer.createTimer(deleteName);

        for (int c = 0; c < nbrIt; c++) {
            if ((c != 0) && ((c % 100) == 0)) {
                session.reorganize();
                JalistoTimer.summary();
                JalistoTimer.clean(createName);
                JalistoTimer.clean(deleteName);
            }

            JalistoTimer.timerStart(createName);
            tx.begin();
            for (int i = 0; i < nbrCreate; i++) {
                Object[] bookInArray = BookWithAuthor.newBook().toArray();
                Object oid = session.createObject(bookInArray, BookWithAuthor.class);
                oids.add(oid);
            }
            tx.commit();
            JalistoTimer.timerStop(createName, false);

            JalistoTimer.timerStart(deleteName);
            tx.begin();
            for (int i = 0; i < nbrCreate; i++) {
                int index = random.nextInt(oids.size());
                Object oid = oids.get(index);
                session.deleteObjectByOid(oid);
                oids.remove(index);
            }
            tx.commit();
            JalistoTimer.timerStop(deleteName, false);
        }

        JalistoTimer.removeTimer(createName);
        JalistoTimer.removeTimer(deleteName);
    }


    /**
     * ********************************************************************************
     */

    public void initSession() {
        super.initSession(false);
    }

    public void define(ClassDescription classDescription) {
        super.define(classDescription);
    }

    public void cleanUp(Class aClass) {
        super.cleanUp(aClass);
    }

    public void populateBookWithAuthor(int nbr) {
        super.populateBookWithAuthor(nbr);
    }

    public void buildIndex(Class aClass, String fieldName) {
        super.buildIndex(aClass, fieldName);
    }

    public void deleteIndex(Class aClass, String fieldName) {
        super.deleteIndex(aClass, fieldName);
    }

    public void printIndex(Class aClass, String fieldName) {
        super.printIndex(aClass, fieldName);
    }
}
